#!/bin/bash

# -----------------------------------------------------------------------
# Copyright GPL v3 - Paramvir Likhari 2012/2017
# -----------------------------------------------------------------------


# This defines the output of set -x debugging xtrace
# export PS4='+ (${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
# wxdebug='Y'
if [[ -e '/tmp/wxdebug' || $wxdebug == 'Y' ]]; then
    rm -rf '/tmp/wxdebug'
    export PS4='+ (${LINENO}): ${FUNCNAME[0]:+${FUNCNAME[0]}(): }'
    # export P4=$(read time junk < /proc/$$/schedstat; echo "@@@ $time @@@ " )
    exec 4>>'/tmp/wxdebug'
    BASH_XTRACEFD=4
    set -x
fi
# set -e
msgout() { printf "\n%b\n" "$1" >&2; }

# set -x

version=7_build_333

export version

cwxpath="$( cd "$(dirname "$0")" && pwd )"

Prog_Name=$0

# setting config paths
config_dir_path="$HOME/.config/conkywx"
conkywx_internal_dir="$config_dir_path/conkywx_internal"
system_config_file="$config_dir_path/system.conf"


derrmsg() {
    dashline='\n============================================================\n'

    if [[ $1 == "info" ]]; then
        if [[ $wxfupdate ]]; then msgout "${dashline} ${2}" ; fi

    elif [[ ! -s "$1" ]]; then
        msgout "${dashline} Error: :( \nUnable to download $1 ${dashline} "

    elif [[ $2 && $2 == "history" ]]; then
        msgout "${dashline} Error: :( \nNo daily or hourly history data available in \n$1 \nTrend graphs will show run time as data is collected.${dashline} "
    fi

}

if [ ! -d "$config_dir_path" ]; then
    mkdir "$config_dir_path"
fi

# handle system config file here
if [[ -s "$system_config_file" ]]; then
    . "$system_config_file"
elif [[ -s "$cwxpath/examples/system.conf" ]]; then
    cp "$cwxpath/examples/system.conf" "$system_config_file" 2> /dev/null
    . "$system_config_file"
else
    msgout "system.conf NOT found - Reinstall conkywx."
    exit 1
fi

local_apps="$HOME/.local/share/applications"

wxloc=$(grep -x 'wxloc=.*' "$system_config_file")
if [[ ${wxloc} == '' ]]; then
    echo "wxloc=\"$cwxpath\"" >> "$system_config_file"
elif [[ ${wxloc} != "wxloc=\"$cwxpath\"" ]]; then
    sed -i 's`'"${wxloc}"'`'"wxloc=\"$cwxpath\""'`' "$system_config_file"
fi

### print all variables to file - part 1 of 2 messages file
# ( set -o posix ; set ) > /tmp/var.org

language_for_msgs=${language_for_msgs:-"english"}
lang_path="${cwxpath}/lang"
language_for_msgs=${language_for_msgs,,}
msgs_file="${lang_path}/${language_for_msgs}.msgs"

if [[ -s "$msgs_file" ]]; then
    . "$msgs_file"
else
    if [[ -s "${lang_path}/english.msgs" ]]; then
        . "${lang_path}/english.msgs"
        msgout "$no_lang_msgs"
    else
        msgout "$no_msg_file"
        exit 1
    fi
fi

if [[ $? -eq 1 ]]; then
    msgout "Error :-( \nThere is an error in the conkywx messages file:\nconkywx stopped!\n"
    exit 1
fi


### print all variables to file - part 2 of 2 messages file
# ( set -o posix ; set ) > /tmp/var.tmp

# ### find difference and get local variables for messages file only
# # awk 'NR==FNR{a[$0]=1;next}!a[$0]' /tmp/var.org /tmp/var.tmp > /tmp/wxMessages.msg
# grep -Fxvf /tmp/var.org /tmp/var.tmp > /tmp/wxMessages.msg

if [[ ! $( fc-list | grep -i conky ) ]]; then
    msgout "$no_fonts"
    exit 1
fi

# -----------------------------------------------------------------------
# show pop-up notification
# -----------------------------------------------------------------------
show_noti() {

    if [[ $show_notification -eq 1 || $2 ]]; then
        if [[ $2 ]]; then
            show_duration=50000
        else
            show_duration=5000
        fi

        if [[ $(type -p notify-send) ]]; then
            notify-send "$notimsg" \
            "$1" \
            -i "$cwxpath/images/wxicons-2/clear.png" \
            -t $show_duration
        fi
    fi
}

set_config() {

    # this dir holds program internal files which need to be persistent
    if [[ ! -d "$conkywx_internal_dir" ]]; then
        mkdir "$conkywx_internal_dir"
    fi

    if [[ ! -d "$conkywx_internal_dir/trends" ]]; then
        mkdir "$conkywx_internal_dir/trends"
    fi

    if [[ ! -s "$conkywx_internal_dir/READ_ME" ]]; then
        printf "%b\n" "$readme_msg" > "$conkywx_internal_dir/READ_ME"
    fi

    conf1="${config_file##*/}${copytmpR}"

    if [[ $clopt1 == "-d" ]]; then
        temppath="/tmp/wx-${USER}-$conf1-d-pChA_r_27u0-m39"
    else
        temppath="/tmp/wx-${USER}-$conf1-pChA_r_27u0-m39"
    fi

    if [[ ! -d "$temppath" ]]; then
        mkdir "$temppath"
    fi

    if [[ -s "$config_file" ]]; then
        . "$config_file"
    else
        msgout "$no_config_file"
        exit 1
    fi

    if [[ $? -eq 1 ]]; then
        msgout "Error :-( \nThere is an error in the conkywx config file:\n$config_file \nconkywx stopped!\n"
        exit 1
    fi

    case "$config_file" in
    .*)
        local myconf=${config_file#?}
        config_file="${cwxpath}${myconf}"
        ;;
    esac

}

fixurl() {
    inUrl="$1"
    inUrl=( ${inUrl//\// } )
    case $1 in
    *msn*)
        inUrl[2]='en-us'
        inUrl[3]='weather'
        inUrl[4]='today'
        ;;
    *accu*)
        inUrl[2]='en'
        inUrl[6]='current-weather'
        if [[ "$2" -eq 1 ]]; then
            inUrl[6]='daily-weather-forecast'
        elif [[ "$2" -eq 2 ]]; then
            inUrl[6]='weather-warnings'
        fi
        ;;
    esac
    inUrl="${inUrl[@]}"
    inUrl="${inUrl// /\/}"
    inUrl="${inUrl/http:\//http:\/\/}"
    inUrl="${inUrl/https:\//https:\/\/}"
    inUrl="${inUrl/\?*/}"
    echo "${inUrl}"
}

cleanNumber() {
    echo "$(echo "$1" | tr -cd /0-9.\\-//)"
}

set_variables () {

    set_config

    # -----------------------------------------------------------------------
    # Data constants
    # -----------------------------------------------------------------------
    tides="$temppath/tides"
    seismic="$temppath/seismic"
    seismic_dataA="$temppath/seismic_dataA"
    seismic_dataB="$temppath/seismic_dataB"
    wwx_image="$temppath/wwx_image.jpg"
    gisi_image_sur="$temppath/gisi_image_sur.jpg"
    gisi_image_sat="$temppath/gisi_image_sat.jpg"
    gwsi_image="$temppath/gwsi_image"
    gsi_image="$temppath/gsi_image.gif"
    die_error="$temppath/die_error"
    dl_currdata="$temppath/dl_currdata"

    orghistin="$temppath/orghistin"
    orghistwu="$temppath/orghistwu"
    histcall="$temppath/histcall"

    currdatax="$temppath/wxcurrdatax.xml"
    currdata="$temppath/wxcurrdata.html"
    dl_currdata_2="$temppath/dl_currdata_2"
    currdata_2="$temppath/wxcurrdata_2.xml"
    imp_cookies="$temppath/wxcookie"
    fct_data1="$temppath/fct_data1.html"
    fct_data="$temppath/fct_data.html"
    old_data="$temppath/old_data"

    fct_impcookies="$temppath/fct_impcookies"
    wund_debug_log="$temppath/wxwund_debug.log"
    conkywx_debug="$temppath/conkywx_debug.tar.bz2"

    data_check="$temppath/wxdata_check"

    uvdata="$conkywx_internal_dir/uvdata_wx-${USER}-$conf1-pChA_r_27u0-m39"
    forecast_trends="$conkywx_internal_dir/trends/forecast_trends-$conf1-${USER}"

    # -----------------------------------------------------------------------
    # Data default settings
    # -----------------------------------------------------------------------
    # If these variables are NOT defined in config file - then default values are used
    minimum_fetch_time=${minimum_fetch_time:-1750}
    enable_new_version_check=${enable_new_version_check:-0}

    website='http://www.wunderground.com'
    wund_url='http://www.wunderground.com/global/stations/'
    xml_url='http://api.wunderground.com/weatherstation/WXCurrentObXML.asp?ID='

    if [[ ! $wxdevelopment ]]; then
        wxdevelopment=
    elif [[ $wxdevelopment && ! -s $currdata ]]; then
        msgout "Data file does not exist.\nCannot use wxdevelopment.\nYou need to download data first."
        exit 1
    fi

    if [[ $wxfupdate ]]; then
        qdload=''
        rm "$temppath/wxcurrdata.html"
        exec "$0" && exit
    else
        qdload='-q'
    fi

    # if [[ $Latitude && $Longitude ]]; then
    #     have_posn=1
    # fi

    default_fct_url=""
    redo_hist=0

    default_wx_org=${default_wx,,}
    default_wx=${default_wx,,}

    case $default_wx in
    *:*)
        default_wx=${default_wx%:*}
        default_fct=${default_wx_org#*:}
        ;;
    *)
        default_fct=$default_wx
        ;;
    esac

    local -A dwx=( [1]=intellicast_id [2]=weatherdotcom_id [3]=msn_location [w]=Location [m]=icao_id [a]=accuwx [d]=darksky )
    local -A dfc=( [1]=intellicast_id [2]=weatherdotcom_id [3]=msn_location [w]=Location [a]=accuwx [d]=darksky )

    darksky=${darksky:-d}
    accuwx=${accuwx:-a}
    msn_location=${msn_location:-3}

    if [[ ${dwx[$default_wx]} ]]; then
        if [[ ${!dwx[$default_wx]} ]]; then
            default_wx=$default_wx
        else
            msgout "Error :( (${default_wx} -> \"${dwx[$default_wx]}\") for current weather not defined in config file.\n"
            exit 1
        fi
    else
        msgout "Error :( Default_wx current weather value \"$default_wx\" is out of range in config file.\n"
        exit 1
    fi
    if [[ ${dfc[$default_fct]} ]]; then
        if [[ ${!dfc[$default_fct]} ]]; then
            default_fct=$default_fct
        else
            msgout "Error :( (${default_fct} -> \"${dwx[$default_fct]}\") for forecast weather not defined in config file.\n"
            exit 1
        fi
    else
        msgout "Error :( Default_wx forecast weather value \"$default_fct\" is out of range in config file.\n"
        exit 1
    fi


}

trim() {
    echo $1
}

check_for_running_wx_process () {

    ### there are many ways to bell the cat - this option set for ps works for me !!!
    OLDIFS="$IFS"; IFS=$'\n'
    runningProcsArray=($(ps aux | perl -ne '
        if (/conky\s+\-c/) {
            my ($x,$y) = split(/\s+\-c\s+/, $_ );
            my (@z) = split(/\s+/, $x );
            print "$z[1]|$y" if $y;
        }'))
    IFS="$OLDIFS"

    confName=$(basename "$config_file")
    confName9="${confName}"
    confName10="${confName}',"

    conkyOptions="conkywx\s+\-t\s+|conkywx.sh(',)?"

    for d in "${!runningProcsArray[@]}"
    do
        OLDIFS="$IFS"; IFS=$'|'
        local larr=($(echo "${runningProcsArray[$d]}"))
        IFS="$OLDIFS"
        local lppid="${larr[0]}"
        local lpros="${larr[1]}"
        ### look for full path + file name
        if [[ $(grep -c "$config_file" "$lpros" 2> /dev/null) -gt 0 ]]; then
            ppidwx=$lppid
            break
        ### look for file name conky 1.10x style
        elif [[ $(grep -E -c ".*${confName10}$" "$lpros" 2> /dev/null) -gt 0 ]]; then
            ppidwx=$lppid
            break
        ### look for file name conky 1.9x old style
        elif [[ $(grep -E -c ".*$confName9$" "$lpros" 2> /dev/null) -gt 0 ]]; then
            ppidwx=$lppid
            break
        ### look for Default conf location - only single conf instance possible
        elif [[ $(grep -E -c ".*(${conkyOptions}).*" "$lpros" 2> /dev/null) -gt 0 && \
            $config_file == "/home/${USER}/.config/conkywx/conkywx.conf" ]]; then
            ppidwx=$lppid
            break
        fi
    done

    proc_check="$conkywx_internal_dir/${ppidwx}_id"

    if [[ ! -s "$proc_check" ]]; then
        printf "\"%b\"\n" "${ppidwx}" "${config_file}" "${forecast_trends}" > "$proc_check"
    fi

    id_files=($(find $conkywx_internal_dir/*_id -type f -printf "%f\n" 2> /dev/null ))

    for d in "${!id_files[@]}"
    do
        valid_proc=($(cat /proc/${id_files[$d]/_id}/stat 2> /dev/null))
        if [[ ${valid_proc[1]} != '(conky)' ]]; then
            rm -f "$conkywx_internal_dir/${id_files[$d]}" 2> /dev/null
        fi
    done

}

getWget() {

    local uagent="--user-agent=Mozilla/9.0 (Linux; Android 7.0.1; XT0045 Build/KXB21.14-L1.45) AppleWebKit/537.63 (KHTML, like Gecko) Chrome/39.0.2171.59 Mobile Safari/537.36"

    local tries=3

    case "$2" in
    *aviationweather*)
        local uagent="--user-agent=Mozilla/4.0 (compatible; MSIE 9.0; Windows NT 6.1; Trident/4.0; FDM; MSIE; Media Center PC 5.1)"
        ;;
    *\|\|*)
        local url1=${2%||*}; #msgout $default_wx
        local url2=${2#*||}; #msgout $default_fct
        local fA="$temppath/fct_dataA"
        local fB="$temppath/fct_dataB"
        ;;
    esac
    if [[ $wehaveAccu ]]; then
        wget "$qdload" -o /dev/null --tries=$tries --no-check-certificate --timeout=30 "$uagent"  -4 -O "$fA" "$url1"
        wget "$qdload" -o /dev/null --tries=$tries --no-check-certificate --timeout=30 "$uagent"  -4 -O "$fB" "$url2"
        accuUnitFct=$(sed -n '/<div class="control-wrapper">/{n;s/.*;<\/span//;p;}' "$fA")
            # msgout "$accuUnitWx"
        perl -ne 'print if /<!-- \/.feed-controls/../<!-- \/.feed-tabs -->/;' $fA > $fct_data
        perl -ne 'print if /<!-- \/.feed-controls/../<!-- \/.feed-tabs -->/;' $fB >> $fct_data
        rm "$imp_cookies" $fB $fA  2> /dev/null
        wehaveAccu=
    else
        if [[ -s "$imp_cookies" ]]; then
            wget "$qdload" -o /dev/null --tries=$tries --no-check-certificate --timeout=30 "$uagent" --load-cookies "$imp_cookies" -4 -O "$1" "$2"
            rm "$imp_cookies"  2> /dev/null
        else
            wget "$qdload" -o /dev/null --tries=$tries --no-check-certificate --timeout=30 "$uagent" -4 -O "$1" "$2"
        fi
        # msgout "$? $2"
    fi
    # if [[ $? -eq 4 ]]; then
    #     local err_DL="\nError :( Connection timed out !!! \nUnable to download from : \n$2 \n"
    #     msgout "$err_DL"
    #     show_noti "$err_DL" 1
    # fi
}


# -----------------------------------------------------------------------
# check for conkywx new version
# -----------------------------------------------------------------------
check_for_new_version () {

    # live version
    local new_ver="${conkywx_internal_dir}/conkywx_new_version_test"
    local ver_url='https://sites.google.com/site/foreverquest1/conkywx/latest_version'
    # Test version
    # local ver_url="https://sites.google.com/site/foreverquest1/conkywx/latest_version_test"
    case $version in
    *pre*) inner_ver=6.9.9.99  ;;
    *)   inner_ver=$version  ;;
    esac

    if [[ -s "$new_ver" ]]; then
        ver_time_diff=$(( nowtime1 - $( stat -c "%Y" "$new_ver" ) ))
    fi

    if [[ $ver_time_diff -gt 86400 || ! -s "$new_ver" ]]; then
        getWget "${new_ver}" "${ver_url}"
        touch $new_ver
    fi

    inner_ver=${inner_ver//[A-Z._]/}
    new_ver=$( cat "$new_ver" )
    new_ver=${new_ver//[A-Z._0]/}

    if [[ $inner_ver -lt $new_ver ]]; then
        echo "$new_ver"
    else
        echo 999
    fi

}


get_histcall() {
    # loc gets you xml_ID and wund history if already not required then we skipt this
    # if [[ ! $hist_in_url && $xml_ID != "nouse" ]]; then
    # if [[ ! $hist_in_url ]]; then
        derrmsg "info" "down loading loc lat lon"
        hist_url1="http://www.wunderground.com/cgi-bin/findweather/hdfForecast?query=${Latitude},${Longitude}"
        getWget "$histcall" "$hist_url1"
    # fi
}

get_history() {

    if [[ $1 ]]; then
        derrmsg "info" "downloading intellicast history file"
        getWget "$orghistin" "$hist_in_url"
    else
        # get historical data ###########################
        OLDIFS="$IFS"; IFS=$'-'
            if [[ $tz_long ]]; then
                dstr=($(TZ=$tz_long date +%F))
            else
                dstr=($(date +%F))
            fi
        IFS="$OLDIFS"

        if [[ -s $histcall ]]; then
            filetoread=$histcall
        else
            filetoread=$currdata
        fi

        if [[ ! "$station_id" ]]; then
            station_id=$(sed -n '/history_url/{s#.*ID=##;s#","ob_url.*##;p}' $filetoread)
        fi

        if [[ "$historyID" ]]; then
            station_id="$historyID"
        fi

        # msgout "st:$station_id, type:$st_type"
        if [[ $station_id ]]; then
            if [[ -s $histcall ]]; then
                rm -f $histcall  2> /dev/null
            fi
            # case $st_type in
            # pws)
            hist_url="${website}/weatherstation/WXDailyHistory.asp?ID=${station_id}&day=${dstr[2]}&month=${dstr[1]}&year=${dstr[0]}&graphspan=day&format=1"
            #   ;;
            # airport)
            #   hist_url="${website}/history/station/${station_id}/${dstr[0]}/${dstr[1]}/${dstr[2]}/DailyHistory.html?&format=1"
            #   ;;
            # esac
            derrmsg "info" "downloading wund history file"

            if [[ ! -s "$imp_cookies" ]]; then
                wxhdr='.wunderground.com    TRUE    /   FALSE   1577836800'
                wgetcookie="# HTTP cookie file.\n# Generated by ConkyWX !!!\n# Edit at your own risk. \n\n${wxhdr}  DT  ${nowtime1}:4141:365-s4\n${wxhdr}   ASC ${nowtime1}:1\n${wxhdr} Units   english"
                printf "\n%b\n" "$wgetcookie" > "$imp_cookies"
            fi

            getWget "$orghistwu" "$hist_url"
        fi
    fi
}

# -----------------------------------------------------------------------
# Get data from website
# -----------------------------------------------------------------------
get_data_file () {
# set -x
    local t1=
    wehaveAccu=

    local default_wx_url
    ####### Current weather
    case $default_wx in
    1)
        default_wx_url="http://www.intellicast.com/Local/Weather.aspx?unit=F&location=${intellicast_id}"
        hist_in_url="http://www.intellicast.com/Local/Observation.aspx?location=${intellicast_id}"
        if [[ $default_fct -eq 1 ]]; then fct_wx=9a ; fi
        title="intel99"
        ;;
    2)
        default_wx_url="http://wxdata.weather.com/wxdata/weather/local/${weatherdotcom_id}?+unit=F&dayf=10&cc=*"
        if [[ $default_fct -eq 2 ]]; then fct_wx=9a ; fi
        title="wx99"
        ;;
    3)
        # need this to convert locale to us locale - cannot handle language specific stuff
        if [[ $Latitude && $Longitude ]]; then
            msnLoc='https://www.msn.com/en-us/weather/today/US'
            Latitude=$( cleanNumber "${Latitude}"  )
            Longitude=$(cleanNumber "${Longitude}" )
            upos="${Latitude}%2C$Longitude"
            # upos="${Latitude},${Longitude}"
            default_wx_url="${msnLoc}/we-city-${upos}"
        else
            case "$msn_location" in
            3)
                msgout "\n\nError :-( To use MSN weather - declare your Latitude and Longitude or msn_location URL!!\n\nExiting conkywx!\n"
                exit 1
                ;;
            *)
                default_wx_url=$( fixurl "$msn_location" )
                ;;
            esac
        fi
        default_wx_url="${default_wx_url}?weadegreetype=F"
        if [[ $default_fct -eq 3 ]]; then fct_wx=9a ; fi
        title="msn99"
        ;;
    w)
        if [[ $Location =~ ^[0-9]+$ ]]; then
            default_wx_url="${wund_url}${Location}.html"
        elif  [[ $Location =~ locid* ]]; then
            default_wx_url="${website}/q/${Location}"
        elif  [[ $Location =~ http://* ]]; then
            default_wx_url="${Location}"
            # msgout "$Location"
        else
            wund_url='http://www.wunderground.com/cgi-bin/findweather/getForecast?query='
            Location=${Location,,}
            case "$Location" in
            # pos*:pws|pws:pos*)
            #     case "$Location" in
            #     *pws*)
            #         if [[ $xml_ID && $xml_ID != 'nouse' ]]; then
            #             upws="&sp=${xml_ID}" ; xml_ID=nouse
            #         else
            #             nopws=1
            #         fi
            #         ;;
            #     esac
            #     case "$Location" in
            #     *pos*)
            #         if [[ $Latitude && $Longitude ]]; then
            #             Latitude=$(cleanNumber "$Latitude")
            #             Longitude=$(cleanNumber "$Longitude")
            #             upos="${Latitude}%2C$Longitude"
            #         else
            #             nopos=1
            #         fi
            #         ;;
            #     esac
            #     default_wx_url="${wund_url}${upos}${upws}"
            #     ;;
            pws)
                if [[ $xml_ID && $xml_ID != 'nouse' ]]; then
                    upws="pws:${xml_ID}"; xml_ID=nouse
                else
                    nopws=1
                fi
                default_wx_url="${wund_url}${upws}"
                ;;
            pos*)
                if [[ $Latitude && $Longitude ]]; then
                    Latitude=$( cleanNumber "$Latitude"  )
                    Longitude=$(cleanNumber "$Longitude" )
                    upos="${Latitude}%2C$Longitude"
                else
                    nopos=1
                fi
                default_wx_url="${wund_url}${upos}${upws}"
                ;;
            *)
                msgout "\n\nError :-( Not a valid option for Location URL in conkywx config file!!\n\nExiting conkywx!\n"
                exit 1
            esac
            # msgout "${Latitude}, $Longitude"

            if [[ $nopws ]]; then
                msgout "\n\nError :-( To use pws in Location URL you need to state xml_ID in conkywx config file!!\n\nExiting conkywx!\n"
                exit 1
            fi
            if [[ $nopos ]]; then
                msgout "\n\nError :-( To use position in Location URL you need to state Latitude AND Longitude in conkywx config file!!\n\nExiting conkywx!\n"
                exit 1
            fi
            # msgout "$default_wx_url"
            # exit 0

        fi
        wxhdr='.wunderground.com    TRUE    /   FALSE   1577836800'
        wgetcookie="# HTTP cookie file.\n# Generated by ConkyWX !!!\n# Edit at your own risk. \n\n${wxhdr}  DT  ${nowtime1}:4141:365-s4\n${wxhdr}   ASC ${nowtime1}:1\n${wxhdr} Units   english"
        printf "\n%b\n" "$wgetcookie" > "$imp_cookies"
        title="wund99"
        ;;
    d)
        if [[ $Latitude && $Longitude ]]; then
            Latitude=$( cleanNumber "$Latitude"  )
            Longitude=$(cleanNumber "$Longitude" )
            upos="${Latitude}%2C$Longitude"
            default_wx_url="https://darksky.net/forecast/${upos}/us24/en"
        else
            msgout "\n\nError :-( To use Dark Sky you need to state Latitude AND Longitude in conkywx config file!!\n\nExiting conkywx!\n"
            exit 1
        fi
        if [[ $default_fct -eq 'd' ]]; then fct_wx=9a ; fi
        title="darkSky99"
        ;;
    m)
        default_wx_url="http://www.aviationweather.gov/adds/dataserver_current/httpparam?dataSource=metars&requestType=retrieve&format=csv&hoursBeforeNow=1&mostRecent=true&fields=raw_text,metar_type&stationString=${icao_id}"
        title="met99"
        ;;
    a)
        if [[ $Latitude && $Longitude && $accuwx == 'a' ]]; then
            Latitude=$( cleanNumber "$Latitude"  )
            Longitude=$(cleanNumber "$Longitude" )
            accuLoc_url="https://m.accuweather.com/location/currentlocation?lat=${Latitude}&lon=${Longitude}"
            accuLoc="$temppath/accuLoc=${Latitude}:${Longitude}"
            if [[ ! -s "${accuLoc}" ]]; then
                getWget "${accuLoc}" "$accuLoc_url"
            fi
            if [[ -s "${accuLoc}" ]]; then
                accuRealUrl=$(sed -n '/<link rel="canonical"/{s/.*https/https/;s/\".*//;s/weather-forecast/current-weather/;p}' "${accuLoc}")
                default_wx_url="$( fixurl ${accuRealUrl})"
            fi
            # msgout "${default_wx_url};;;;;${accuRealUrl}"
        else
            accuwx=$( echo ${accuwx/weather-forecast/current-weather} )
            default_wx_url=$( fixurl "$accuwx" )
        fi
        # exit 0
        # "$conkywx_internal_dir/accu.url"

        # default_wx_url='https://m.accuweather.com/location/currentlocation?lat=28.163133&lon=-80.60343'
        # unitID=$( echo ${accuwx##*/} )
        # printf "\n%b\n" "$accuCookie" > "$imp_cookies"
        title="accu99"
        ;;
    esac

    msgout "$mdl_data"

    getWget "$dl_currdata" "$default_wx_url"

    # check for valid metar data
    if [[ $default_wx == 'm' && $(grep 'no data available' "$dl_currdata") != "" ]]; then
        local err_met="\n\nError: :( \nChoose another METAR Station\nThis Station does not appear to be sending METAR reports.\n"
        msgout "$err_met"
        show_noti "$err_met" 1
        exit 1
    elif [[ $default_wx == 'w' && $(grep "Oops! There's been an error." "$dl_currdata") != "" ]]; then
        local err_wund="\n\nError: :( \nChoose another Wunderground Location\nThis does not appear to be a valid Location.\n"
        msgout "$err_wund"
        show_noti "$err_wund" 1
        exit 1
    fi

# set -x
    if [[ ! -s "$dl_currdata" ]]; then
        if [[ -s "$currdata" ]]; then
            touch "$old_data"
            rm -f "$dl_currdata" 2> /dev/null
            t1=9999
        else
            msgout "$warning_msg_no_download"
            show_noti "$warning_msg_no_download"
            rm -f "$dl_currdata" 2> /dev/null
            exit 1
        fi
    else
        if [[ $(type -p iconv) ]]; then
            sed '/^\s*$/d' $dl_currdata | iconv -c -f UTF-8 -t UTF-8 - -o $currdata
        else
            sed '/^\s*$/d' $dl_currdata > $currdata
        fi
        rm -f "$dl_currdata" "$old_data" 2> /dev/null
        if [[ $default_wx == 'a' ]]; then
            # accuwarn=$( fixurl "$accuwx" 2 )
            accuwarn=$( fixurl "${accuRealUrl}" 2 )
            # accuwarn=$( echo ${accuwx/current-weather/weather-warnings} )
            if [[ $(grep -m 1 -F "$accuwarn"  $currdata) != '' ]]; then
                getWget "$temppath/accuwarn" "${accuwarn}"
            else
                rm "$temppath/accuwarn" 2> /dev/null
            fi
        fi
    fi

    ####### Forecast
    case $default_fct in
    1)
        default_fct_url="http://www.intellicast.com/Local/Weather.aspx?unit=F&location=${intellicast_id}"
        ;;
    2)
        default_fct_url="http://wxdata.weather.com/wxdata/weather/local/${weatherdotcom_id}?+unit=F&dayf=10&cc=*"
        ;;
    3)
        if [[ $fct_wx != '9a' ]]; then
            if [[ $Latitude && $Longitude ]]; then
                msnLoc='https://www.msn.com/en-us/weather/today/US'
                Latitude=$( cleanNumber "${Latitude}"  )
                Longitude=$(cleanNumber "${Longitude}" )
                upos="${Latitude}%2C$Longitude"
                # upos="${Latitude},${Longitude}"
                default_fct_url="${msnLoc}/we-city-${upos}"
            else
                case "$msn_location" in
                3)
                    msgout "\n\nError :-( To use MSN weather forecast - declare your Latitude and Longitude or\nmsn_location URL!!\n\nExiting conkywx!\n"
                    exit 1
                    ;;
                *)
                    default_fct_url=$( fixurl "$msn_location" )
                    ;;
                esac
                # default_wx_url=$( fixurl "$msn_location" )
            fi
            default_fct_url="${default_fct_url}?weadegreetype=F"
        fi
        ;;
    a)
        if [[ $Latitude && $Longitude && $accuwx == 'a' ]]; then
            Latitude=$( cleanNumber "$Latitude"  )
            Longitude=$(cleanNumber "$Longitude" )
            accuLoc_url="https://m.accuweather.com/location/currentlocation?lat=${Latitude}&lon=${Longitude}"
            accuLoc="$temppath/accuLoc=${Latitude}:${Longitude}"
            if [[ ! -s "${accuLoc}" ]]; then
                getWget "${accuLoc}" "$accuLoc_url"
            fi
            if [[ -s "${accuLoc}" ]]; then
                accuRealUrl=$(sed -n '/<link rel="canonical"/{s/.*https/https/;s/\".*//;s/weather-forecast/current-weather/;p}' "${accuLoc}")
                accuwxa="$( fixurl ${accuRealUrl})"
                accuft=$( fixurl "$accuwxa" 1 )
                accuft="${accuft}?day=6"
                if [[ $default_wx != 'a' ]]; then
                    default_fct_url="${accuwxa}||${accuft}"
                    wehaveAccu=1
                else
                    default_fct_url="${accuft}"
                fi
                unitID=$( echo ${accuwxa##*/} )
            fi
        else
            accuwx=$( fixurl "$accuwx"   )
            accuft=$( fixurl "$accuwx" 1 )
            accuft="${accuft}?day=6"
            if [[ $default_wx != 'a' ]]; then
                default_fct_url="${accuwx}||${accuft}"
                wehaveAccu=1
            else
                default_fct_url="${accuft}"
            fi
            unitID=$( echo ${accuwx##*/} )
        fi
        # printf "\n%b\n" "$accuCookie" > "$imp_cookies"
        ;;
    d)
        if [[ $Latitude && $Longitude ]]; then
            Latitude=$( cleanNumber "$Latitude"  )
            Longitude=$(cleanNumber "$Longitude" )
            upos="${Latitude}%2C$Longitude"
            default_fct_url="https://darksky.net/forecast/${upos}/us24/en"
        else
            msgout "\n\nError :-( To use Dark Sky you need to state Latitude AND Longitude in conkywx config file!!\n\nExiting conkywx!\n"
            exit 1
        fi
        ;;
    w)
        case $default_wx in
        w)
            # wundKey=$( sed -n '/conditions\/labels/{s#/conditions/labels.*##;s#.*api/##;p}' $currdata)
            wundKey=$(perl -ne 'if (m{com/api/(.*?)/conditions/labels/}) {print $1}' $currdata)
            # msgout ">>>>wunderground fct key>>>:$wundKey"

            default_fct_url="https://api-ak.wunderground.com/api/${wundKey}/forecast10day/lang:EN/units:english/bestfct:1/v:2.0/q/${Latitude},${Longitude}.json?ttl=120"
            # default_fct_url="${baseWundFct}"
            ;;
        *)
            msgout "\n\nError: :( \nChoose Wunderground for main weather. Wunderground forecast cannot be used with another station.\n"
            ;;
        esac
    esac

    # download forecast weather
    if [[ $fct_wx != "9a" ]]; then
        getWget "${fct_data1}" "${default_fct_url}"
    else
        rm "$temppath/fct_data.html" 2> /dev/null
    fi

    # exit 0

    if [[ $do_history || $hist_time -gt 21600 ]]; then

        derrmsg "info" "removing hiscall n orghistdata files"

        rm "$histcall" "$orghistin" "$orghistwu"  2> /dev/null

        if [[ $xml_ID && $xml_ID != 'nouse' ]]; then
            station_id=$xml_ID
            get_history
        elif [[ $historyID ]]; then
            get_history
        elif [[ $default_wx == 'w' ]]; then
            get_history
        elif [[ $Latitude && $Longitude ]]; then
            get_histcall             # if posn in config - get wund page
            get_history              # get history data
        fi
    fi

    validate_xml

    get_opt2_data

    ( get_opt_data ) &

    # if [[ -s "$orghistwu" ]]
    : '
        need to get sol value if
        - main is not darksky
            - xml thing is not with sol thing
                - get data from darksky
            - xml thing is with sol thing
                - get data from xml thing

        if [[ $default_wx != 'd' ]]; then
            if [[-s "$orghistwu" && $(grep 'SolarRadiationWatts' "$orghistwu") != ""]]; then
                OLDIFS="$IFS"; IFS=$','
                solValue=($(tail -2 ./orghistwu | head -1))
                IFS=$OLDIFS
            fi
        fi

        If you set OFS=",", you can simply write print $3, $4, $5
    '
    if [[ $default_wx != 'd' ]]; then
        # if [[ -s "$orghistwu" && $(grep 'SolarRadiationWatts' "$orghistwu") != "" ]]; then
        #     # OLDIFS="$IFS"; IFS=$','
        #     # solValue=($(tail -2 "$orghistwu" | head -1))
        #     # IFS=$OLDIFS
        #     # solValue=${solValue[13]}
        #     # we handle this later
        #     solValue=''
        # else
            if [[ $Latitude && $Longitude ]]; then
                Latitude=$( cleanNumber "$Latitude"  )
                Longitude=$(cleanNumber "$Longitude" )
                upos="${Latitude}%2C$Longitude"
                darksky_wx_url="https://darksky.net/forecast/${upos}/us24/en"
                getWget "$temppath/solValueF" "${darksky_wx_url}"
                # if [[ -s "$temppath/solValueF" ]]; then
                #     solValue=$(perl -ne 'if (m{currently = \{.*,"ghi":(.*?),"dhi":}) {print $1}' "$temppath/solValueF")
                # fi
            fi
        # fi
    fi

    # msgout "$solValue"


    if [[ $enable_new_version_check ]]; then
        if [[ ! $debug ]]; then
            t1=$( check_for_new_version )
        else
            t1=999
        fi
    elif [[ ! $t1 ]]; then
        t1=999
    fi

    if [[ $first_run && $t1 -ne 9999 ]]; then
        data_checker
    fi

    local wx_msg
    case $t1 in
    999)    wx_msg="$wx_update_ok"                      ;;
    9999)   wx_msg="$warning_msg_no_download"           ;;
    *)      wx_msg="$wx_update_ok.\n$new_ver_msg $t1"   ;;
    esac

    msgout "$wx_msg"
    show_noti "$wx_msg"

}

get_opt2_data() {

    if [[ $get_seismic ]]; then
        if [[ $there_is_an_opt_change6 ]] ; then
            get_seismic_data
        elif [[ -s "$seismic_dataA" ]]; then
            if [[ $(($nowtime1 - $(date +%s -r "$seismic_dataA"))) -gt 1200 ]]; then
                get_seismic_data
            fi
        else
            get_seismic_data
        fi
    else
        if [[ -s $seismic_dataA || -s $seismic_dataB || -s $seismic ]]; then
            rm $seismic_dataA $seismic_dataB $seismic 2> /dev/null
        fi
    fi

    if [[ $tides_url ]]; then
        local tidestr=${tides_url//\// }
        tidearr=($(echo ${tidestr}))
        tide_data="$conkywx_internal_dir/tide_data_wx-${USER}-$conf1-pChA_r_27u0-m39_${tidearr[3]}"

        if [[ -s $tide_data ]]; then
            if [[ $(($nowtime1 - $(date +%s -r "$tide_data"))) -gt 1728000 ]]; then
                get_tides
            fi
        else
            get_tides
        fi
    elif [[ -s $tides ]]; then
        rm $tides 2> /dev/null
    fi

    if [[ $gdacsUrl ]]; then
        get_gdacsrss_data
    elif [[ -s "$temppath/gdacs" ]]; then
        rm "$temppath/gdacs" 2> /dev/null
    fi
    if [[ $newsUrl ]]; then
        get_newsrss_data
    elif [[ -s "$temppath/news"  ]]; then
        rm "$temppath/news" 2> /dev/null
    fi

}

get_opt_data() {
# set -x

    if [[ $cimss_surface_image ]]; then
        if [[ $there_is_an_opt_change1 ]] ; then
            get_cimss_surface_image
        elif [[ -s $gsi_image ]]; then
            if [[ $(($nowtime1 - $(date +%s -r "$gsi_image"))) -gt 10800 ]]; then
                get_cimss_surface_image
            fi
        else
            get_cimss_surface_image
        fi
    else
        rm $gsi_image 2> /dev/null
    fi

    if [[ $wund_surface_image ]]; then
        if [[ $there_is_an_opt_change2 ]] ; then
            get_wund_surface_image
        elif [[ -s "$gwsi_image" ]]; then
            if [[ $(($nowtime1 - $(date +%s -r "$gwsi_image"))) -gt 10800 ]]; then
                get_wund_surface_image
            fi
        # elif [[ -s "$gwsi_image.jpg" ]]; then
        #   if [[ $(($nowtime1 - $(date +%s -r "$gwsi_image.jpg"))) -gt 10800 ]]; then
        #       get_wund_surface_image
        #   fi
        else
            get_wund_surface_image
        fi
    else
        rm "${gwsi_image}" 2> /dev/null
    fi

    if [[ $intel_surface_image ]]; then
        if [[ $there_is_an_opt_change3 ]] ; then
            get_intel_surface_image
        elif [[ -s $gisi_image_sat ]]; then
            if [[ $(($nowtime1 - $(date +%s -r "$gisi_image_sat"))) -gt 10800 ]]; then
                get_intel_surface_image
            fi
        else
            get_intel_surface_image
        fi
    else
        rm $gisi_image_sat $gisi_image_sur 2> /dev/null
    fi

    if [[ $world_wx_image ]]; then
        if [[ $it_is_time || ! -s $wwx_image || $there_is_an_opt_change4 ]]; then
            get_world_wx_image
        fi
    else
        rm $wwx_image 2> /dev/null
    fi

}


get_intel_surface_image() {
    local inImage=( "afric" "nwafr" "eafri" "wafri" "safri" "asian" "rusia" "india" "indon" "chjap" "midea" "seasi" "austr" "neaus" "seaus" "waust" "nzeal" "namer" "camer" "samer" "nsamr" "ssamr" "carib" "brazl" "mexic" "ccana" "ecana" "wcana" "europ" "neeur" "nweur" "scand" "seeur" "sweur" "uking" "conus" "neusa" "nwusa" "ncusa" "ecusa" "ceusa" "wcusa" "scusa" "seusa" "swusa" )

    imageName="${inImage[$((intel_surface_image-1))]}"

    gisi_url_sur="http://images.intellicast.com/WxImages/SurfaceAnalysis/${imageName}.jpg"
    gisi_url_sat="http://images.intellicast.com/WxImages/Satellite/${imageName}.jpg"

    msgout "Updating intellicast surface image..."

    getWget "$gisi_image_sat" "$gisi_url_sat" &
    getWget "$gisi_image_sur" "$gisi_url_sur" &
}


get_cimss_surface_image() {

    # gsi_base_url="http://tropic.ssec.wisc.edu/real-time"

    local cimss_surface_image1=${cimss_surface_image,,}
    local high_low=${cimss_surface_image,,}

    case $cimss_surface_image1 in
    *:*)
        cimss_surface_image1=${cimss_surface_image1%:*};
        high_low=${high_low#*:};
        ;;
    esac

    local gsi_area=( "indian" "eastpac" "westpac" "seastpac" "atlantic" "europe" "austeast" "austwest" "atlpac" )
    local gsi_high=( "wm5wvir" "wg9wvir" "wgmswvir" "wg10swvir" "wg8wvir" "wm7wvir" "wgmswvir" "wgmswvir" "wg810wvir" )
    local gsi_low=( "wm5ir" "wg9ir" "wgmsir" "wg10sir" "wg8ir" "wm7ir" "wgmsir" "wgmsir" "wg810ir" )

    case $cimss_surface_image1 in
    6|9)
        local nz=1 ;;
    esac

    local gsi_fname
    case $high_low in
    l)   gsi_fname=${gsi_low[$((cimss_surface_image1-1))]}       ;;
    u)   gsi_fname=${gsi_high[$((cimss_surface_image1-1))]}       ;;
    lz)
        if [[ ! $nz ]]; then
            gsi_fname="${gsi_low[$((cimss_surface_image1-1))]}Z"
        else
            gsi_fname="${gsi_low[$((cimss_surface_image1-1))]}"
        fi
        ;;
    uz)
        if [[ ! $nz ]]; then
            gsi_fname="${gsi_high[$((cimss_surface_image1-1))]}Z"
        else
            gsi_fname="${gsi_high[$((cimss_surface_image1-1))]}"
        fi
        ;;
    *)   gsi_fname=${gsi_high[$((cimss_surface_image1-1))]}      ;;
    esac

    local gsi_url="http://tropic.ssec.wisc.edu/real-time/${gsi_area[$((cimss_surface_image1-1))]}/winds/${gsi_fname}.GIF"

    msgout "Updating CIMSS surface image..."
    getWget "$gsi_image" "$gsi_url" &
}

get_wund_surface_image() {

    # temp=st, heat-index=hi, windchill=wc,windspeed=ws,rhumdi=rh, dewp=dp,vis=vs
    local typeArr=( 'na' 'st' 'hi' 'wc' 'ws' 'rh' 'dp' 'vs' )
    local locArr=( 'na' 'eu' 'i_uk' 'i_sn' 'i_dl' 'i_sp' 'i_fr' 'i_pl' 'i_ro' 'i_tu' 'i_gr' 'i_iy' 'af' 'as' 'a2' 'i_in' 'i_id' 'i_ah' 'i_jp' 'au' 'cn' 'ak' 'us' 'ne' 'se' 'nw' 'sw' 'mw' 'sp' 'ca' 'sa' 'an' )
    local wundImageUrl="https://icons.wxug.com/data/640x480/2x${locArr[${wund_surface_image%:*}]}_${typeArr[${wund_surface_image#*:}]}.gif"

    case $wundImageUrl in
    *na*)
        ;;
    *)
        # msgout "sss ${wundImageUrl}"
        msgout "Updating wunderground surface image..."
        getWget "${gwsi_image}.gif" "$wundImageUrl" &
        ;;
    esac

}

get_world_wx_image() {
    local wwxi
    case $world_wx_image in
    1)  wwxi="world"    ;; # Imperial
    2)  wwxi="world2"   ;; # Metric
    esac
    local wwxi_url="http://www.eldoradocountyweather.com/current/skymet/skymet_${wwxi}.jpeg"
    msgout "Updating World Weather Image..."
    getWget "${wwx_image}" "${wwxi_url}" &
}

get_tides() {
    local tide_data1="$temppath/tide_data1.html"
    rm $tide_data $tides 2> /dev/null
    msgout "Updating Tidal data..."
    getWget "$tide_data1" "$tides_url"
    if [[ -s "$tide_data1" ]]; then
        sed '/^\s*$/d' "$tide_data1" > "$tide_data"
        rm -f "$tide_data1" 2> /dev/null
    fi
}

get_gdacsrss_data() {
    local inGdacsUrl=( ${gdacsUrl//\// } )
    if [[ $inGdacsUrl ]]; then
        msgout "Updating "${inGdacsUrl[1]}" rss data..."
        getWget "${temppath}/gdacsData" "$gdacsUrl"
    fi
}

get_newsrss_data() {
    # msgout "${#newsUrl[@]}"
    for i in "${!newsUrl[@]}"; do
        local inNewsUrl=( ${newsUrl[$i]//\// } )
        if [[ $inNewsUrl ]]; then
            msgout "Updating "${inNewsUrl[1]}" rss data..."
            getWget "${temppath}/newsRSS${i}" "${newsUrl[$i]}"

            charSet="$(file -bi "${temppath}/newsRSS${i}")"

            if [[ $i -eq 0 ]]; then
                    case "$charSet" in
                    *us-ascii*)
                        iconv -f UK -t UTF-8 "${temppath}/newsRSS${i}" > "${temppath}/newsData"
                        ;;
                    *)  cat "${temppath}/newsRSS${i}" > "${temppath}/newsData"
                        ;;
                    esac
            else
                printf "\n<item><title>@@@${i}@@@</title><description>------------------------</description><pubDate>$(date -R)</pubDate></item> \n" >> "${temppath}/newsData"
                case "$charSet" in
                *us-ascii*)
                    iconv -f UK -t UTF-8 "${temppath}/newsRSS${i}" >> "${temppath}/newsData"
                    ;;
                *)  cat "${temppath}/newsRSS${i}" >> "${temppath}/newsData"
                    ;;
                esac
            fi
        fi
        rm "${temppath}/newsRSS"*
        perl -pi -e 's/^\s*$//;s/\r$//' "${temppath}/newsData"
    done
}

get_seismic_data() {

    # rm $seismic_dataA $seismic_dataB $seismic 2> /dev/null

    local seismic_urlb='http://earthquake.usgs.gov/fdsnws/event/1/query?format=csv&limit=60&minmagnitude=2.5'
    local seismic_url1='http://www.seismicportal.eu/fdsnws/event/1/query?limit='
    local seismic_url2='&format=text&minmag='
    local seismic_urla="${seismic_url1}60${seismic_url2}2.5"

    msgout "Updating seismic data..."
    getWget "${seismic_dataA}" "${seismic_urla}"
    getWget "${seismic_dataB}" "${seismic_urlb}"
}

get_xml() {

    local err_xml0 err_xml err_xml1
    msgout "$xdl_data"

    derrmsg "info" "downloading xml pws data file"
    getWget "$dl_currdata_2" "${xml_url1}&format=XML"

    if [[ -s "$dl_currdata_2" && $(grep '#WXSTATION.CUR.StationID' "$dl_currdata_2") != "" ]]; then
        rm "$dl_currdata_2" "$currdatax" "$currdata_2" 2> /dev/null
        if [[ ${xml_ID2} ]]; then
            msgout "Trying secondary pws xml_ID2..."
            getWget "$dl_currdata_2" "${xml_url2}&format=XML"
            if [[ -s "$dl_currdata_2" && $(grep '#WXSTATION.CUR.StationID' "$dl_currdata_2") != "" ]]; then
                err_xml0='change xml_ID and \nxml_ID2 in conkywx config file. conkywx is unable to use \nsecondary pws xml_ID2. This Station also does not \nappear to be sending data.\nconkywx will use main station for weather.\n'
                rm "$dl_currdata_2" "$currdatax" "$currdata_2" 2> /dev/null
            else
                mv -f "$dl_currdata_2" "$currdata_2"
                station_id=${xml_ID2}
                get_history
                err_xml0='change xml_ID in conkywx config file.\nconkywx is using secondary pws xml_ID2 for pws data.'
                touch "$temppath/UsingXML2"
            fi
        else
            err_xml0='change xml_ID and \ndefine xml_ID2 in conkywx config file. '
            err_xml1='conkywx will \nuse main station for weather as no secondary \npws xml_ID2 defined in conkywx config file.\nconkywx will use main station for weather.\n'
        fi
        err_xml="\n\nError: :( \nChoose another PWS Station(s) - ${err_xml0}${err_xml1}\n"
        msgout "$err_xml"
        show_noti "$err_xml" 1
    else
        mv -f "$dl_currdata_2" "$currdata_2"
    fi
}

validate_xml() {
    local done_xml
    if [[ $do_xml -eq 1 && ! -f "$old_data" ]]; then
        if [[ $xml_ID != 'nouse' ]]; then
            if [[ ${st_type} == "pws" && ${station_id} ]]; then
                # automagic xml station
                xml_url1=${xml_url}${station_id}
                xml_url2=${xml_url}${xml_ID2}
                done_xml=1
                get_xml
            elif [[ ${xml_ID} ]]; then
                # manual override station
                xml_url1=${xml_url}${xml_ID}
                xml_url2=${xml_url}${xml_ID2}
                done_xml=1
                get_xml
            else
                if [[ ! $done_xml ]]; then rm $currdatax $currdata_2 2> /dev/null; fi
            fi
        else
            rm $currdatax $currdata_2 2> /dev/null
        fi
    fi
}


do_gdf() {

    if [[ ! $1 ]]; then
        do_xml=1
        get_data_file
        # msgout ">>> here 3"
    elif [[ $there_is_an_opt_change ]]; then
        # msgout "in go_gdf"
        get_opt2_data
        get_opt_data
        # msgout ">>> here 4"
    fi

    if [[ $wxdevelopment || $debug ]]; then
        get_present_data
        # msgout ">>> here 1"
    else
        get_present_data &
        # msgout ">>> here 2"
    fi

    devtest=1
}

data_checker() {

    # create a dummy file as template for -d options ;-)
    if [[ $clopt1 == "-d" ]] && [[ ! "$input_template" ]]; then
        input_template="$temppath/dummy_template"
        printf "%s\n" "dummy template file" > "$input_template"
    fi

    local testit1
    if [[ -s "$input_template" && -s "$config_file" && -s "$system_config_file" ]]; then
        testit1=($(md5sum "$input_template" "$config_file" "$system_config_file"))
    fi

    check_now=( "md5_1=${testit1[0]}" "md5_2=${testit1[2]}" "md5_3=${testit1[4]}" "loc9_1=${Location}" "loc9_2=${xml_ID}" "loc9_3=${xml_ID2}" "loc9_4=${weatherdotcom_id}" "loc9_5=${msn_location}" "loc9_6=${intellicast_id}" "loc9_7=${default_wx_org}" "loc9_8=${icao_id}" "loc9_9=${accuwx}" "loc9_10=${Latitude}" "loc9_11=${Longitude}" "loc9_12=${historyID}" "opt9_1=$cimss_surface_image" "opt9_2=$wund_surface_image" "opt9_3=$intel_surface_image" "opt9_4=$world_wx_image" "opt9_5=$tides_url" "opt9_6=$get_seismic" "opt9_9=${gdacsUrl}" "opt9_10=${newsUrl[*]}" "config_1=$config_file" "template_1=$input_template"  "systemConfig_1=$system_config_file" )


    if [[ -s "$temppath/wxdata_check" ]]; then
        OIFS=$IFS; IFS=$'\n'
            check_store=($(< "$temppath/wxdata_check"))
        IFS=$OIFS
    else
        printf "%s\n" "${check_now[@]}" > "$temppath/wxdata_check"
    fi

}


# -----------------------------------------------------------------------
# check location n xml or minimum fetch time is adhered to
# -----------------------------------------------------------------------
check_for_data () {

    do_history=
    if [[ -s "$graphdata" ]]; then
        hist_time=$(($nowtime1 - $(date +%s -r "$graphdata")))
    elif [[ -s "$orghistin" ]]; then
        hist_time=$(($nowtime1 - $(date +%s -r "$orghistin")))
    elif [[ -s "$orghistwu" ]]; then
        hist_time=$(($nowtime1 - $(date +%s -r "$orghistwu")))
    else
        hist_time=$((21600 + $nowtime1))
    fi

    if [[ -s "$currdata" ]]; then
        actualsize=($(stat -c "%s %Y" "$currdata"))

        # ${actualsize[0]}=file size in bytes,
        # ${actualsize[1]}=is last modify file time
        time_diff1=$(( $nowtime1 - ${actualsize[1]} ))

        if [[ $time_diff1 -gt $minimum_fetch_time ]]; then it_is_time=1 ; fi

        data_checker

        str1="md5"; str2="loc9"; str3="opt9"
        for e in ${!check_now[@]}
        do
            if [[ "${check_now[$e]}" != "${check_store[$e]}" ]]; then
                if [[ "${check_now[$e]}" =~ .*$str1.* ]]; then
                    md5change=1
                elif [[ "${check_now[$e]}" =~ .*$str3.* ]]; then
                    check_lt="${check_now[$e]/=*}"
                    check_lt="${check_lt/*_}"
                    case $check_lt in
                    1)  there_is_an_opt_change1=1 ;;
                    2)  there_is_an_opt_change2=1 ;;
                    3)  there_is_an_opt_change3=1 ;;
                    4)  there_is_an_opt_change4=1 ;;
                    5)  there_is_an_opt_change5=1 ;;
                    6)  there_is_an_opt_change6=1 ;;
                    7)  there_is_an_opt_change7=1 ;;
                    8)  there_is_an_opt_change7=1 ;;
                    esac
                    if [[ $there_is_an_opt_change7 ]]; then
                        rm -f "$uvdata" 2> /dev/null
                        md5change=1
                    else
                        there_is_an_opt_change=1
                    fi
                elif [[ "${check_now[$e]}" =~ .*$str2.* ]]; then
                    there_is_a_change=1
                fi
                printf "%s\n" "${check_now[@]}" > "$data_check"
            fi
        done

        if [[ $wxsupdate ]]; then
            there_is_an_opt_change6=1
            there_is_an_opt_change=1
            md5change=1
        fi
        first_run=
    else
        first_run=1
    fi

    if [[ ! $wxdevelopment ]]; then
        if [[ ! -s "$currdata"                    || \
                    $it_is_time                   || \
                    ${actualsize[0]} -eq 0        || \
                    $there_is_a_change ]]; then
            touch "$temppath/loading"
            there_is_a_change=1
            do_history=1
            do_gdf
            # msgout ">>>here at A"
        else
            if [[ $md5change || $clopt1 == "-d" || ! -s "$temppath/graphdata" ]]; then
                touch "$temppath/loading"
                msgout "$recal_data"
                do_gdf 1
                # msgout ">>>here at B"
            fi
        fi
    elif [[ ! -s $currdata || $there_is_a_change ]]; then
        touch "$temppath/loading"
        there_is_a_change=1
        do_history=1
        do_gdf
        # msgout ">>>here at C"
    elif [[ $wxdevelopment ]]; then
        touch "$temppath/loading"
        do_gdf 1
        # msgout ">>>here at D"
    else
        touch "$temppath/loading"
        do_gdf 1
        # msgout ">>>here at E"
    fi
}

# -----------------------------------------------------------------------
# Extract data from downloaded files pass it on to Perl
# -----------------------------------------------------------------------
get_present_data () {

    if [[ -s "$temppath/fct_data1.html" ]] && [[ ! $wehaveAccu ]]; then
        sed '/^\s*$/d' "$temppath/fct_data1.html" > "$temppath/fct_data.html"
        rm -f "$temppath/fct_data1.html" 2> /dev/null
    fi

    if [[ $default_wx == 'a' ]]; then
        accuUnitWx=$(sed -n '/<div class="control-wrapper">/{n;s/.*;<\/span//;s/\/n//;p;}' $currdata)
    fi
    if [[ $default_wx == 'a' && $default_fct == 'a' ]]; then
    # if [[ $default_fct == 'a' ]]; then
        # accuUnitFct=$(sed -n '/<div class="control-wrapper">/{n;s/.*;<\/span//;s/\/n//;p;}' "$temppath/fct_data.html" )
        accuUnitFct=$accuUnitWx
    fi
    if [[ $default_wx != 'd' ]]; then
        # if [[ -s "$orghistwu" && $(grep 'SolarRadiationWatts' "$orghistwu") != "" ]]; then
        #     OLDIFS="$IFS"; IFS=$','
        #     solValue=($(tail -2 "$orghistwu" | head -1))
        #     IFS=$OLDIFS
        #     solValue=${solValue[13]}
        # else
            if [[ -s "$temppath/solValueF" ]]; then
                solValue=$(perl -ne 'if (m{currently = \{.*,"ghi":(.*?),"dhi":}) {print $1}' "$temppath/solValueF")
            fi
        # fi
    fi

(
    if [[ $wxprofile ]]; then
        perl -d:NYTProf "${cwxpath}/lib/conkywx.pl" "${temppath}|${input_template}|${cwxpath}|${input_element}|${config_file}|${forecast_trends}|$msgs_file|${clopt1}|${default_wx}|${default_fct}|${accuUnitWx}|${accuUnitFct}|${solValue}"
        nytprofhtml --open
    else
        perl "${cwxpath}/lib/conkywx.pl" "${temppath}|${input_template}|${cwxpath}|${input_element}|${config_file}|${forecast_trends}|$msgs_file|${clopt1}|${default_wx}|${default_fct}|${accuUnitWx}|${accuUnitFct}|${solValue}"
    fi
)

}

# -----------------------------------------------------------------------
# Finaly get the data out of production to sales
# -----------------------------------------------------------------------
process_data () {

    if [[ -s $die_error ]]; then
        OIFS="$IFS"; IFS=$'\n'
        set -f
        derror=( $(< "$die_error" ) )
        set +f
        IFS="$OIFS"
        msgout "${derror[@]}\n"
        exit 1
    fi

    case "$clopt1" in
    -t)
        if [[ -s "$temppath/wxtemp_template" && ! $do_not_output_to_conky ]]; then
            cat "$temppath/wxtemp_template"
        fi
        if [[ -s "$temppath/wxtemp_templateSC" ]]; then
            if [[ $link_tmp_folder ]]; then
                rm -f "$temppath/wxtemp_templateSC" 2> /dev/null
            else
                cat "$temppath/wxtemp_templateSC"
            fi
        fi
        ;;
    esac
}


do_debug () {

    msgout "$dbg_msg_str1 $FUNCNAME"
    exec 6>&2
    exec 2>>"$wund_debug_log"
    # set -x

    conky_version=($(conky -v))
    printf "%b\n" "$debug_msg" > "$wund_debug_log"

    msgout "\ndebug time: $(date)\n"
    msgout "\nconky_version=${conky_version[1]}\n\nconkywx_version=${version}\n"

    do_gdf

    exec 2>&6 6>&-

enddebugmsg="
$ca_dash_line
$dbg_01
$ca_dash_line
$dbg_02
${config_file}
$dbg_03
${conkywx_debug}
$dbg_04
$ca_dash_line\n"
msgout "$enddebugmsg" N

    cd $temppath

    if [[ -s $conkywx_debug ]]; then
        rm -f $conkywx_debug 2> /dev/null
    fi

    userconfarray[0]="\n==================Users Config file================="
    cp $config_file "$temppath/wxuser.conf"

    if [[ -e '/tmp/wxdebug' ]]; then
        set +x
        mv '/tmp/wxdebug' "$temppath"
    fi

    tar -jcf $conkywx_debug se* org* gra* ti* wx* hi* fc* 2> /dev/null

    if [[ -s $wund_debug_log ]]; then
        rm -f $wund_debug_log "$temppath/wxuser.conf" "$temppath/wxdebug" 2> /dev/null
    fi

}


# -----------------------------------------------------------------------
# Process engineering
# -----------------------------------------------------------------------
do_main () {

    # set_config
    set_variables

    if [[ -f "$temppath/dummy_debug" ]]; then
        debug=1
        do_debug
        rm -f "$temppath/dummy_debug" 2> /dev/null
        check_for_running_wx_process
    else
        if [[ "$link_tmp_folder" == */ ]]; then link_tmp_folder=${link_tmp_folder%/*}; fi
        if [[ "$link_tmp_folder" ]]; then
            sleep 1
            # msgout "we are here inside link 0"
            tmp9=$( echo "$link_tmp_folder" | sed "s/\/tmp\/wx-${USER}-//;s/-pChA_r_27u0-m39//" )
            tmpConf="${config_file%/*}/${tmp9}"
            tmpConf=$(grep -e "^default_wx*" ${tmpConf})
            if [[ ! -f "$link_tmp_folder/loading" ]]; then
                if [[ $tmpConf ]]; then
                    default_wx=${tmpConf##*=}
                    default_wx_org=${default_wx,,}
                    default_wx=${default_wx,,}
                    case $default_wx in
                    *:*)
                        default_wx=${default_wx%:*}
                        default_fct=${default_wx_org#*:}
                        ;;
                    *)
                        default_fct=$default_wx
                        ;;
                    esac
                fi
                rm "${temppath}/*.html" "${temppath}/*.xml" 2> /dev/null
                cp -fr "${link_tmp_folder}/." -t "$temppath"
                rm "${temppath}/wxtemp_template" "${temppath}/seism*" 2> /dev/null
                if [[ $wxdevelopment || $debug ]]; then
                    get_present_data
                else
                    get_present_data &
                fi
            else
                while [ -f "$link_tmp_folder/loading" ]
                do
                    # msgout "we are here inside link 2"
                    sleep 1
                done
                do_main
            fi
        else
            # msgout "we are here outside link"
            check_for_data
        fi
        if [[ ! $devtest ]]; then
            process_data &
        fi
        check_for_running_wx_process &
    fi
}


ask_config () {

    proc_ppid=(); proc_rc=(); proc_conf=(); proc_trend=()
    conkywx_internal_dir="$HOME/.config/conkywx/conkywx_internal"

    case $ca_opts in
    1)  do_msg="$f_update"  ;;
    2)  do_msg="$f_debug"   ;;
    3)  do_msg="$r_trend"
        OLDIFS=$IFS; IFS=$'\n'
            fileA2=($(find $conkywx_internal_dir/trends/forecast* -type f -printf "%f\n"))
        IFS=$OLDIFS
        ;;
    esac

    OLDIFS=$IFS; IFS=$'\n'
        id_files=($(find ${conkywx_internal_dir}/*_id -type f -printf "%f\n"))
    IFS=$OLDIFS

    for d in "${!id_files[@]}"
    do
        if [[ -s "${conkywx_internal_dir}/${id_files[$d]}" ]]; then
            OLDIFS=$IFS; IFS=$'\n'
            input_data=($(< "${conkywx_internal_dir}/${id_files[$d]}"))
            IFS=$OLDIFS
        fi
        # in_data=( ${input_data//|/ } )
        proc_ppid+=( "${input_data[0]//\"}" )
        proc_conf+=( "${input_data[1]//\"}" )
        proc_trend+=( "${input_data[2]//\"}" )
    done

    while true
    do
        clear
        printf "\n\t\t%s\n" "$ca_menu_hdg"
        printf "\t%b\n" "$ca_dash_line"
        case $ca_opts in
        3)
            printf "\t   %b %b\n" "${#fileA2[@]} $list_found_t"
            printf "\t%b\n" "$ca_dash_line"
            printf "\t%s %b\n" "$ca_file_list_t" "$do_msg"
            printf "\t%b\n" "$ca_dash_line"
            for c in "${!fileA2[@]}"
            do
                fileA2_t1="${fileA2[$c]/*trends-}"
                fileA2_t1="${fileA2_t1/-$USER}"
                if [[ $( echo ${proc_trend[@]} | grep -s "${fileA2_t1}" ) == "" ]]; then
                    printf "\t%4s   :  %b\n" $c "  ${fileA2[$c]}"
                else
                    printf "\t%4s   :  %b\n" $c "> ${fileA2[$c]}"
                fi
            done
            ;;
        1|2|4|5)
            OLDIFS="$IFS"; IFS=$'\n'
            runningProcsArray=($(ps aux | grep conky | grep -v grep))
            IFS="$OLDIFS"

            for d in "${!runningProcsArray[@]}"
            do
                local larr=($(echo "${runningProcsArray[$d]}"))
                local lppid=${larr[1]}
                local lpros=${larr[12]}
                if [[ $lpros != '-c' ]]; then
                    for c in "${!proc_ppid[@]}"
                    do
                        if [[ ${proc_ppid[$c]} == $lppid ]]; then
                            proc_conf1=${proc_conf[$c]##*/}
                            proc_conf1=${proc_conf1/\"}
                            confArr[$c]=" $c   :  > ${proc_conf1}"
                        fi
                    done
                fi
            done
            printf "\t   %b %b\n" "${#confArr[@]} $list_found"
            printf "\t%b\n" "$ca_dash_line"
            printf "\t%b %b\n" "$ca_file_list" "$do_msg"
            printf "\t%b\n" "$ca_dash_line"
            printf "\t  %4s\n" "${confArr[@]}"
            ;;
        esac

        printf "\t%b\n" "$ca_dash_line"
        printf "\t%b" "$ca_choice"
        read config_choice
        # read -p "$ca_choice" config_choice
        if [[ $config_choice == "p" ]]; then
            conkywx_assistant
            exit 0
        elif [[ $config_choice =~ ^[0-9]+$ ]]; then
            case $ca_opts in
            3)
                forecast_trends="$conkywx_internal_dir/trends/${fileA2[$config_choice]}"
                if [[ -f $forecast_trends ]]; then
                    clear
                    printf "\n\t%b\n" "${fileA2[$config_choice]}"
                    printf "\t%b" "$confirm_delete_msg"
                    read confirm_del
                    case $confirm_del in
                    YES)
                        rm -f $forecast_trends
                        conkywx_assistant
                        ;;
                    *)
                        printf "\n\t%b" "$ops_cancel_msg"
                        sleep 3
                        conkywx_assistant
                        ;;
                    esac
                fi
                ;;
            1|2|4|5)
                if [[ -f "${proc_conf[$config_choice]}" ]]; then
                    conf1=${proc_conf[$config_choice]##*/}
                    temppath="/tmp/wx-${USER}-$conf1-pChA_r_27u0-m39"
                    # if [ ! -d "$temppath" ]; then
                    #   mkdir "$temppath"
                    # fi

                    if [[ -d "${temppath}" ]]; then
                        case $ca_opts in
                        1)
                            if [[ -f "${temppath}/wxcurrdata.html" ]]; then
                                # touch "$temppath/wxforce_update"
                                msgout "$ca_conky_restart" N;
                                rm "$temppath/wxcurrdata.html" "$temppath/fct_data.html" 2> /dev/null
                                # exec "$0" && exit
                                if [[ -s $ConkyStartScript ]]; then
                                    $ConkyStartScript
                                else
                                    clear
                                    msgout "$ConkyRestartMsg"
                                fi
                                exit 0
                            else
                                msgout "$ca_tryagain_nt" N; sleep 5
                            fi
                            ;;
                        2) #set -x
                            if [[ -s "${proc_conf[$config_choice]}" ]]; then
                                touch "$temppath/dummy_debug"
                                touch '/tmp/wxdebug'
                                msgout "$ca_conky_restart" N;
                                if [[ -s $ConkyStartScript ]]; then
                                    $ConkyStartScript
                                else
                                    clear
                                    msgout "$ConkyRestartMsg"
                                fi
                                exit 0
                            else
                                msgout "$ca_conf_nf" N; sleep 3
                            fi
                            # set+x
                            ;;
                        4)
                            if [[ -f "${temppath}/graphdata" ]]; then
                                # touch "$temppath/wxforce_update"
                                msgout "$ca_conky_restart" N;
                                rm "$temppath/graphdata"
                                # exec "$0" && exit
                                if [[ -s $ConkyStartScript ]]; then
                                    $ConkyStartScript
                                else
                                    clear
                                    msgout "$ConkyRestartMsg"
                                fi
                                exit 0
                            else
                                msgout "$ca_tryagain_nt" N; sleep 5
                            fi
                            ;;
                        5)
                            dbgTemplate=$(grep "template_1" "$temppath/wxdata_check" | sed 's/.*=//')
                            case $dbgTemplate in
                            ./*)    dbgTemplate="${cwxpath}${dbgTemplate#.}" ;;
                            esac

                            if [[ -s $dbgTemplate ]]; then
                                perl -ne '
                                    $tcRegex = qr/(?<=\})([\w\.]{1}[\w\-\s\.]*)(?=\$\{|\#|::|\-)/;
                                    m/$tcRegex/g;
                                    print "$1\n" if $1 !~ /^\s*$/ and not $seen{$1}++ ;
                                ' $dbgTemplate > "$temppath/template_label_list"
                            fi
                            if [[ -s "$temppath/template_label_list" ]]; then
                                msgout "\n\t${templateMsg1}\n\t$temppath/template_label_list\n\n\t${templateMsg2}"
                            else
                                msgout "\n\tError :( Not able to create file:\n\t$temppath/template_label_list\n\n"
                            fi
                            exit 0
                        esac
                    else
                        msgout "$ca_tryagain_nt" N; sleep 5
                        conkywx_assistant
                    fi
                fi
                ;;
            esac
        elif [[ $config_choice == "q" ]]; then
            printf "\n\t%b\n\n" "$ca_bye_msg"; exit 0
        fi
    done
}

conkywx_assistant () {

    while true
    do
        clear
        printf "\n\t%b" "$ca_menu"
        read qtype

        case $qtype in
        1)  clear; msgout "$usage_msg"; exit 0                              ;;
        2)  xdg-open "$cwxpath/help/conkywx.html" >/dev/null 2>&1; exit 0   ;;
        3)  # force update
            ca_opts=1; ask_config; exit 0                                   ;;
        4)  # Debug
            ca_opts=2; ask_config; exit 0                                   ;;
        5)  # Reset forecast
            ca_opts=3; ask_config; exit 0                                   ;;
        6)  # reset graphdata
            ca_opts=4; ask_config; exit 0                                   ;;
        7)  # View Template Labels
            ca_opts=5; ask_config; exit 0                                   ;;
        q)  printf "\n\t%b\n\n" "$ca_bye_msg"; exit 0                       ;;
        esac

    done
}


# -----------------------------------------------------------------------
# Lets do the do - command line options
# -----------------------------------------------------------------------

nowtime1=$(date "+%s")

# msgout "$nowtime1"

if [[ -s $HOME/.config/conkywx/conkywx.conf ]]; then
    Default_config_file="$HOME/.config/conkywx/conkywx.conf"
elif [[ -s $cwxpath/conkywx.conf ]]; then
    Default_config_file="$cwxpath/conkywx.conf"
fi

# test for correct number of arguments and get values
if [ $# -eq 0 ]; then
    conkywx_assistant; exit 0
elif [ $# -gt 6 ]; then
    msgout "$usage_msg" ; msgout "$too_many_args\n"; exit 1
else
    while [ $# -gt 0 ]
    do
        case "$1" in
        -h)             conkywx_assistant; exit 0                                       ;;
        --help)         xdg-open "$cwxpath/help/conkywx.html" >/dev/null 2>&1; exit 0   ;;
        -v|--version)   printf "%s\n" "$version"; exit 0                                ;;
        -c|--config)    shift; config_file_new="$1"                                     ;;
        # -e|--extra)       shift; config_copy="$1"                                         ;;
        -t|--template)  shift;
                        if [[ $1 ]]; then
                            input_template="$1";
                            if [[ ! -s $input_template ]]; then
                                msgout "$missing_template_msg" ; exit 1
                            fi
                        else
                            msgout "$missing_template_msg" ; exit 1
                        fi
                        clopt1="-t"                                                     ;;
        -d|--data)      shift; input_element="$1"; clopt1="-d"
                        if [[ ! $config_file_new ]]; then
                            config_file_new="$Default_config_file"
                        fi                                                              ;;
        *)              msgout "$usage_msg"; msgout "$invalid_args\n"; exit 1           ;;
        esac
        shift
    done

    config_file=${config_file_new:-$Default_config_file}

    # darr=($(date "+%d %I:%M %P"))
    darr=($(date "+%d %I:%M:%S %P"))
    case ${darr[0]} in
    1|21|31)    pst="st" ;;
    2|22)       pst="nd" ;;
    3|23)       pst="rd" ;;
    *)          pst="th" ;;
    esac
    dstr="on ${darr[0]}$pst at ${darr[1]} ${darr[2]}"
    if [[ $clopt1 != "-d" ]]; then
        # set_config
        TIMEFORMAT="$prog_time_taken %3lR $dstr"
        time do_main
        # do_main
    else
        # set_config
        do_main
    fi

fi

# need this line for propah handling of "$link_tmp_folder" thing
if [[ -f "$temppath/loading" ]]; then rm "$temppath/loading"; fi

exit 0

: '
-----------------------------------------------
TODO:

'
