#!/usr/bin/perl

=comments
    -----------------------------------------------------------------------
    Copyright GPL v3 - Paramvir Likhari 2012/2018
    -----------------------------------------------------------------------
=cut

my $version   = $ENV{version} ;
my $progSince = '2012/2018';

use strict;
use warnings;

sub msgout  { print STDERR "@_ \n" }
sub msgoutn { print STDERR "@_"    }

if (! defined $ARGV[0]) { die "Error :( no command line arguments found $!" }
my ( $temppath, $input_template, $cwxpath, $input_element, $config_file, $forecast_trends, $msgs_file, $clopt1, $default_wx, $default_fct, $accuUnitWx, $accuUnitFct, $solValue ) = split(/\|/,"$ARGV[0]" );

use Encode;
use FindBin;
use lib "$FindBin::RealBin";
use wxDate::wxDate;
use fixConfig;

# get the current date time information
my ($sec,$min,$hour,$day,$mon,$year,$yday,$monAbb,$monFull,$wdayAbb,$wdayFull,$nowtime) =
    split(/:/,time2str("%S:%M:%H:%-e:%m:%Y:%j:%b:%B:%a:%A:%s"));
my $ltime    = "$wdayAbb $monAbb $day ${hour}:${min}:${sec} ${year}";
my $wx_tzone = tz_offset();

my $HOME = $ENV{"HOME"};
my $config_dir_path      = "$HOME/.config/conkywx";
my $conkywx_internal_dir = "$config_dir_path/conkywx_internal";
# my $system_config_file   = "$config_dir_path/system.conf";

my ($tempname) = $temppath =~ m{/tmp/(.*?)$};

my $die_error       = "$temppath/die_error";
my $currdatam       = "$temppath/wxcurrdata.html" ;
my $currdatax       = "$temppath/wxcurrdata_2.xml";
my $temp_template   = "$temppath/wxtemp_template";
my $temp_templatesc = "$temppath/wxtemp_templateSC";
my $wund_debug_log  = "$temppath/wxwund_debug.log";
my $stalexml        = "$temppath/stalexml";
my $gdacsData       = "$temppath/gdacsData";
my $newsData        = "$temppath/newsData";

# locally global variables
my ( %conf, %Mag_2day, %Mag_Nday, %TodayInfo, %output, @trends_out, %fctinfo, %LMsg,
    $debug, $iwxpath, $iwxpathfcast, $iwindpath, %langx, %wx, %wxtrend, $xmlin, );

my %wxsite = (
    '1' => 'intel',
    '2' => 'wx',
    '3' => 'msn',
    'w' => 'wund',
    'm' => 'metar',
    'a' => 'accu',
    'd' => 'darkSky',
    );

my $wxMainTitle = $wxsite{$default_wx} ;

my $wxFctTitle  = $wxsite{$default_fct} ;

my %timeFormat = (
   '1' => '%I:%M %p' ,
   '2' => '%I:%M %P' ,
   '3' => '%I:%M'    ,
   '4' => '%-I:%M%P' ,
   '5' => '%R'       ,
    );

if ( -e $stalexml ) {
    $wx{alertdata} .= "Using only Main Data - pws data is not reliable as it is stale. Change xml_ID and xml_ID2 in conkywx config file.";
    unlink $stalexml
}

### constants
my ( %gkey, @naK, %naKeys, %dateSuffix, @suffix, $asciiMapKeyList, %asciiMap,
    %sm_m, %fd_d, %m_sm, %d_fd, %text_ref, %num_ref_main, %num_ref_l_check_main1,
    %num_ref_l_check_main2, %naList, @wxstate, @rainCond );

CONSTANTS: {

    @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);

    %text_ref = qw( Location_Name 1 Long1 1 NowImage 1 Lat1 1 Updated 1 Station_ID 1 Station_Name 1 curr_icon 1 condition 1 METAR 1 station_id 1 last_update_xml 1 Wind_From 1 wind_dir 1 CLC1 1 CLC2 1 CLC3 1 CLC4 1);

    %num_ref_main = qw(Rainfall 1 Wind_Speed 1 Wind_Gust 1 Pressure 1 Visibility 1 CLH1 1 CLH2 1 CLH3 1 CLH4 1 );

    %num_ref_l_check_main1 = qw( Temp_now 1 Dew_Point 1 Humidity 1);

    %num_ref_l_check_main2 = qw( Temp_now 1 Dew_Point 1 );

    %naList = qw(-999.00 1 -999.0 1 -999 1 -9999.00 1 -9999.0 1 -9999 1 N/A 1 NA 1 null 1 "" 1 -- 1 CALM 1 Calm 1 calm 1 ) ;

    @wxstate = (
        [ 'thunderstorm',   'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 'thunder',        'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 'tstorm',         'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 't-storm',        'tstorm',  'n', 'n', 'thunderstorms'  ],
        [ 'tornado',        'tstorm',  'n', 'n', 'tornado'        ],
        [ 'hurricane',      'tstorm',  'n', 'n', 'hurricane'      ],
        [ 'tropical storm', 'tstorm',  'n', 'n', 'tropical storm' ],
        [ 'blizzard',       'sleet',   'x', 'x', 'blizzard'       ],
        [ 'mixed',          'sleet',   'x', 'x', 'sleet'          ],
        [ 'hail',           'sleet',   'x', 'x', 'hail'           ],
        [ 'sleet',          'sleet',   'x', 'x', 'sleet'          ],
        [ 'flurr',          'sleet',   'x', 'x', 'flurries'       ],
        [ 'snow',           'snow',    'q', 'q', 'snow'           ],
        [ 'frig',           'pcloudy', 'x', 'x', 'very cold'      ],
        [ 'very cold',      'pcloudy', 'x', 'x', 'very cold'      ],
        [ 'spray',          'rain',    'i', 'i', 'spray'          ],
        [ 'squall',         'rain',    'i', 'i', 'squalls'        ],
        [ 'rain',           'rain',    'i', 'i', 'rain'           ],
        [ 'precip',         'rain',    'i', 'i', 'rain'           ],
        [ 'shower',         'rain',    'i', 'i', 'showers'        ],
        [ 'downpour',       'rain',    'i', 'i', 'rain'           ],
        [ 'drizzle',        'rain',    'i', 'i', 'drizzle'        ],
        [ 'fog',            'fog',     '9', '9', 'fog'            ],
        [ 'mist',           'fog',     '9', '9', 'mist'           ],
        [ 'sandstorm',      'dust',    '7', '7', 'sandstorm'      ],
        [ 'duststorm',      'dust',    '7', '7', 'duststorm'      ],
        [ 'ash',            'dust',    '7', '7', 'ash'            ],
        [ 'dust',           'dust',    '7', '7', 'dust'           ],
        [ 'sand',           'dust',    '7', '7', 'sand'           ],
        [ 'haz',            'hazy',    '9', '9', 'hazy'           ],
        [ 'smok',           'hazy',    '9', '9', 'smoke'          ],
        [ 'blust',          'windy',   '6', '6', 'wind gusts'     ],
        [ 'wind',           'windy',   '6', '6', 'windy'          ],
        [ 'partly cloudy',  'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'pcloudy',        'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'p cloudy',       'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'p cldy',         'pcloudy', 'c', 'C', 'partly cloudy'  ],
        [ 'mostly sunny',   'fair',    'b', 'B', 'clear'          ],
        [ 'msunny',         'fair',    'b', 'B', 'clear'          ],
        [ 'mostly clear',   'pcloudy', 'c', 'C', 'mostly clear'   ],
        [ 'mclear',         'pcloudy', 'c', 'C', 'mostly clear'   ],
        [ 'psunny',         'pcloudy', 'd', 'D', 'partly cloudy'  ],
        [ 'partly sunny',   'pcloudy', 'd', 'D', 'partly cloudy'  ],
        [ 'mcloudy',        'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'm cloudy',       'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'm cldy',         'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'mostly cloudy',  'mcloudy', 'd', 'D', 'mostly cloudy'  ],
        [ 'sun',            'clear',   'a', 'A', 'clear'          ],
        [ 'clear',          'clear',   'a', 'A', 'clear'          ],
        [ 'pleasant',       'clear',   'a', 'A', 'clear'          ],
        [ 'breezy',         'clear',   'a', 'A', 'clear'          ],
        [ 'wundy',          'clear',   'a', 'A', 'clear'          ],
        [ 'fine',           'clear',   'a', 'A', 'clear'          ],
        [ 'nice',           'clear',   'a', 'A', 'clear'          ],
        [ 'dry',            'clear',   'a', 'A', 'clear'          ],
        [ 'few',            'fair',    'b', 'B', 'clear'          ],
        [ 'fair',           'fair',    'b', 'B', 'clear'          ],
        [ 'cloud cover',    'cloudy',  'f', 'f', 'overcast'       ],
        [ 'cloudy',         'cloudy',  'f', 'f', 'overcast'       ],
        [ 'cldy',           'cloudy',  'f', 'f', 'overcast'       ],
        [ 'overcast',       'cloudy',  'f', 'f', 'overcast'       ],
        [ 'funnel cloud',   'cloudy',  'f', 'f', 'overcast'       ],
        [ 'considerable cloud',   'cloudy',  'f', 'f', 'overcast' ],
        [ 'cloud',          'pcloudy', 'd', 'D', 'partly cloudy'  ],
        [ 'ice',            'sleet',   'x', 'x', 'ice'            ],
        [ 'unknown',        'na',      '-', '-', 'na'             ],
        [ 'n/a',            'na',      '-', '-', 'na'             ],
        [ 'na',             'na',      '-', '-', 'na'             ],
        [ 'unknown precipitation','rain','i','i','rain' ],
    );

    %fd_d = (
        Today     => "Tod", Tonight   => "Ton", Monday    => "Mon",
        Tuesday   => "Tue", Wednesday => "Wed", Thursday  => "Thu",
        Friday    => "Fri", Saturday  => "Sat", Sunday    => "Sun",
        );
    %d_fd = reverse %fd_d ;

    %m_sm = (
        January   => "Jan", February  => "Feb", March     => "Mar",
        April     => "Apr", May       => "May", June      => "Jun",
        July      => "Jul", August    => "Aug", September => "Sep",
        October   => "Oct", November  => "Nov", December  => "Dec",
        );
    %sm_m = reverse %m_sm;

    %asciiMap = (
            quot   => '"', apos   => "'", amp    => '&', lt     => '<', gt     => '>',
            nbsp   => '',  iexcl  => '¡', cent   => '¢', pound  => '£', curren => '¤',
            yen    => '¥', brvbar => '¦', sect   => '§', uml    => '¨', copy   => '©',
            ordf   => 'ª', laquo  => '«', not    => '¬', shy    => '­',  reg    => '®',
            macr   => '¯', deg    => '°', plusmn => '±', sup2   => '²', sup3   => '³',
            acute  => '´', micro  => 'µ', para   => '¶', middot => '·', cedil  => '¸',
            sup1   => '¹', ordm   => 'º', raquo  => '»', frac14 => '¼', frac12 => '½',
            frac34 => '¾', iquest => '¿', times  => '×', divide => '÷', Agrave => 'À',
            Aacute => 'Á', Acirc  => 'Â', Atilde => 'Ã', Auml   => 'Ä', Aring  => 'Å',
            AElig  => 'Æ', Ccedil => 'Ç', Egrave => 'È', Eacute => 'É', Ecirc  => 'Ê',
            Euml   => 'Ë', Igrave => 'Ì', Iacute => 'Í', Icirc  => 'Î', Iuml   => 'Ï',
            ETH    => 'Ð', Ntilde => 'Ñ', Ograve => 'Ò', Oacute => 'Ó', Ocirc  => 'Ô',
            Otilde => 'Õ', Ouml   => 'Ö', Oslash => 'Ø', Ugrave => 'Ù', Uacute => 'Ú',
            Ucirc  => 'Û', Uuml   => 'Ü', Yacute => 'Ý', THORN  => 'Þ', szlig  => 'ß',
            agrave => 'à', aacute => 'á', acirc  => 'â', atilde => 'ã', auml   => 'ä',
            aring  => 'å', aelig  => 'æ', ccedil => 'ç', egrave => 'è', eacute => 'é',
            ecirc  => 'ê', euml   => 'ë', igrave => 'ì', iacute => 'í', icirc  => 'î',
            iuml   => 'ï', eth    => 'ð', ntilde => 'ñ', ograve => 'ò', oacute => 'ó',
            ocirc  => 'ô', otilde => 'õ', ouml   => 'ö', oslash => 'ø', ugrave => 'ù',
            uacute => 'ú', ucirc  => 'û', uuml   => 'ü', yacute => 'ý', thorn  => 'þ',
            yuml   => 'ÿ',
        );
    $asciiMapKeyList = join("|", keys %asciiMap) ;

    @suffix = (qw(th st nd rd th th th th th th)) x 4;
    @suffix[11,12,13] = qw(th th th);
    %dateSuffix = map { $_ => $suffix[$_] } 0..$#suffix;

    %gkey =  ( D1D=>1, D2D=>1, D3D=>1, D4D=>1, D5D=>1, D6D=>1, D7D=>1, D8D=>1, D9D=>1, D10D=>1 );

    @naK = qw( PTBI PIF PIC PT FL HM BF CT LF CLH1 CLH2 CLH3 CLH4 BI UVM METAR SR SS LOV LOD LODN LODNT CTR CTS NTR NTS ATR ATS SMT MMT SDC MDC MIF MR MS MA MV MROC MROT MIC LU D1F D1I D1T D2F D2I D2T D3F D3I D3T D4F D4I D4T D5F D5I D5T D6F D6I D6T D7F D7I D7T D8F D8I D8T D9F D9I D9T D10F D10I D10T FCTPF FCTTF FCTWF FCTHF FCTUF FCTDF FCTVF FCTSRF FCTSSF FCTMRF FCTMSF FCTLLF FCTLDF FCTMIF FCTTYF GT GP GD GH GW GG GS UU TMP SOL VI BR DP );

    if ( $wxFctTitle ne "accu" ) {
        @naK = ( @naK, qw( D1P D2P D3P D4P D5P D6P D7P D8P D9P D10P ));
    }
    %naKeys = map {  $_ => undef  } @naK ;
}

if ( -e "${temppath}/dummy_debug" ) { $debug = 1 }
else { $debug = 0 }

# Astronomy module - figure out if machine is 64 or 32 bit
my $cwx_magellan;
if ( -e "${cwxpath}/lib/magellan" ) {
    $cwx_magellan = "${cwxpath}/lib/magellan";
}
else {
    my $machine_is = `uname -m`;
    chomp($machine_is);
    $cwx_magellan = (index($machine_is,'x86_64') != -1 ) ? "${cwxpath}/lib/magellan_64" :
    "${cwxpath}/lib/magellan_32";
}


sub do_debug {
    my ( $msg, @datawx1 ) = @_ ;

    open (out_data, '>>', "$wund_debug_log") or die "$LMsg{no_open_file} wund_debug_log: $!";
        print out_data "************* $msg data *************\n";
        print out_data @datawx1;
        print out_data "*****************************************************************\n";
    close (out_data);

}

sub read_msgs {

    if ( -s $msgs_file ) {

        %LMsg = &fixConf($msgs_file, 1);

        if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }
    }
}
&read_msgs;


sub sub_name { ( caller(1) )[3], $LMsg{dbg_msg_str2}, ( caller(1) )[2] ;}

sub read_config {

    if ( -s "$config_file" ) {

        %conf = &fixConf($config_file, 1);

        # Units_downloaded now hardcoded in conkywx.sh as well
        $conf{Units_downloaded}    = "IM"            ;
        # define some defaults
        $conf{today_date_format}   ||= '%A %b %d %Y' ;
        $conf{minimum_fetch_time}  ||= 900           ;
        $conf{Time_format}         ||= '%R'          ;
        $conf{show_degsym_fc_temp} ||= 5             ;
        $conf{moon_outline}        ||= 0             ;
        $conf{moon_time}           ||= '%b %d'       ;
        $conf{use_short_date}      ||= 0             ;

        # wx icons defined -----------------------------------------
        $conf{wx_iconset} ||= 2;
        my ($wp1, $wp2, $wp3, $wp4);
        $wp1 = ($conf{wx_iconset} =~ /^[1-9]$/) ? $conf{wx_iconset} : 0;

        if ( $wp1 ) {
            $iwxpath = "$cwxpath/images/wxicons-$wp1";
        }
        elsif ( -e $conf{wx_iconset} or die $LMsg{no_wx_iconset_path}) {
            $iwxpath = $conf{wx_iconset};
        }

        if ( $conf{wx_fcast_iconset} ) {
            $wp1 = ($conf{wx_fcast_iconset} =~ /^[1-9]$/) ? $conf{wx_fcast_iconset} : 0;
            if ( $wp1 ) {
                $iwxpathfcast = "$cwxpath/images/wxicons-$wp1";
            }
            elsif ( -e "$conf{wx_fcast_iconset}" or die $LMsg{no_wx_iconset_path}) {
                $iwxpathfcast = "$conf{wx_fcast_iconset}";
            }
        }
        else {
            $iwxpathfcast = $iwxpath ;
        }

        # wind icons defined -----------------------------------------
        $conf{wx_wind_iconset} ||= 1;
        $wp1 = ($conf{wx_wind_iconset} =~ /^[1-3]$/) ? $conf{wx_wind_iconset} : 0;

        if ( $wp1 ) {
            $iwindpath = "$cwxpath/images/windrose-$wp1";
        }
        elsif ( -e "$conf{wx_wind_iconset}" or die $LMsg{no_wx_iconset_path}) {
            $iwindpath = "$conf{wx_wind_iconset}";
        }

        $conf{wx_wind_fontset} ||= 2;

        # if ( $wxMainTitle eq 'accu' or $wxFctTitle eq 'accu' ) {
        # if ( $accuUnitWx eq 'accu' or $wxFctTitle eq 'accu' ) {
        #     # undef $conf{Units_to_Use}
        #     $conf{Units_to_Use} = 'MI'
        # }

        ### Note: Units_to_Use is undefined if 'SI' - so only check for defined or not
        $conf{Units_to_Use} ||= 'IM';
        $conf{Units_to_Use} = uc($conf{Units_to_Use});
        my %unitsUse = ( 'IM' => 1, 'MI' => 1 );
        if ($unitsUse{$conf{Units_to_Use}}) { $conf{Units_to_Use} = 'IM' }
        else { undef $conf{Units_to_Use} }


        # ### print debug data
        # if ( $debug or $ENV{debug} ) {
        #     msgout( $LMsg{dbg_msg_str1}, sub_name() );
        #     do_debug( "do_cli_opts dump", %conf );
        # }
    }
}
&read_config;

# -----------------------------------------------------------------------
# show pop-up notification
# -----------------------------------------------------------------------
sub show_noti {
    my $msg = shift;
    my $opt = shift or undef;
    if ( $conf{show_notification} or $opt ) {
        my $show_duration = ( $opt ) ? 50000 : 5000 ;
        `/usr/bin/notify-send "$msg" -i "$cwxpath/images/wxicons-2/clear.png" -t $show_duration 2> /dev/null`
    }
}


sub die_errors {
    open (out_data, '>', $die_error ) or die "$LMsg{no_open_file} die_error : $!";
        print out_data "@_ \n";
    close (out_data);
}

# -----------------------------------------------------------------------
# conversions
# -----------------------------------------------------------------------
# temperature
sub c_f     { return (( $_[0] * 1.8 ) + 32  ) }
sub f_c     { return (( $_[0] - 32 ) / 1.8  ) }
# distance - speed km/h to mi/h is same as km to mi
sub km_mi   { return (  $_[0] * 0.62137119  ) }
sub mi_km   { return (  $_[0] / 0.62137119  ) }
sub mm_in   { return (  $_[0] * 0.039370078 ) }
sub in_mm   { return (  $_[0] / 0.039370078 ) }
sub ft_mtr  { return (  $_[0] * 0.3048      ) }
sub mtr_ft  { return (  $_[0] / 0.3048      ) }
sub kt_mi   { return (  $_[0] * 1.1507794   ) }
# pressure
sub hPa_in  { return (  $_[0] * 0.02953     ) }
sub in_hPa  { return (  $_[0] / 0.02953     ) }
sub unity   { return (  $_[0]               ) }


my ( $d, $u_temp, $u_speed, $u_dist_big, %fn, $u_dist_mid, $u_dist_small, $u_pressure, $lua_wx_units, %un, );

if ( $conf{Units_to_Use} ) {
    %fn = (
        f_temp       => \&c_f,
        f_speed      => \&km_mi,
        f_dist_big   => \&km_mi,
        f_dist_mid   => \&mtr_ft,
        f_dist_small => \&mm_in,
        f_pressure   => \&hPa_in,
    );
    $d               = '%1.2f';
    $lua_wx_units    = 'F';
}
else {
    %fn = (
        f_temp       => \&f_c,
        f_speed      => \&mi_km,
        f_dist_big   => \&mi_km,
        f_dist_mid   => \&ft_mtr,
        f_dist_small => \&in_mm,
        f_pressure   => \&in_hPa,
    );
    $d               = '%1.0f';
    $lua_wx_units    = 'C';
}

if ($conf{Units_show}) {
    if ( $conf{Units_to_Use} ) {
        $u_temp       = '°F';
        $u_speed      = 'mph';
        $u_dist_big   = 'mi';
        $u_dist_mid   = 'ft';
        $u_dist_small = 'in';
        $u_pressure   = 'in';
    }
    else {
        $u_temp       = '°C';
        $u_speed      = 'km/h';
        $u_dist_big   = 'km';
        $u_dist_mid   = 'm';
        $u_dist_small = 'mm';
        $u_pressure   = 'hPa';
    }
}
else {
    $u_temp           = '°';
    $u_speed          = '';
    $u_dist_big       = '';
    $u_dist_mid       = '';
    $u_dist_small     = '';
    $u_pressure       = '';
}

sub wx_limits {
    ### $_[0] is value $_[1] is unit
    if    ($_[1] eq 'Temp_now'  and $_[0] < -130 and $_[0] > 140) {
        return( 'NA' )
    }
    elsif ($_[1] eq 'Dew_Point' and $_[0] < -100 and $_[0] > 100) {
        return( 'NA' )
    }
    elsif ($_[1] eq 'Humidity'  and $_[0] <  0   and $_[0] > 100) {
        return( 'NA' )
    }
    else {
        return($_[0])
    }
}


sub trim { $_[0] =~ s/^\s+//; $_[0] =~ s/\s+$//; return $_[0] }

sub round {
    my $intval = $_[0] - int($_[0]);
    return ( $intval >= 0.5 ) ? int( $_[0] + int(($intval)+0.5) ) : int($_[0])
}

my @fwd = qw( n nne ne ene e ese se sse s ssw sw wsw w wnw nw nnw north northnortheast northeast eastnortheast east eastsoutheast southeast southsoutheast south southsouthwest southwest westsouthwest west westnorthwest northwest northnorthwest );
my @dwf = qw(North NNE NE ENE East ESE SE SSE South SSW SW WSW West WNW NW NNW North);

my %convUnits = (
    'Depth'      => [ $fn{f_dist_big}  , "%1.1f $u_dist_big"   ],
    'Temp_now'   => [ $fn{f_temp}      , "%1.0f$u_temp"        ],
    'Dew_Point'  => [ $fn{f_temp}      , "%1.0f$u_temp"        ],
    'Wind_Speed' => [ $fn{f_speed}     , "%1.1f $u_speed"      ],
    'Wind_Gust'  => [ $fn{f_speed}     , "%1.1f $u_speed"      ],
    'Pressure'   => [ $fn{f_pressure}  , "$d $u_pressure"      ],
    'Visibility' => [ $fn{f_dist_big}  , "%1.1f $u_dist_big"   ],
    'Rainfall'   => [ $fn{f_dist_small}, "%1.1f $u_dist_small" ],
    'CLH1'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'CLH2'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'CLH3'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'CLH4'       => [ $fn{f_dist_mid}  , "%1.0f $u_dist_mid"   ],
    'Tide'       => [ $fn{f_dist_mid}  , "%5.2f $u_dist_mid"   ],
);


# -----------------------------------------------------------------------
# convert wind in degrees to cardinals and inter-cardinals
# -----------------------------------------------------------------------
sub find_wind_dir {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    return $_[0] if $_[0] eq 'NA' ;

    $_[0] = lc $_[0];

    if ( $_[0] =~ m{([A-Za-z]+)} ) {
        my $findIdx = sub { 1 while $_[0] ne pop; $#_ };
        my $fidx = &$findIdx($1,@fwd);
        $fidx = ($fidx > 15) ? $fidx - 16 : ($fidx == -1) ? 0 : $fidx ;
        return ($fidx * 22.5, $dwf[$fidx])
    }
    else {
        return $dwf[sprintf("%.0f", ($_[0] % 360)/22.5)]
    }
    return $_[0]
}

# -----------------------------------------------------------------------
# check if value is number including float -ve etc
# -----------------------------------------------------------------------
sub check_num {

    # %text_ref = qw( Location_Name 1 Long1 1 NowImage 1 Lat1 1 Updated 1 Station_ID 1 Station_Name 1 curr_icon 1 condition 1 METAR 1 station_id 1 last_update_xml 1 Wind_From 1 wind_dir 1 CLC1 1 CLC2 1 CLC3 1 CLC4 1);
    # %num_ref_main = qw(Rainfall 1 Wind_Speed 1 Wind_Gust 1 Pressure 1 Visibility 1 CLH1 1 CLH2 1 CLH3 1 CLH4 1 );
    # %num_ref_l_check_main1 = qw( Temp_now 1 Dew_Point 1 Humidity 1);
    # %num_ref_l_check_main2 = qw( Temp_now 1 Dew_Point 1 );
    # %naList = qw(-999.00 1 -999.0 1 -999 1 -9999.00 1 -9999.0 1 -9999 1 N/A 1 NA 1 "" 1 -- 1) ;

    my ($mynum, $otext) = @_;
    my ( $oval );
    if ( ! $mynum ) { $mynum = 0 }
    if ( ! $otext ) { $otext = "NA" }
    # chomp($mynum);
    $mynum = trim($mynum);
    $mynum =~ tr/\x0a\x22\x27//d;
    # $mynum =~ tr/\x0a\x20\x22\x27//d;
    my $tNum = $mynum;

    if ( exists $naList{$mynum} ) { $oval = "NA" }
    elsif (lc($mynum) eq "calm" ) { $oval = "Calm" }
    else {
        if    ( exists $text_ref{$otext}    ) { $oval = $mynum }
        elsif ( $otext eq 'Clouds'   ) { $oval = ( $mynum eq "" ) ? "NA" : $mynum }
        elsif ( $otext eq 'Gmt_Time' ) { $oval = fix_tzone($mynum) }
        else {
            $tNum =~ tr/0-9.-//cd;
            # msgout($tNum);
            if ( $tNum ne '' ) {
                $mynum =~ tr/\x20//d;
                if ( exists $num_ref_main{$otext} ) {
                    $oval = ($mynum < 0) ? 'NA' : convert_unit($mynum, $otext)
                }
                elsif ( exists $num_ref_l_check_main1{$otext} ) {
                    $oval = (wx_limits($mynum,$otext) eq 'NA')
                    ? 'NA'
                    : exists $num_ref_l_check_main2{$otext}
                    ? convert_unit($mynum, $otext)
                    : $mynum;
                }
                else { $oval = "$mynum" }
            }
            else { $oval = "NA" }
        }
    }
    return ($oval);
}

sub fix_tzone { my $zone = shift ; return tz_offset($zone)/3600; }

sub normalize {

    my @arr002 = @_;
    my ( @arr001, $nval, $intval, $min, $max);
    my $fval = "";

    # control limits
    my $a1 = 0;
    my $b1 = 5;

    for (@arr002) {
        $_ = ( $naList{$_} or $_ == 0 ) ? 0.1 : $_ ;
        $min = $_ if !$min || $_ < $min;
        $max = $_ if !$max || $_ > $max;
    }
    for (@arr002) {
        if ( $max != $min ) {
            $nval = $a1+($_-$min)*(($b1-$a1)/($max-$min));
        }
        else {
            $nval = $a1+($_-$min)*(($b1-$a1)/1);
        }
        $intval = $nval - int($nval);
        push @arr001, ($intval>=0.5) ? int($nval + (($intval)+0.5)) : int($nval) ;
    }
    # msgout  "@arr002";
    # msgout  "@arr002\n@arr001 = $min | $max";
    return join ',', @arr001 ;
}


sub do_get_DI {

    # if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $opt1, $cond2, $cimage, $daynite ) = @_ ;

    if ( not $cond2 ) { return '' }

    # my $trickWords = 'nice';

    $daynite ||= 2 ; # 2 is daylight time

    if ( $opt1 ) {

        # msgout($opt1,$cond2);
        chomp($cond2);
        my $cond1 = $cond2;
        if ( $opt1 != 3 ) {
            ( $cond1 ) = (lc($cond2) =~ m{^(?:nt_)?(.*?)$}) ? $1 : 'NA'
        }

        my ( $imageout, $fday, $fnit, $cnt11, $fnd, $notFound ) ;

        foreach my $ky ( @wxstate ) {
            if ( index( $cond1, @$ky[0] ) != -1 ) {
                msgout("Original Cond: $cimage ${cond1}") if $debug;
                ($cond1, $imageout,$fday,$fnit) = (@$ky[4],@$ky[1],@$ky[2],@$ky[3]);
                msgout("wx Cond      : ${cond1}") if $debug;
                last
            }
        }
        if ( not $imageout ) { $imageout = "na"; $fday = "-"; $fnit = "-" }

        # make conditions title case
        $cond1 = join '', map { ucfirst lc } split /(\W+)/, $cond1;
        # fix clouds thing
        if ( $opt1 == 3 ) { return $cond1 }

        if ( $opt1 == 1 ) { $wx{uvwx} = $imageout }
        my ($dn_font);
        if ( $opt1 == 1 and $daynite == 1 ) {
            if ( $conf{wx_iconset} == 5 ) {
                if ( $imageout !~ /na|up|tstorm/ ) {
                    $imageout = "nt_" . ${imageout}
                }
            }
            else {
                if ( $imageout =~ /fair|mcloudy|pcloudy|clear/ ) {
                    $imageout = "nt_" . ${imageout}
                }
            }
            $dn_font  = $fnit ;
        }
        elsif ( $opt1 == 1 and $daynite == 2 ) { $dn_font = $fday }

        # copy present condition font and image
        if ( $opt1 == 1 ) {
            $TodayInfo{NowImage} = "${iwxpath}/${imageout}.png";
            return ($dn_font, $cond1)
        }
        # copy daily images and fonts are returned to calling sub
        if ( $opt1 == 2 ) {
            $fctinfo{"d_" . ${cimage} . "_cond"} = $cond1;
            $fctinfo{"d_" . ${cimage} . "_image" } = "${iwxpathfcast}/${imageout}.png" ;
            return $fday
        }
    }
}


sub metar_sky {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($inval, $otr1) = @_ ;
    # - Light
    # + Heavy
    my %msky = (
        BC     => "Patches",          BL  => "Blowing",       BR   => "Mist",
        DR     => "Low Drifting",     DS  => "Duststorm",     DU   => "Dust",
        DZ     => "Drizzle",          FC  => "Funnel Cloud",  FG   => "Fog",
        FU     => "Smoke",            FZ  => "Freezing",      GR   => "Hail",
        GS     => "Snow Pellets",     HZ  => "Haze",          IC   => "Ice Crystals",
        MI     => "Shallow",          NA  => "NA",            PL   => "Ice Pellets",
        PO     => "Dust/Sand Whirls", PR  => "Partial",       PY   => "Spray",
        RA     => "Rain",             SA  => "Sand",          SG   => "Snow Grains",
        SH     => "Showers",          SN  => "Snow",          SQ   => "Squalls",
        SS     => "Sandstorm",        TS  => "Thunderstorm",  UP   => "Unknown Precipitation",
        VA     => "Volcanic Ash",     VC  => "Nearby",        OVC  => "Overcast",
        BKN    => "Mostly Cloudy",    SCT => "Partly Cloudy", FEW  => "Clear",
        CLR    => "Clear",            SKC => "Clear",         CAVU => "Clear",
        CAVOK  => "Clear",
    );

    $otr1  ||= 0  ;
    if ( $inval eq '' ) { return $inval }

    if ( $otr1 == 1 ) {
        return( $msky{$inval} ? $msky{$inval} : "Unknown :(" );
    }
    else {
        my ( @metarr, @mwxcond, $mwxclen ) ;
        $mwxclen = length($inval);

        my $intensity = substr( $inval, 0, 1 ) ;
        if    ( $intensity eq '-' ) { $intensity = "Light";  $mwxclen -= 1  }
        elsif ( $intensity eq '+' ) { $intensity = "Heavy";  $mwxclen -= 1  }
        else                        { $intensity = ""                       }

        if ( $inval =~ /\s+/ ) {
            $inval = substr( $inval, -${mwxclen} ) ;
            @metarr = split(/\s+/, $inval) ;
            for (@metarr) { push( @mwxcond , $msky{$_} ) }
        }
        elsif ( $mwxclen > 2 ) {
            # $inval =~ tr/-//d;
            # $inval =~ tr/+//d;
            $inval =~ tr/\x2b\x2d//d;
            @metarr = unpack '(a2)*', $inval;
            for (@metarr) { push( @mwxcond , $msky{$_} ) if (length($_) == 2) }
        }
        else {
            $inval = substr( $inval, -${mwxclen} ) ;
            push( @mwxcond , $msky{$inval} )
        }

        my $mxr = join( " ", @mwxcond ) ;

        if ( $intensity ) { return $intensity. " " . $mxr ; }
        else { return $mxr ; }
    }
}

# sub custom_sort {
#   my ($x, $y, @keys) = @_;
#   for (@keys) {
#       my $cmp = $x->{$_} cmp $y->{$_};
#       return $cmp if $cmp;
#   }
#   return 0;
# }

sub seismic {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my $seismic_dataA = "$temppath/seismic_dataA";
    my $seismic_dataB = "$temppath/seismic_dataB";

    if ( -e $seismic_dataA and -e $seismic_dataB ) {

        my ( $ineqfileA, $ineqfileB, %eq, %eqB, @eqsetA, %seen, $cntr, $cntr2, $cntr3, @sorted2,%hdr,%ftr,%mid,%top, );

        ## USGS data
        {
        open(FILE, "<:crlf", "$seismic_dataB") or die "$LMsg{no_open_file} seismic_dataB data file $!";
            local $/ = undef;
            $ineqfileB = <FILE> ;
        close(FILE);
        }
        $ineqfileB =~ s/time,lat.*\n//;
        $ineqfileB =~ s/\.\d+Z//g;
        foreach (split('\n',$ineqfileB)) {
            ($eq{time},$eq{lat},$eq{long},$eq{depth},$eq{mag},$eq{loc}) = (split(',', $_))[0,1,2,3,4,13];
            $seen{$eq{time}} = 1;
            push( @eqsetA, {%eq} );
        }
        ## Euro data
        {
        open(FILE, "<:crlf", "$seismic_dataA") or die "$LMsg{no_open_file} seismic_dataA data file $!";
            local $/ = undef;
            $ineqfileA = <FILE> ;
        close(FILE);
        }
        $ineqfileA =~ s/#.*\n//;
        $ineqfileA =~ s/\.\d+Z//g;
        $ineqfileA =~ s/\n/\#\n/g;
        foreach (split('\n',$ineqfileA)) {
            ($eq{time},$eq{lat},$eq{long},$eq{depth},$eq{mag},$eq{loc}) = (split('\|', $_))[1,2,3,4,10,12];
            unless ($seen{$eq{time}}) { push( @eqsetA, {%eq} ) }
        }

        $conf{num_of_events}     ||= 30;
        $conf{minimum_magnitude} ||= 4.5;
        $conf{alarm_magnitude}   ||= 5;
        $conf{alarm_flash}       ||= 7;

        my $choice = "rpt3";


        # for my $i ( 0 .. $#eqsetA ) {
        #   for my $role ( keys %{ $eqsetA[$i] } ) { msgout($i,$eqsetA[$i]{$role}) }
        # }

        my (%eqf, @scwatch, $scwatch_re );

        if ($conf{loc_width}) {
            if ( $conf{loc_width} < 24 ) { $eqf{all} = 24 }
            else { $eqf{all} = $conf{loc_width} }
        }
        else { $eqf{all} = 45 }

        if ($conf{loc_watch}) {
            # msgout($conf{loc_watch});
            if (index($conf{loc_watch}, ':') != -1) {
                ($scwatch_re = $conf{loc_watch}) =~ tr/:/|/;
            }
            else {
                $scwatch_re = $conf{loc_watch}
            }
        }

        $eqf{txt}  = $eqf{all} - 2 ;
        $eqf{txt1} = $eqf{all} - 1 ;
        $eqf{txt2} = $eqf{all} + 1 ;
        $eqf{hd}   = $eqf{all} - 1 ;

        my ($hLine, $vLine, $xJoin, $tlCorn, $trCorn, $blCorn, $brCorn, $bJoin, $tJoin, $lJoin, $rJoin, $box_char, $hvset, $jset);
        $conf{box_char} ||= '1:1';
        my %hvsetl = (
            '1' => '┄ |',
            '2' => '╴ ╷',
            '3' => '- |',
            '4' => '~ ║',
            '5' => '═ ║',
        );
        my %jsetl = (
            '1' => '╬ ╩ ╦ ╠ ╣ ╔ ╗ ╚ ╝',
            '2' => '┼ ┴ ┬ ├ ┤ ╭ ╮ ╰ ╯',
        );
        my $count = ($conf{box_char} =~ tr/://);
        if ($conf{box_char}) {
            if (length($conf{box_char}) eq 1 or $count == 0 ) {
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = ($conf{box_char}) x 11;
            }
            elsif ($count eq 1 ) {
                my ($l, $r) = split(/:/,$conf{box_char});
                $l = ($hvsetl{$l}) ? $l : 1;
                $r = ($jsetl{$r})  ? $r : 1;
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = split(/\s/, $hvsetl{$l} .' '. $jsetl{$r});
            }
            elsif ($count eq 10 ) {
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = split(/:/, $conf{box_char});
            }
            else {
                ($hLine, $vLine, $xJoin, $bJoin, $tJoin, $lJoin, $rJoin, $tlCorn, $trCorn, $blCorn, $brCorn ) = split(/\s/, $hvsetl{1} .' '. $jsetl{1});
            }
        }

        my @sorted =  sort { $b->{time} cmp $a->{time} } @eqsetA ;
        # foreach my $ky (@sorted) {
        foreach my $i (0..$#sorted) {
            # msgout($sorted[$i]{mag});
            # msgout($$ky{time},$$ky{lat}.$$ky{long});
            if ($sorted[$i]{mag} >= $conf{minimum_magnitude} ) {
                $cntr++;
                if ($cntr <= $conf{num_of_events} ) {
                    $sorted[$i]{lat}  = ( $sorted[$i]{lat}  < 0 ) ? (sprintf("%5.2f S",$sorted[$i]{lat}*-1 ))
                    : (sprintf("%5.2f N",$sorted[$i]{lat} ));
                    $sorted[$i]{long} = ( $sorted[$i]{long} < 0 ) ? (sprintf("%6.2f W",$sorted[$i]{long}*-1))
                    : (sprintf("%6.2f E",$sorted[$i]{long}));

                    $sorted[$i]{time} = &time2str($conf{seismic_time_format},$sorted[$i]{time}.'Z') ;
                    # msgout($sorted[$i]{time},length($sorted[$i]{time}));

                    $eqf{time} = length($sorted[$i]{time});

                    $sorted[$i]{depth} = convert_unit($sorted[$i]{depth}, 'Depth');
                    $sorted[$i]{mag}  = sprintf("%4.1f",$sorted[$i]{mag});
                    $sorted[$i]{loc} =~ tr/"//d;
                    if ($scwatch_re) {
                        if ( lc($sorted[$i]{loc}) =~ /$scwatch_re/o ) { $sorted[$i]{loc} = "*". $sorted[$i]{loc} }
                    }

                    if ($sorted[$i]{mag} >= $conf{alarm_flash} ) {
                        $wx{alertdata} .= "Seismic alert:  $sorted[$i]{mag}Mag. $sorted[$i]{loc}, Posn:$sorted[$i]{lat} $sorted[$i]{long} | ";
                        $sorted[$i]{loc} = '@'. $sorted[$i]{loc}
                    }

                    if (index($sorted[$i]{loc},'#') != -1) {
                        $sorted[$i]{loc} =~ tr/#//d;
                        $sorted[$i]{loc} =~ s/(\w+)/\u\L$1/g;
                    }
                    $wx{eq_count} = $cntr;
                    $eqf{hd1} = $eqf{time} + 0 if $eqf{time};

                    my $sprt;

                    if ($sorted[$i]{mag} >= $conf{alarm_magnitude} and
                        $sorted[$i]{mag} < $conf{alarm_flash} ) {
                        $cntr2++;
                        if ( index($sorted[$i]{loc}, '*') != -1 ) {
                            $sprt = "$vLine> %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        else {
                            $sprt = "$vLine> %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine %-$eqf{txt1}.$eqf{txt}s$vLine"
                        }
                    }
                    elsif ($sorted[$i]{mag} >= $conf{alarm_flash} ) {
                        if ( index($sorted[$i]{loc}, '@*') != -1 ) {
                                $sprt = "$vLine>>%-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{txt2}.$eqf{txt}s$vLine"
                        }
                        elsif ( index($sorted[$i]{loc}, '@') != -1 ) {
                                $sprt = "$vLine>>%-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        else {
                            $sprt = "$vLine>>%-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine %-$eqf{txt1}.$eqf{txt}s$vLine"
                        }
                    }
                    else {
                        if ( index($sorted[$i]{loc}, '*') != -1 ) {
                            $sprt = "$vLine  %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        elsif ( index($sorted[$i]{loc}, '@') != -1 ) {
                                    $sprt = "$vLine  %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine%-$eqf{all}.$eqf{txt}s$vLine"
                        }
                        else {
                            $sprt = "$vLine  %-$eqf{hd1}s $vLine%4s $vLine%9s $vLine%8s $vLine%9s $vLine %-$eqf{txt1}.$eqf{txt}s$vLine"
                        }
                    }
                    $sorted[$i]{rpt3} = sprintf($sprt,$sorted[$i]{time},$sorted[$i]{mag},$sorted[$i]{depth},$sorted[$i]{lat}, $sorted[$i]{long}, $sorted[$i]{loc});
                    push( @sorted2, $sorted[$i] );
                    # msgout($sorted[$i]);
                }
            }
        }

        my $mag_alarm;
        if ( $eqf{all} < 33 ) {
            $mag_alarm = ($cntr2)
            ? "Loc alarm > |$conf{alarm_magnitude}| = $cntr2/$wx{eq_count} "
            : "Location";
        }
        else {
            $mag_alarm = ($cntr2)
            ? "Location alarm > |$conf{alarm_magnitude}| = $cntr2/$wx{eq_count} "
            : "Location";
        }

        if ( $eqf{time} ) {
            $eqf{hd1} = $eqf{time} + 1;
            $hdr{rpt3} = sprintf("$vLine %-$eqf{hd1}s $vLine%4s $vLine %-8s $vLine %-7s $vLine %-8s $vLine %-$eqf{hd}s$vLine", "Time","Mag","Depth","Lat", "Long", $mag_alarm);

            $eqf{hd1} = $eqf{time} + 3;
            $mid{rpt3} = sprintf("$lJoin%s$xJoin%s$xJoin%s$xJoin%s$xJoin%s$xJoin%s$rJoin",$hLine x $eqf{hd1},$hLine x 5,$hLine x 10,$hLine x 9,$hLine x 10,$hLine x $eqf{all} );
            $top{rpt3} = sprintf("$tlCorn%s$tJoin%s$tJoin%s$tJoin%s$tJoin%s$tJoin%s$trCorn",$hLine x $eqf{hd1},$hLine x 5,$hLine x 10,$hLine x 9,$hLine x 10,$hLine x $eqf{all} );
            $ftr{rpt3} = sprintf("$blCorn%s$bJoin%s$bJoin%s$bJoin%s$bJoin%s$bJoin%s$brCorn",$hLine x $eqf{hd1},$hLine x 5,$hLine x 10,$hLine x 9,$hLine x 10,$hLine x $eqf{all} );

            open(SFILE, '>', "$temppath/seismic" ) or die "$LMsg{no_open_file} seismic file $!";
                # print SFILE "Wx Seismic Table\n";
                print SFILE "$top{$choice}\n";
                print SFILE "$hdr{$choice}\n";
                print SFILE "$mid{$choice}";
                for my $i ( 0 .. $#sorted2 ) {
                    for my $role ( keys %{ $sorted2[$i] } ) {
                        print SFILE "\n$sorted2[$i]{$role}" if ( $role eq $choice);
                    }
                }
                print SFILE "\n$ftr{$choice}\n";
            close(SFILE);
        }
        else {
            &show_noti('Warning :| no Seismic events match the minimum_magnitude settings in conkywx config file',1);
        }
    }
}

sub wrapText {

    # from module Text:Wrap - simple n sleek - now with 3 times the speed

    my $break = '(?=\s)\X';
    my $huge = 'wrap'; # alternatively: 'die' or 'overflow'
    my $separator = "\n";
    my $separator2 = undef;

    my $CHUNK = qr/\X/;

    my $xlen = sub { scalar(() = $_[0] =~ /$CHUNK/g) };

    my $ip = '';
    my $xp = ' ';

    my ($columns, @t) = @_;
    $columns //= 45;  # <= screen width

    my $r = "";
    my $tail = pop(@t);
    my $t = join("", (map { /\s+\z/ ? ( $_ ) : ($_, ' ') } @t), $tail);
    my $lead = $ip;
    my $nll = $columns - &$xlen($xp) - 1;
    if ($nll <= 0 && $xp ne '') {
        my $nc = &$xlen($xp) + 2;
        $columns = $nc;
        $nll = 1;
    }
    my $ll = $columns - &$xlen($ip) - 1;
    $ll = 0 if $ll < 0;
    my $nl = "";
    my $remainder = "";

    pos($t) = 0;
    while ($t !~ /\G(?:$break)*\Z/gc) {
        if ($t =~ /\G((?:(?=[^\n])\X){0,$ll})($break|\n+|\z)/xmgc) {
            $r .= $nl . $lead . $1;
            $remainder = $2;
        }
        elsif ($huge eq 'wrap' && $t =~ /\G((?:(?=[^\n])\X){$ll})/gc) {
            $r .= $nl . $lead . $1;
            $remainder = defined($separator2) ? $separator2 : $separator;
        }
        elsif ($huge eq 'overflow' && $t =~ /\G((?:(?=[^\n])\X)*?)($break|\n+|\z)/xmgc) {
            $r .= $nl . $lead . $1;
            $remainder = $2;
        }
        elsif ($huge eq 'die') {
            die "couldn't wrap '$t'";
        }
        else {
            die "This shouldn't happen";
        }

        $lead = $xp;
        $ll   = $nll;
        $nl   = defined($separator2)
            ? ($remainder eq "\n" ? "\n" : $separator2)
            : $separator;
    }
    $r .= $remainder;

    my($opos) = pos($t);

    $r .= $lead . substr($t, pos($t), length($t) - pos($t))
        if pos($t) ne length($t);

    return $r;

}

sub rssd {
    # gdacs information - RSS link - http://www.gdacs.org/xml/RSS.xml
    # Near real-time alerts about natural disaster with a potential humanitarian impact
    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my (%rssConf);

    my $whatToSee = sub {
        my ($text) = @_ ;
        return 'true' if not $rssConf{notReqType} and not $rssConf{notReqLevel};
        return '' if not $text ;


        my ($notReqType, $notReqLevel);
        if (defined $rssConf{notReqType} ) { $notReqType  = lc $rssConf{notReqType} }
        if (defined $rssConf{notReqLevel}) {
            ($notReqLevel = $rssConf{notReqLevel}) =~ s/(\w+)/\u\L$1/g
        }
        $notReqType  ||= 'nil';
        $notReqLevel ||= 'nil';

        my @excWords  = ('tropical cyclone', 'earthquake', 'tsunami', 'flood', 'volcano');
        my @excColour = ('Red', 'Orange', 'Green');
        for (@excWords) {
            if ($_ ne $notReqType) {
                if (index($text, $notReqType) == -1 ) {
                    for (@excColour) {
                        if ($_ ne $notReqLevel) {
                            if (index($text, $notReqLevel) == -1 ) {
                                return('true');
                            }
                        }
                    }
                }
            }
        }
    };

    my $gic = sub {
        my ($opt, $inval1, $inval2, $inval3 ) = @_ ;

        if ( $opt eq 'u' ) {
            return( sprintf("%.1f", $inval1 * 0.62137119 ) );
        }
        elsif ( $opt eq 'p' ) { #Fri, 25 Nov 2016 14:24:29 GMT
            if ( not $rssConf{pastEventDays} ) { return 1 }
            elsif ($inval1) {
                return ($nowtime - &str2time($inval1)) < (86400 * $rssConf{pastEventDays}) ? 1 : 0 ;
            }
            else { return 1 }
        }
        elsif ( $opt eq 't' ) {
            if ( $inval3 eq 'UTC') {
                return &time2str("%o %b, %Y %R","$inval1 $inval2 $inval3 DMY") . " LT" ;
            }
            elsif ( $inval3 eq 'AM' or $inval3 eq 'PM' ) {
                return &time2str("%o %b, %Y %I:%M %p","$inval1 $inval2 $inval3 MDY") ;
            }
        }
        elsif ( $opt eq 'd' ) {
            return &time2str("%o %b, %Y","$inval1",'dmy') ;
        }
    };

    my $getString = sub {
        my ($str, $startTag, $endTag, $from) = @_;

        return '' if not $$str;

        my $idx1 = index($$str, $startTag);

        if ( $idx1 != -1 ) {
            my $st1    = $idx1 + length($startTag);
            my $offset = $st1 + 1;
            my $nd1    = index($$str, $endTag, $offset);
            my $retval = substr($$str, $st1, $nd1 - $st1);
            if (index($retval,'<![CDATA[') != -1 ) {
                $retval =~ s/<!\[CDATA\[//;
                $retval =~ s/\]\]>//;
            }
            return $retval ;
        }
    };

    my $rss_unescape;
    $rss_unescape = sub {
        my $code_to_ascii = sub {
            if ( $_[0] <= 0x007F ) { return pack( "C" => $_[0] ) }
            return encode("utf-8", chr($_[0]));
        };
        my $fixStr = sub {
            $_[0] =~ s{
                    (&(?:\#(\d{1,4})|\#x([0-9a-fA-F]{1,2})|($asciiMapKeyList));)
                }{
                    $4 ? $asciiMap{$4} : &$code_to_ascii( $3 ? hex($3) : $2, $1 )
                }gex ;
            return $_[0]
        };
        $_[0] = &$fixStr($_[0]);
        $_[0] = &$fixStr($_[0]); # do this again to fix strings completely n return
        return  $_[0];
    };

    my $rssParse = sub {
        my ( $rssStr, %rs, @rss, @headText ) ;
        {
            open (FILE, "<:crlf", $rssConf{rssIn}) or
                        die "$LMsg{no_open_file} $rssConf{rssIn} data file $!";
                local $/ = undef;
                $rssStr = <FILE> ;
            close (FILE);
        }

        ### parse rss =====================================
        $rssStr =~ s/<!\[CDATA\[//g;
        $rssStr =~ s/\]\]>//g;
        $rssStr =~ tr/\x0a\x0a//d;

        if ( index($rssConf{rssOut}, 'news') != -1 ) {
            while ( $rssStr =~ m{<channel>.*?<title>(.*?)</title>}g ) {
                push(@headText, $1) if $1 and $1 ne '';
            }
        }

        ## get number of items
        my $c = () = $rssStr =~ /<item>/g;
        if ($rssConf{numberOfItemsToView} and
            $rssConf{numberOfItemsToView} <= $c) {
            $c = $rssConf{numberOfItemsToView}
        }

        my $cnt = 0;
        my @rssT;
        $cnt = 0;
        while ( $rssStr =~ m{<item>.*?<pub[dD]ate>(.*?)</pub[dD]ate>}g ) {
            $rssT[$cnt++][0] = $1 ;
        }
        $cnt = 0;
        while ( $rssStr =~ m{<item>.*?<title>(.*?)</title>}g ) {
            $rssT[$cnt++][1] = $1  ;
        }
        $cnt = 0;
        while ( $rssStr =~ m{<item>.*?<description>(.*?)</description>}g ) {
            $rssT[$cnt][2] = ($1) ? $1 : '' ;

            if (index($rssT[$cnt][2],'&lt;') != -1 ) { $rssT[$cnt][2] =~ s/&lt;/</g }

            if (index($rssT[$cnt][2],'&gt;') != -1 ) { $rssT[$cnt][2] =~ s/&gt;/>/g }

            $rssT[$cnt][2] =~ s/<[^>]*>/\x20/g;

            if (index($rssT[$cnt][2],'http') != -1 ) { $rssT[$cnt][2] =~ s/http.*$//g }

            if (index($rssT[$cnt][2],'To read this article') != -1 ) {
                $rssT[$cnt][2] =~ s/To read this article.*$//g
            }

            if (index($rssT[$cnt][2],'mailto:') != -1 ) { $rssT[$cnt][2] =~ s/mailto:.*$//g }

            $rssT[$cnt][2] =~ tr/\x20//s;

            $rssT[$cnt][2] =~ s/^\s+//;

            $cnt++;
        }

        my $numItems = $#{rssT};

        my $atype;
        my $aHeader = &$rss_unescape($headText[0]);

        my $aHeaderLine=sprintf("%s",'-' x $rssConf{textWrapColumn});

        my @printArray;
        my $pubCount = 0;
        if ($numItems > 0) {

            open(SFILE, ">", $rssConf{rssOut} ) or
                        die "$LMsg{no_open_file} $rssConf{rssOut} file $!";

            push @printArray, "\#gdacs - rss data\n@@|@@|@@|\n";

            my $talert  = ($#{headText} > 1) ? $numItems - $#{headText} : $numItems ;

            my $articleCount = 1;

            my @goodOnes; $goodOnes[0] = 0;
            for my $i (0..$numItems) {
                if ( &$whatToSee($rssT[$i][1]) eq 'true' and
                                &$gic('p',$rssT[$i][0]) == 1 ) {
                    $pubCount++;
                    $goodOnes[$i] = 1
                }
                else {
                    $goodOnes[$i] = undef
                }
            }
            $pubCount = $pubCount - $#{headText} if $#{headText} > 1;

            push @printArray, ":: Showing data for past $rssConf{pastEventDays} day(s) :: \n@@|@@|@@|\n" if $rssConf{pastEventDays};
            push @printArray,  ":: $aHeader :: \n";

            ## spaces required between new line for lua sm scroller to handle empty lines
            push @printArray, "$aHeaderLine\n@@|@@|@@|\n";
            my ($cntNo);
            for my $i (0..$numItems) {

                if ( $goodOnes[$i] ) {

                    if ($rssConf{showImperial} == 1) {
                        $rssT[$i][1] =~ s/\s*([\.\d]+)\s*km\/h\)/" ".&$gic('u',$1)." mph \)"/eg ;
                        $rssT[$i][2]  =~ s/\s*([\.\d]+)\s*km\/h\)/" ".&$gic('u',$1)." mph \)"/eg ;
                        $rssT[$i][1] =~ s/\s*([\.\d]+)\s*km/" ".&$gic('u',$1)." mi"/eg ;
                        $rssT[$i][2]  =~ s/\s*([\.\d]+)\s*km/" ".&$gic('u',$1)." mi"/eg ;
                    }

                    $rssT[$i][1] = &$rss_unescape($rssT[$i][1]);
                    $rssT[$i][2]  = &$rss_unescape($rssT[$i][2]);
                    $rssT[$i][1] =~ s/\s*([\/\d]+)\s+([\:\d]+)\s+(.*?),/" ".&$gic('t',$1,$2,$3)/eg;
                    $rssT[$i][2]  =~ s/\s*([\/\d]+)\s+([\:\d]+)\s+(.*?),/" ".&$gic('t',$1,$2,$3)/eg;
                    $rssT[$i][2]  =~ s/\s*(\d{2}\/\d{2}\/\d{4})/" ".&$gic('d',$1)/eg;

                    { # start of block
                        no warnings; # ignore pesky Unicode wide character warnings

                        if (index($rssT[$i][1],'@@@') != -1 ) {
                            ($cntNo) = $rssT[$i][1] =~ /(\d+)/;
                            push @printArray,  ":: $headText[$cntNo] :: \n";
                            push @printArray,  "$aHeaderLine\n@@|@@|@@|\n";
                            next
                        }
                        else {
                            if (not $cntNo) {
                                if ( index(lc($aHeader), 'gdacs') != -1 ) {
                                    $atype = 'Alert' }
                                else { $atype = 'News' }
                            }
                            else {
                                if ( index(lc($headText[$cntNo]), 'gdacs') != -1 ) {
                                    $atype = 'Alert' }
                                else { $atype = 'News' }
                            }
                            push @printArray, ":: $atype :: $articleCount of $pubCount :: Total $talert :: \n";
                        }

                        push @printArray, &time2str("Published on:: $rssConf{PublishDateFormat}",
                            $rssT[$i][0])."\n" if $rssT[$i][0];

                        if ( not $rssConf{reqTitleOrDesc} or $rssConf{reqTitleOrDesc} == 1 ) {
                            push @printArray, wrapText($rssConf{textWrapColumn}, $rssT[$i][1])."\n";
                        }

                        if ( not $rssConf{reqTitleOrDesc} or $rssConf{reqTitleOrDesc} == 2 ) {
                            push @printArray, wrapText($rssConf{textWrapColumn}, $rssT[$i][2])."\n";
                        }

                        push @printArray, "@@|@@|@@|\n";

                        $articleCount++

                    } # end of block
                }
            }
            print SFILE @printArray;
        }
        else { print SFILE "\n\nNo rss data to show\n\n"}

        $numItems = ($numItems > 0) ? $numItems + 1 : 0 ;
        my $noEventsMsg1 = "No $atype to show for the past $rssConf{pastEventDays} day(s).\nThere are a total of $numItems $atype items.\n";
        my $noEventsMsg2 = "To see more, change \"pastEventDays\" option for rssd in conkywx config file:: ${config_file} ";

        if ($pubCount == 0) {
            print SFILE wrapText($rssConf{textWrapColumn},$noEventsMsg1);
            print SFILE wrapText($rssConf{textWrapColumn},$noEventsMsg2);
        }
        close(SFILE);
    };

    if ( -e $gdacsData and $conf{gdacsUrl} ) {
        undef %rssConf;
        $rssConf{rssUrl}            = $conf{gdacsUrl};
        $rssConf{rssIn}             = $gdacsData;
        $rssConf{rssOut}            = "${temppath}/gdacs";
        $rssConf{textWrapColumn}    = $conf{gtextWrapColumn} // 45;
        $rssConf{showImperial}      = $conf{gshowImperial} // 0 ;
        $rssConf{pastEventDays}     = $conf{gpastEventDays} // 0 ;
        $rssConf{notReqType}        = $conf{gnotReqType};
        $rssConf{notReqLevel}       = $conf{gnotReqLevel};
        $rssConf{reqTitleOrDesc}    = $conf{greqTitleOrDesc};
        $rssConf{PublishDateFormat} = $conf{gPublishDateFormat} // '%o %b, %Y %I:%M %p';
        &$rssParse();
    }
    if ( -e $newsData and $conf{newsUrl} ) {
        undef %rssConf;
        $rssConf{rssUrl}              = $conf{newsUrl};
        $rssConf{rssIn}               = $newsData;
        $rssConf{rssOut}              = "${temppath}/news";
        $rssConf{textWrapColumn}      = $conf{ntextWrapColumn} // 45;
        $rssConf{showImperial}        = $conf{nshowImperial} // 0 ;
        $rssConf{pastEventDays}       = $conf{npastEventDays} // 0 ;
        $rssConf{reqTitleOrDesc}      = $conf{nreqTitleOrDesc};
        $rssConf{numberOfItemsToView} = $conf{nNumberOfItemsToView};
        $rssConf{PublishDateFormat}   = $conf{nPublishDateFormat} // '%o %b, %Y %I:%M %p';
        $rssConf{notReqType}          = $conf{gnotReqType};
        $rssConf{notReqLevel}         = $conf{gnotReqLevel};
        &$rssParse();
    }
}

sub get_line_num {

=comments
    paramvir 2016 get_line_num ver 3
    Does not match the tags - be careful with the 3rd option
    same but much faster as /pat/../pat/ range
    Usage:
    1st opts <- Input String reference \$in_string
        $str = a string reference original string as \$OriginalStringName
    2nd opts <- text to look for
        $startTag = is the start of required block
    3rd opts endTag text
        $endTag can be 1 of 3 options -
            This has to be unique block following the $startTag block
    4th opts
        $choi = arr - get from start to end text as array
        $choi = str - get from start to end text as single string [default]
    5th opts
        $extended = only used in the case of accuweather

    $extended - use 1 for looking up next index value
    $extended - use 2 added length of $startTag

    string startTag endTag returnType extended
=cut

    my ($str, $startTag, $endTag, $choi, $extended ) = @_;
    my ($returnstr, @returnarr ) ;
    $choi ||= 'str' ;
    if ( ! defined($startTag) ) { return \$_ }

    # msgout($extended) if $extended;
    my $st1 = index($$str, $startTag);
    if ($st1 != -1) {
        my $offset = $st1 + 1;
        my $nd1    = index($$str, $endTag, $offset);
        $returnstr = substr($$str, $st1, $nd1 - $st1);
        if ($extended) {
            $offset = $nd1 + 1;
            $st1    = index($$str, $startTag, $offset);
            if ($st1 != -1) {
                $offset = $st1 + 1;
                $nd1    = index($$str, $endTag, $offset);
                $returnstr .= substr($$str, $st1, $nd1 - $st1);
            }
        }
        if ( $choi eq 'str' ) { return \$returnstr }
        else { @returnarr=split('\n',$returnstr); return \@returnarr }
    }
    else { return \$_ }
}

sub metar_parser {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($x, $y) = @_ ;

    $wx{ltxt} = `"$cwx_magellan" --metarwx "$x"`;

    my @base_wx_list = qw{city name nameid latitude longitude elevation updated icon condition temperature dewpoint humidity pressure precip_today visibility snow uv_index pollen metar wind_speed wind_dir wind_dir_degrees wind_gust_speed cloud1h cloud1t cloud2h cloud2t cloud3h cloud3t cloud4h cloud4t};

    for ( split( /\n/, $wx{ltxt} )) {
        next if (/^\s+$/);
        next if (/^#.*$/);
        my ($var, $val) = split( /\s+:\s+/, $_, 2);
        $wx{$var} = $val if($var);
    }
    $wx{wind_gust_speed} = ($wx{wind_gust_speed}) ? $wx{wind_gust_speed} : 0;

    if ( $wx{temperature} and $wx{dewpoint} ) {
        $wx{humidity} = int(100*((112-(0.1*$wx{temperature})+$wx{dewpoint}) / (112+(0.9*$wx{temperature})))**8)
    }
    else { $wx{humidity} = 'NA' }

    $wx{wind_dir} = ($wx{wind_dir_var})
    ? "Var"
    : ($wx{wind_dir_degrees})
    ? find_wind_dir($wx{wind_dir_degrees})
    : 'NA';

    $wx{condition} = ($wx{condition})
    ? metar_sky($wx{condition})
    : ($wx{cloud4t})
    ? metar_sky($wx{cloud4t}, 1 )
    : ($wx{cloud3t})
    ? metar_sky($wx{cloud3t}, 1 )
    : ($wx{cloud2t})
    ? metar_sky($wx{cloud2t}, 1 )
    : ($wx{cloud1t})
    ? metar_sky($wx{cloud1t}, 1 )
    : "Clear" ;

    $wx{city} =~ s/(\w+)/\u\L$1/g;
    $wx{city} = $wx{city} . "$y Wx";
    $wx{name} ||= $wx{city};

    for (@base_wx_list) { if (! $wx{$_}) { $wx{$_} = 'NA'}  }
}

sub wx_parser {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($instr) = @_;
    # msgout($instr);

    my (%wxta,%wxtb);
    if ( $wxMainTitle eq 'msn' ) {
        %wxta = (
            city        => [ 'today/',         '/we-city', 'al' ],
            latitude    => [ 'we-city-',       ',',        'n'  ],
            longitude   => [ 'we-city-',       '?iso=',    'nl' ],
            temperature => [ 'temperature is', 'degrees',  'n'  ],
            condition   => [ 'condition is ',  ' and the', 'a'  ],
            wind_dir    => [ 'winddir',        '</div>',   'w'  ],
            wind_speed  => [ 'winddir',        '</li>',    'n'  ],
            visibility  => [ 'Visibility',     '</li>',    'n'  ],
            pressure    => [ 'Barometer',      '</li>',    'n'  ],
            humidity    => [ 'Humidity',       '%',        'n'  ],
            dewpoint    => [ 'Dew Point',      '</li>',    'n'  ],
        );
    }
    if ( $wxMainTitle eq 'accu' ) {
        %wxta = (
            city        => [ '"current-city"><h1>',      '</h1>',     'a'  ],
            latitude    => [ '<span class="latitude">',  '</span>',   'n'  ],
            longitude   => [ '<span class="longitude">', '</span>',   'n'  ],
            temperature => [ '<span class="large-temp">','</span>',   'n'  ],
            condition   => [ '<span class="cond">',      '</span>',   'a'  ],
            wind_dir    => [ 'wind-point',               '"></div>',  'a'  ],
            wind_speed  => [ 'class="wind"><strong>',    '</strong>', 'n'  ],
            visibility  => [ 'Visibility:',              '</strong>', 'n'  ],
            pressure    => [ 'Pressure: <strong>',       '</strong>', 'n'  ],
            humidity    => [ 'Humidity',                 '%',         'n'  ],
            dewpoint    => [ 'Dew Point',                '</strong>', 'n'  ],
            cloud1t     => [ 'Cloud Cover:',             '</strong>', 'n'  ],
            cloud1h     => [ 'Ceiling:',                 '</strong>', 'n'  ],
        );
    }
    if ( $wxMainTitle eq 'wx' ) {
        %wxta = (
            updated          => ['<lsup>'    , '</lsup>'    , 'al' ],
            city             => ['<obst>'    , '</obst>'    , 'al' ],
            condition        => ['</flik> <t>', '</t> <icon>' , 'al' ],
            visibility       => ['<vis>'     , '</vis>'     , 'al' ],
            humidity         => ['<hmid>'    , '</hmid>'    , 'al' ],
            dewpoint         => ['<dewp>'    , '</dewp>'    , 'al' ],
            temperature      => ['<tmp>'     , '</tmp>'     , 'al' ],
            nameid           => ['<loc id="' , '">'         , 'al' ],
            latitude         => ['<lat>'     , '</lat>'     , 'al' ],
            longitude        => ['<lon>'     , '</lon>'     , 'al' ],
            name             => ['<dnam>'    , '</dnam>'    , 'al' ],
            pressure         => ['<r>'       , '</r>'       , 'al' ],
            wind_gust_speed  => ['<gust>'    , '</gust>'    , 'al' ],
            wind_dir_degrees => ['</gust> <d>', '</d>'      , 'al' ],
            wind_speed       => ['<s>'       , '</s>'       , 'al' ],
        );
    }

    my (%idxl, @idxarr, %idxl2);
    $instr =~ tr/\x0a\x20/ /s ;
    # $instr =~ s/\n\s*//g;
    # msgout($instr);
    for my $ky ( keys %wxta) {
        my $st   = $wxta{$ky}[0];  # string where it starts
        my $nd   = $wxta{$ky}[1];  # string where it ends
        my $type = $wxta{$ky}[2];  # what to do with it
        my $st1  = index($instr, $st) + length($st); # get posn
        my $nd1  = index($instr, $nd, $st1);  # get number of char to pull
        my $fnd  = substr($instr, $st1, $nd1-$st1 );
        # msgout($ky,$st1, $nd1,'>>>'.$fnd.'<<<' );
        if    ($type eq 'n' ) { $wx{$ky} = ($fnd =~ m{(-?(\d+(\.\d+)?))}) ? $1 : 'NA' }
        elsif ($type eq 'nl') { $wx{$ky} = ($fnd =~ m{,(-?(\d+(\.\d+)?))}) ? $1 : 'NA' }
        elsif ($type eq 'a' ) {
            $wx{$ky} = ($fnd =~ m{(\w+((([,|\-|\s]+\w+))+)?)}) ? $1 : 'NA' ;
            if ($wx{wind_dir}) {
                ($wx{wind_dir_degrees},$wx{wind_dir}) = find_wind_dir($wx{wind_dir})
            }
        }
        elsif ($type eq 'al') {
            if ($wx{wind_dir_degrees} and not exists $wx{wind_dir} ) {
                $wx{wind_dir} = find_wind_dir($wx{wind_dir_degrees});
            }
            $wx{$ky} = $fnd ;
        }
        elsif ($type eq 'w' ) {
            $wx{wind_dir} = ($fnd =~ m{^.*">(.*?)$}) ? $1 : "NA"   ;
            if ($wx{wind_dir}) {
                ($wx{wind_dir_degrees},$wx{wind_dir}) = find_wind_dir($wx{wind_dir});
            }
        }
        # msgout($ky,$wx{$ky}) ;
    }
    if ( $wxMainTitle eq 'accu' ) { $wx{cloud1t} = ($wx{cloud1t}) ? $wx{cloud1t}.'%' : '0%' }
}

sub fct_parser {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

=comments
    the hash %wxta uses 4 parts - delimiter is ';;'
    1st = start context
    2nd = end context
    3rd = type of data expected
    4th = optional - use 1 to use rindex in place of index to search
=cut
    my $cntn = 0;
    my ($delim, %wxta);
    my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);
    if ( $wxFctTitle eq "msn"  ) {
        $delim = ';;';
        %wxta = (
            day   => [ 'for ',            ' is',        'd'  ],
            cond  => [ '"image skyimg" ', ' data-icon', 'c'  ],
            temp  => [ 'High of ',        '<div title', 't' ],
            pop   => [ 'Precipitation',   '</span>',    'p'  ],
        );
    }
    # $wx{fctstr} =~ tr/\x0a\x20/ /s ;


    # msgout($wx{fctstr});
    for ( split /$delim/, $wx{fctstr} ) {
        # msgout('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'.$_);
        next if /Early AM/;
        $cntn++;
        for my $ky (keys %wxta) {
            # my $val;
            my $st     = $wxta{$ky}[0];
            my $nd     = $wxta{$ky}[1];
            my $type   = $wxta{$ky}[2];
            my $offset = $wxta{$ky}[3];
            $st  = index($_,$st) + length($st);;
            if ($offset) { $nd  = rindex($_,$nd) - $st }
            else { $nd  = index($_,$nd, $st) - $st }
            my $fnd = substr($_, $st, $nd );
            for ( $type ) {
                # msgout($ky,$st, $nd,$fnd);
                if    ( /d/ )  { # dates day
                    $fctinfo{"d_" . $cntn . "_day" } = get_short_day($fnd) ;
                }
                elsif ( /c/ )  { # alphanumeri with space cond
                    if ($fnd =~ m{title="(.*?)"}) {
                        $fctinfo{"d_" . $cntn . "_font"} = do_get_DI(2, $1,$cntn)
                    }
                }
                elsif ( /t/ ) { # numbers only - high low temps
                    if ($fnd =~ m{(--?|(-?([\.\d]+)?)).*(?:low of )(--?|(-?([\.\d]+)?))}) {
                        $fctinfo{"d_" . $cntn . "_temp"} = ch_fcunits("$1/$4")
                    }
                }
                elsif ( /p/ )  { # numbers only - pop
                    if ($fnd =~ m{(\d+(\%)?)}) {
                        $fctinfo{"d_" . $cntn . "_cor" } =  $1
                    }
                }
            }
            last if ( $cntn > 10 )
        }
    }
}

sub parse_html {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    if ( -e $currdatam ) {

        $wx{dataisthere} = 1;

        my ( $htmlstr  ) ;
        {
        open (FILE, "<:crlf", $currdatam) or die "$LMsg{no_open_file} main data file $!";
            local $/ = undef;
            $htmlstr = <FILE> ;
        close (FILE);
        }
        # &Testing(\$htmlstr);
        # msgout($htmlstr);

        ### wunderground weather ################################################
        if ( $wxMainTitle eq "wund" ) {
            my $wundwarn;
            if (index($htmlstr,'class="alert-items">') != -1) { $wundwarn=1 }
            # @wundhash{@wundarr} = @base_wx_list;
            if ($wundwarn) {
                my $c;
                for my $k (split('<span>', ${&get_line_num( \$htmlstr,'<div class="alert-items">','</div>','str')})) {
                    $k =~ s/<+[^>,]*>//g;
                    next if $k =~ /^\s*$/;
                    $c++;
                    $wx{alertdata} .= "Weather alert $c: $k | ";
                }
            }

            my $wundCurrData = ${&get_line_num(\$htmlstr,'"current_observation"','"cod_wspd":','str')};

            my %wundh = (
                city       => 'city'      , name         => 'name'        ,
                id         => 'nameid'    , latitude     => 'latitude'    ,
                longitude  => 'longitude' , elevation    => 'elevation'   ,
                epoch      => 'updated'   , icon         => 'icon'        ,
                condition  => 'condition' , temperature  => 'temperature' ,
                dewpoint   => 'dewpoint'  , humidity     => 'humidity'    ,
                pressure   => 'pressure'  , precip_today => 'precip_today',
                visibility => 'visibility', snow         => 'snow'        ,
                pollen     => 'pollen'    , metar        => 'metar'       ,
                wind_speed => 'wind_speed', wind_dir     => 'wind_dir'    ,
                height1    => 'cloud1h'   , cover_text1  => 'cloud1t'     ,
                height2    => 'cloud2h'   , cover_text2  => 'cloud2t'     ,
                height3    => 'cloud3h'   , cover_text3  => 'cloud3t'     ,
                height4    => 'cloud4h'   , cover_text4  => 'cloud4t'     ,
                wind_dir_degrees => 'wind_dir_degrees',
                wind_gust_speed  => 'wind_gust_speed' ,
                solarradiation   => 'solarRad' ,
                );

            my $ia=1;
            $wundCurrData =~ s/\b(height)\b/"$1".$ia++/eg;
            $ia=1;
            $wundCurrData =~ s/(cover_text)/"$1".$ia++/eg;
            $wundCurrData =~ tr/\x7b\x7d/\x2c/;
            $wundCurrData =~ tr/\x22\x5b\x5d//d;
            # msgout($wundCurrData);

            for (split /\x2c/,$wundCurrData) {
                next if not $_;
                # msgout($_);
                my ($t1,$t2) = split(/(?::)/,$_) ;
                # msgout($t1,$t2);
                $wx{$wundh{$t1}} = $t2 if $wundh{$t1} and $t2;
            }
            if    ( $wx{name} ne 'null' and $wx{city} eq 'null' ) { $wx{city} = $wx{name} }
            elsif ( $wx{name} eq 'null' and $wx{city} ne 'null' ) { $wx{name} = $wx{city} }

            if ($wx{condition} eq 'null') {
                $wx{condition} = ${&get_line_num(\$htmlstr,'<meta property="og:title"','/>','str')};
                ($wx{condition}) = ($wx{condition} =~ m{>*\|.*\|(.*?)"}) ? $1 : 'unknown';
            }
        }

        ### weather.com weather ################################################
        elsif ( $wxMainTitle eq "wx" ) {
            # current condition
            my $xml1 = ${&get_line_num(\$htmlstr,'</head>','<moon>','str')};
            wx_parser($xml1);

            ### wunderground forecast
            if ( $wxFctTitle eq "wx" ) {
                $xml1 = ${&get_line_num(\$htmlstr,'<dayf>','</dayf>','str')};
                $xml1 =~ tr/\x0a\x20/\x20/s;
                my $cntn = 1;
                while ($xml1 =~ m{<part\s+p="d">.*?</icon>\s+<t>(.*?)</t>}g) {
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn);
                }
                if ( $fctinfo{d_1_font} eq '' ) {
                    if ($xml1 =~ m{<part\s+p="n">.*?</icon>\s+<t>(.*?)</t>}) {
                        $fctinfo{d_1_font} = do_get_DI(2, $1, 1)
                    }
                }
                $cntn = 1;
                while ($xml1 =~ m{<hi>(.*?)</hi>\s+<low>(.*?)</low>}g) {
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$1/$2");
                }
                $cntn = 1;
                while ($xml1 =~ m{<part\s+p="d">.*?<ppcp>(.*?)</ppcp>}g) {
                    $fctinfo{"d_" . $cntn++ . "_cor" } = $1 . '%'
                }
                $cntn = 1;
                while ($xml1 =~ m{<day\s+d=.*?t="(.*?)"\s+dt=".*?\s+(.*?)">}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day("$1 $2");
                }
                $wx{fctstr} = 1; # required to disable forecast re calc
            }
            $wx{updated} = $wx{updated} . ' MDY';
        }

        ### darksky.net weather ################################################
        elsif ( $wxMainTitle eq "darkSky" ) {
            my %dks = (
                currentlytime => 'updated'   , icon           => 'icon',
                summary       => 'condition' , temperature    => 'temperature',
                dewPoint      => 'dewpoint'  , humidity       => 'humidity',
                pressure      => 'pressure'  , visibility     => 'visibility',
                windSpeed     => 'wind_speed', cloudCover     => 'cloud1t',
                windBearing   => 'wind_dir_degrees', windGust => 'wind_gust_speed' ,
                ghi           => 'solarRad' ,
                );
            my $dsData = ${&get_line_num( \$htmlstr, 'currently = {','map_info = {', 'str')};
            $dsData =~ tr/\x22\x7d\x7b\x20\x3d//d;
            for (split ',', $dsData) {
                my ($var, $val) = split( ':', $_, 2);
                # msgout($var, $val) if $val;
                $wx{$dks{$var}} = $val if $dks{$var} and $val;
            }
            if ( $wx{wind_dir_degrees} ) { $wx{wind_dir} = find_wind_dir($wx{wind_dir_degrees}) }
            if ( $wx{pressure} ) { $wx{pressure} = hPa_in($wx{pressure}) }

            $wx{solarRad} = ($wx{solarRad}) ? &round($wx{solarRad}) : 'NA' ;

            if ($conf{darkSkyLoc}) {
                $wx{city} = $wx{name} = $conf{darkSkyLoc}
            }
            else {
                $wx{city} = $wx{name} = "$conf{Latitude},$conf{Longitude}"
            }
            $wx{nameid} = 'Dark Sky';
            $wx{cloud1t} = ($wx{cloud1t}) ? ($wx{cloud1t} * 100) . '% Cloud Cover' : 'NA';

            ### darksky forecast
            if ( $wxFctTitle eq "darkSky" ) {
                my $dsFct = ${&get_line_num( \$htmlstr, '<div class="dayDetails">','<div id="timeMachine"', 'str')};
                $dsFct =~ tr/\x0a\x20/ /s ;
                my $cntn = 1;
                while ($dsFct =~ m{<div class="summary">(.*?)</div>}g) {
                    my $val = ($1) ? $1 : 'NA' ;
                    for (@rainCond) {
                        if (index(lc($val),$_) != -1) {
                            $fctinfo{"d_" . $cntn . "_cor"} = 'Yes'; last
                        }
                        else {
                            $fctinfo{"d_" . $cntn . "_cor"} = 'No'
                        }
                    }
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn);
                }
                $cntn = 1;
                while ($dsFct =~ m{<span class="highTemp.*?"temp">(.*?)˚</span>.*?="lowTemp.*?="temp">(.*?)˚</span>}g) {
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$2/$1");
                }
                $cntn = 1;
                while ($dsFct =~ m{href="\/details\/.*?\/(.*?)\/us24\/en"}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day(&time2str("%a %d",$1));
                }
                $wx{fctstr} = 1; # required to disable forecast re calc
            }

            # die;
        }

        ### msn.com weather ################################################
        elsif ( $wxMainTitle eq "msn" ) {
            # my @htmldb = split('\n',$htmlstr);
            my $msnwarn = ${&get_line_num( \$htmlstr, 'class="alerts','div','str')};
            if ($msnwarn) {
                my $c;
                foreach(split('</div>',$msnwarn)) {
                    if ( m{<h2>(.*?)<span id="alertTime">(.*?)</span>} ){
                        $c++; $wx{alertdata} .= "Weather alert $c: ".$1.$2.' | ';
                    }
                }
            }
            wx_parser(${&get_line_num( \$htmlstr, '<section class="curcond"','data-aop="DailyForecast"', 'str')});
            $wx{wind_gust_speed} = 0; #this info is NA from msn
            $wx{name} = $wx{city};
            $wx{nameid}  = "MSNWX" ;
            $wx{updated} = $ltime;

            ### msn forecast
            if ( $wxFctTitle eq "msn" ) {
                my $fctdb = &get_line_num( \$htmlstr, 'data-aop="DailyForecast"','</section>', 'arr');
                my $fctl;
                for my $i (0..$#{$fctdb}) {
                    my $k = $$fctdb[$i];
                    if (index($k,'aria-label') != -1 )    { $wx{fctstr} .= $k }
                    if (index($k,'"image skyimg"') != -1 )            { $wx{fctstr} .= $k }
                    if (index($k,'<div title="Precipitation"') != -1) { $wx{fctstr}.=$k.";;" }
                }
                &fct_parser
            }
        }

        ### intellicast.com weather ################################################
        elsif ( $wxMainTitle eq "intel" ) {
            my $IntelMet = ${&get_line_num( \$htmlstr, 'Report Text', '</td>','str' )};
            if ($IntelMet) {
                if ($IntelMet =~ m{Report Text:.*;(.*?)\s*$}) {
                # if ($IntelMet =~ m{:.*;(.*?)\s*$}) {
                    ## hack fix temp and pressure joined string 08/06Q1016 to 08/06 Q1016
                    my $metarText;
                    for (split(' ',$1)) {
                        if ((index($_,'/') != -1) and (index($_,'Q') != -1)) {
                            $_ =~ s/(.*?)Q(.*?)/$1 Q$2/;
                        }
                        $metarText .= $_.' '
                    }
                    &metar_parser($metarText, 'Intellicast');
                }
            }
            $wx{updated} = "$monAbb $wx{updated} $year Z";

            ($wx{nameid},$wx{city}) = (${&get_line_num(\$htmlstr,'class="user-location">','</div>','str')} =~ m{location=(.*?)">\s*.*</span>(.*?)\s*</a>}) ? ($1,$2) :( "NA","NA");

            ### intellicast forecast
            if ( $wxFctTitle eq "intel" ) {
                my $inFct = ${&get_line_num(\$htmlstr,"chart_dailyForecast.setDataXML","<dataset seriesName='Lows'",'str')};

                my $inFctCond = ${&get_line_num(\$htmlstr,'<table id="daily">','</table>','str')};

                my $cntn = 1;
                while ($inFctCond =~ m{title="(.*?)"\s*\/>}g) {
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn);
                }
                $cntn = 1;
                while ($inFct =~ m{High:\s*(.*?)F\\nLow:\s*(.*?)F}g) {
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$1/$2");
                }
                $cntn = 1;
                while ($inFct =~ m{Precip:\s*(.*?)\\n}g) {
                    $fctinfo{"d_" . $cntn++ . "_cor" } = $1;
                }
                $cntn = 1;
                while ($inFct =~ m{label='(.*?)'}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day($1);
                }
                $wx{fctstr} = 1; # required to disable forecast re calc
            }
        }

        ### METAR weather ################################################
        elsif ( $wxMainTitle eq "metar" ) {
            if ($wx{ltxt} = ${&get_line_num( \$htmlstr, $conf{icao_id},',','str' )}) {
                metar_parser($wx{ltxt}, 'METAR');
                # msgout($wx{ltxt});
            }
            else {
                msgout "\n\nError: :( \nChoose another METAR Station\nThis Station does not appear to be sending METAR reports.\n" ;
                die;
            }
            $wx{updated} = "${monAbb} $wx{updated} $year Z";
        }

        ### accuweather.com weather ################################################
        elsif ( $wxMainTitle eq "accu" ) {
            my $accustr =
            ${&get_line_num(\$htmlstr,'<span class="current-city">','</span>','str')};
            $accustr .=
            ${&get_line_num(\$htmlstr,'<div id="detail-now"','<!-- /.detail-tab-panel','str')};

            $accustr .=
            ${&get_line_num(\$htmlstr,'<span class="geo">','</div>','str' )}; #'

            ### accu forecast
            ### required for the second part of accu forecast
            if ( $wxFctTitle eq "accu" ) {
                $wx{fctstr} = ${&get_line_num(\$htmlstr,'<!-- /.feed-controls','<!-- /.feed-tabs','str')};
            }
            wx_parser($accustr);
            $wx{wind_gust_speed} = 0; #this info is NA from accu
            $wx{name} = $wx{city};
            $wx{nameid}  = "AccuWX";
            $wx{updated} = $ltime;

            if ( -e "$temppath/accuwarn" ) {
                my ($warnstr, $c);
                {
                    open (FILE, "<:crlf", "$temppath/accuwarn") or die "$LMsg{no_open_file} accuwarn data file $!";
                    local $/ = undef;
                    $warnstr = <FILE> ;
                    close (FILE);
                }
                my $accuwarn = &get_line_num( \$warnstr,'<div class="info">','<div class="content awx-story">', 'arr');
                for (@{$accuwarn}) {
                    if ( $_ =~ m{<h5 .*>(.*?)</h5>} ) {
                        $c++;
                        $wx{alertdata} .= "Weather alert $c: ".$1.' | '
                    }
                }
            }
        }
    }
}

# -----------------------------------------------------------------------
# xml parser for wunderground weather private weather station xml data
# -----------------------------------------------------------------------
sub parse_xml {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }
    $wx{do_xml} = 1;
    if (-e $currdatax) {
        my ( $xmlstr  ) ;
        {
        open (FILE, "<:crlf", "$currdatax") or die "$LMsg{no_open_file} main data file $!";
            local $/ = undef;
            $xmlstr = <FILE> ;
        close (FILE);
        }

        my @wxax = qw(neighborhood latitude longitude elevation station_id observation_time_rfc822 temp_f relative_humidity wind_string wind_dir wind_degrees wind_mph wind_gust_mph pressure_in dewpoint_f solar_radiation precip_today_in);
        my @wxaw = qw(name latitude longitude elevation nameid last_update_xml temperature humidity wind_string wind_dir wind_dir_degrees wind_speed wind_gust_speed pressure dewpoint solarRad precip_today);
        # my %wxa;
        my $cntr = 0;
        for my $k (split '\n',$xmlstr) {
            if (index($k, $wxax[$cntr]) != -1 ) {
                my ($val) = (split(/(?:<[^>]*>)/, $k, 3))[1];
                $wx{$wxaw[$cntr]} = $val;
                if ( $wx{wind_speed} and $naList{$wx{wind_speed}} and $wx{wind_string} =~ /Calm/i ) {
                    $wx{wind_speed} = 'Calm'
                }
                $cntr++;
                last if $cntr > $#wxax ;
            }
        }

        if ( $wx{nameid} eq '#WXSTATION.CUR.StationID' ) {
            msgout("\n\nWarning: :( \nPWS $conf{xml_ID} (xml_ID) data is more than 1 hour old and \n2nd PWS $conf{xml_ID2} (xml_ID2) is not valid !!!\nUsing only main data for weather.\n");
            $wx{alertdata} .= "Data not reliable - pws data is stale. Change xml_ID and xml_ID2 in conkywx config file.";
            if ( -e $currdatax ) { unlink $currdatax } # removing stale pws data
            open (out_data, '>', "$stalexml") or die "$LMsg{no_open_file} stalexml: $!";
                print out_data 'stale pws';
            close (out_data);
            exec($^X, $0, @ARGV); # restarting conkywx.pl without pws data
        }
    }
}


sub validate_xml {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    $wx{xml_time}  = ($wx{last_update_xml}) ? &str2time($wx{last_update_xml}) : "NA" ;

    my $locale_time;
    if ($conf{t_zone_pick}) {
        $locale_time = ($nowtime - $wx_tzone) + (fix_tzone($conf{t_zone_pick}) * 3600) ;
        # msgout($nowtime,$locale_time, $wx{xml_time},$wx_tzone) ;
    }
    else {$locale_time = $nowtime}

    if ( $wx{xml_time} ne 'NA' and ($locale_time - $wx{xml_time}) > 3600 ) {
        if ($conf{xml_ID2}) {
            if ($wx{xml_state} and $wx{xml_state} == 1) {
                msgout("\n\nWarning: :( \nPWS $conf{xml_ID} (xml_ID) and 2nd PWS $conf{xml_ID2} (xml_ID2) data \nis more than 1 hour old !!!\nUsing only main data for weather.\n");
                $wx{alertdata} .= "Data not reliable - pws data is stale. Change xml_ID and xml_ID2 in conkywx config file.";
                if ( -e $currdatax ) { unlink $currdatax } # removing stale pws data
                open (out_data, '>', "$stalexml") or die "$LMsg{no_open_file} stalexml: $!";
                    print out_data 'stale pws';
                close (out_data);
                exec($^X, $0, @ARGV); # restarting conkywx.pl without pws data
            }
            else {
                msgout("\n\nAlert: :)) \nPWS $conf{xml_ID} (xml_ID) data is more than 1 hour old.\nUsing Secondary PWS station $conf{xml_ID2} (xml_ID2)\nPlease update the conkywx config file.\n\n");

                $wx{xml_url} = 'http://api.wunderground.com/weatherstation/WXCurrentObXML.asp?ID=';
                $wx{xml_url} = "$wx{xml_url}$conf{xml_ID2}&format=XML" ;

                `wget -q  -o /dev/null --tries=99 --timeout=30 -O $currdatax $wx{xml_url}`;

                ## also download history from the secondary pws
                my $hday   = ( $day < 10 ) ? '0'.$day : $day ;
                my $hmonth = ( $mon < 10 ) ? '0'.$mon : $mon ;
                my $orghistwu = "$temppath/orghistwu";
                my $hist_url = "http://www.wunderground.com/weatherstation/WXDailyHistory.asp?ID=$conf{xml_ID2}";
                $hist_url   .= "\\&day=${hday}\\&mon=${hmonth}\\&year=${year}\\&graphspan=day\\&format=1" ;

                `wget -q  -o /dev/null --tries=99 --timeout=30 -O $orghistwu $hist_url`;

                parse_xml;
                $wx{xml_state} = 1;
                $wx{alertdata} .= "PWS xml_ID data is old - using xml_ID2."
            }
        }
        else {
            msgout("\n\nAlert: :)) \nPWS $conf{xml_ID} (xml_ID) data is more than 1 hour old.\nThe Secondary PWS station (xml_ID2) in the conkywx config file is not defined.\nPlease update the conkywx config file to view current information.\n\n");
            $wx{alertdata} .= "Data not reliable - pws data is stale. Change xml_ID and add xml_ID2 in conkywx config file.";
        }
    }

    my $UsingXML2 = "${temppath}/UsingXML2";

    if ( -e $UsingXML2 ) {
        $wx{alertdata} .= "Choose another PWS Station - change xml_ID $conf{xml_ID} in conkywx config file. $conf{xml_ID} is not sending data!";
        unlink $UsingXML2
    }
}


sub parse_fct {

    # read downloaded forecast file if intellicast, msn or weather.com used as forecast sites

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    if (( $wxFctTitle eq "accu" ) or ( ! $wx{fctstr} )) {
        my ($fctstr);
        my $fct_data = "$temppath/fct_data.html" ;

        if ( -e $fct_data ) {
            {
            open (FILE, '<:crlf', "$fct_data") or die "$LMsg{no_open_file} forecast data file $!";
                local $/ = undef;
                $fctstr = <FILE> ;
            close (FILE);
            }

            if ( $wxFctTitle eq "darkSky" ) {
                my $dsFct = ${&get_line_num( \$fctstr, '<div class="dayDetails">','<div id="timeMachine"', 'str')};
                $dsFct =~ tr/\x0a\x20/ /s ;
                my $cntn = 1;
                while ($dsFct =~ m{<div class="summary">(.*?)</div>}g) {
                    my $val = ($1) ? $1 : 'NA' ;
                    for (@rainCond) {
                        if (index(lc($val),$_) != -1) {
                            $fctinfo{"d_" . $cntn . "_cor"} = 'Yes'; last
                        }
                        else {
                            $fctinfo{"d_" . $cntn . "_cor"} = 'No'
                        }
                    }
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn);
                }
                $cntn = 1;
                while ($dsFct =~ m{<span class="highTemp.*?"temp">(.*?)˚</span>.*?="lowTemp.*?="temp">(.*?)˚</span>}g) {
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$2/$1");
                }
                $cntn = 1;
                while ($dsFct =~ m{href="\/details\/.*?\/(.*?)\/us24\/en"}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day(&time2str("%a %d",$1));
                }
                # $wx{fctstr} = 1; # required to disable forecast re calc
            }
            elsif ( $wxFctTitle eq "intel" ) {
                $wx{fctstr} = ${&get_line_num(\$fctstr,"chart_dailyForecast.setDataXML","<dataset seriesName='Lows'",'str')};

                $wx{fctstr1} = ${&get_line_num(\$fctstr,'<table id="daily">','</table>','str')};

                my $cntn = 1;
                while ($wx{fctstr1} =~ m{title="(.*?)"\s*\/>}g) {
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn);
                }
                $cntn = 1;
                while ($wx{fctstr} =~ m{High:\s*(.*?)F\\nLow:\s*(.*?)F}g) {
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$1/$2");
                }
                $cntn = 1;
                while ($wx{fctstr} =~ m{Precip:\s*(.*?)\\n}g) {
                    $fctinfo{"d_" . $cntn++ . "_cor" } = $1;
                }
                $cntn = 1;
                while ($wx{fctstr} =~ m{label='(.*?)'}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day($1);
                }
            }
            elsif ( $wxFctTitle eq "wund" ) {
                $fctstr = ${&get_line_num(\$fctstr,'"forecast":',');','str')};
                $fctstr =~ tr/a-zA-Z0-9_,.\/: //cd;
                # msgout($fctstr);
                my $cntn = 1;
                while ($fctstr =~ m{,condition:\s+(.*?),icon:.*?day:}g) {
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn)
                }
                $cntn = 1;
                while ($fctstr =~ m{high:\s+(.*?),low:\s+(.*?),condition:.*?day:}g) {
                    # msgout($cntn, $1, $2);
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$1/$2")
                }
                $cntn = 1;
                while ($fctstr =~ m{iso8601:.*?pop:\s+(.*?),.*?day:}g) {
                    $fctinfo{"d_" . $cntn++ . "_cor" } = "$1%"
                }
                $cntn = 1;
                while ($fctstr =~ m{rfc822:\s+(.*?),\s+(.*?)\s+.*?iso8601}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day("$1 $2")
                }
            }
            elsif ( $wxFctTitle eq "msn" ) {
                my $fctdb = &get_line_num(\$fctstr,'data-aop="DailyForecast"','</section>', 'arr');
                for my $i (0..$#{$fctdb}) {
                    my $k = $$fctdb[$i];
                    if (index($k,'aria-label') != -1 )                { $wx{fctstr} .= $k    }
                    if (index($k,'"image skyimg"') != -1 )            { $wx{fctstr} .= $k    }
                    if (index($k,'<div title="Precipitation"') != -1) { $wx{fctstr}.=$k.";;" }
                }
                &fct_parser
            }
            elsif ( $wxFctTitle eq "accu" ) {
                # my @rainCond = qw(thunderstorm thunder tstorm t-storm tornado hurricane tropical blizzard mixed hail sleet flurries snow spray squalls rain shower downpour drizzle);

                $wx{fctstr} .=
                ${&get_line_num(\$fctstr,'<!-- /.feed-controls','<!-- /.feed-tabs','str',1)};
                $wx{fctstr} =~ tr/\x0a\x20/\x20/s;

                # msgout($wx{fctstr});
                my $cntn = 1;
                while ($wx{fctstr} =~ m{<span class="cond">(.*?)</span>.*?<!-- /.info -->}g) {
                    my $val = ($1) ? $1 : 'NA' ;
                    for (@rainCond) {
                        if (index(lc($val),$_) != -1) {
                            $fctinfo{"d_" . $cntn . "_cor"} = 'Yes'; last
                        }
                        else {
                            $fctinfo{"d_" . $cntn . "_cor"} = 'No'
                        }
                    }
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $val ,$cntn)
                }
                $cntn = 1;
                while ($wx{fctstr} =~ m{"large-temp(.*?)</span>.*?(?:temp-label|small-temp)(.*?)</span>.*?<!-- /.info -->}g) {
                    my ($ls,$rs) = ($1,$2);
                    if (index($rs,'tonight') != -1 ) {
                        $ls =~ tr/0-9.-//cd ;
                        ($rs) = ($ls ne '') ? $ls : '--' ;
                        $ls = '--';
                    }
                    else {
                        $ls =~ tr/0-9.-//cd ;
                        ($ls) = ($ls ne '') ? $ls : '--' ;
                        $rs =~ tr/0-9.-//cd ;
                        ($rs) = ($rs ne '') ? $rs : '--' ;
                    }
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$ls/$rs")
                }
                $cntn = 1;
                while ($wx{fctstr} =~ m{<a href="\#">(.*?)<.*?>.*?([\d]+)</h4>.*?<!-- /.info -->}g) {
                    $fctinfo{ "d_" . $cntn++ . "_day" } = get_short_day("$1 $2")
                }
            }
            elsif ( $wxFctTitle eq "wx" ) {
                my $xml1 = ${&get_line_num(\$fctstr,'<dayf>','</dayf>','str')};
                $xml1 =~ tr/\x0a\x20/\x20/s;
                my $cntn = 1;
                while ($xml1 =~ m{<part\s+p="d">.*?</icon>\s+<t>(.*?)</t>}g) {
                    $fctinfo{"d_" . $cntn++ . "_font"} = do_get_DI(2, $1 ,$cntn);
                }
                if ( $fctinfo{d_1_font} eq '' ) {
                    if ($xml1 =~ m{<part\s+p="n">.*?</icon>\s+<t>(.*?)</t>}) {
                        $fctinfo{d_1_font} = do_get_DI(2, $1, 1)
                    }
                }
                $cntn = 1;
                while ($xml1 =~ m{<hi>(.*?)</hi>\s+<low>(.*?)</low>}g) {
                    $fctinfo{"d_" . $cntn++ . "_temp"} = ch_fcunits("$1/$2");
                }
                $cntn = 1;
                while ($xml1 =~ m{<part\s+p="d">.*?<ppcp>(.*?)</ppcp>}g) {
                    $fctinfo{"d_" . $cntn++ . "_cor" } = $1 . '%'
                }
                $cntn = 1;
                while ($xml1 =~ m{<day\s+d=.*?t="(.*?)"\s+dt=".*?\s+(.*?)">}g) {
                    $fctinfo{"d_" . $cntn++ . "_day" } = get_short_day("$1 $2");
                }
            }
        }
    }

    my ($imageout2, $rpt );
    ## fixing forecast information
    my @fd = qw( day temp cond cor font image );
    for ($rpt=1; $rpt<=10; $rpt++ ) {
        foreach (@fd) {
            # msgout($_,$fctinfo{"d_" . $rpt . "_". $_}) if /cond/;
            if ( ! $fctinfo{"d_" . $rpt . "_". $_} ) {
                if ( $_ eq "image" ) {
                    $fctinfo{"d_" . $rpt . "_". $_} = "${iwxpathfcast}/na.png";
                }
                elsif ( $_ eq "font" ) {
                    $fctinfo{"d_" . $rpt . "_". $_} = "-";
                }
                else {
                    $fctinfo{"d_" . $rpt . "_". $_} = "NA" ;
                }
            }
            else {
                if ( -e "${temppath}/na.png" ) { unlink "${temppath}/na.png" ; }
            }
        }
    }

    $wx{icon} = ( not $wx{icon} or $wx{icon} eq 'NA') ? $wx{condition} : $wx{icon};
    my $iconval = ($wx{icon}) ? $wx{icon} : 'NA' ;
    $iconval =~ s/nt_//;
    $wx{condition} = ( not $wx{condition} ) ? $iconval : $wx{condition} ;
    $wx{condition} = ($wx{condition} eq 'NA') ? $fctinfo{d_1_cond} : $wx{condition} ;

    if ($wx{alertdata} ) {
        $wx{alertdata} =~ s{</a>(?:,)?}{ | }g;
        $wx{alertdata} =~ s/<.*?>|\(.*?\)|&nbsp|;/ /g;
        $wx{alertdata} =~ s/\s+/ /g;
        $wx{alertdata} =~ s{\|\s*$}{} ;
    }

    $wx{humidity} = ( $wx{humidity} ne 'NA' or $wx{humidity} ne 'null' and $wx{humidity} < 0 ) ? 'NA' : $wx{humidity} ;

    if ( $wx{humidity} eq 'NA' and $wx{temperature} and $wx{dewpoint} ) {
        # 100*(EXP((17.625*TD)/(243.04+TD))/EXP((17.625*T)/(243.04+T)))
        my $tempC = f_c($wx{temperature});
        my $dpC   = f_c($wx{dewpoint});
        $wx{humidity} = int(100*((112-(0.1*$tempC)+$dpC) / (112+(0.9*$tempC)))**8);
        $wx{humidity} = ( $wx{humidity} >= 90 ) ? 90 : $wx{humidity};
        # $wx{humidity} = int(100*((112-(0.1*$wx{temperature})+$wx{dewpoint}) / (112+(0.9*$wx{temperature})))**8)
    }
    else { $wx{humidity} = 'NA' }

    if ( not $wx{solarRad} or $wx{solarRad} eq 'NA' ) {
        $wx{solarRad} = ( $solValue ) ? $solValue : 'NA'
    }

    %TodayInfo = (
        Location_Name => $wx{city},             Station_Name => $wx{name},
        Station_ID    => $wx{nameid},           Lat1         => $wx{latitude},
        Long1         => $wx{longitude},        Updated      => $wx{updated},
        curr_icon     => $wx{icon},             Temp_now     => $wx{temperature},
        condition     => $wx{condition},        Visibility   => $wx{visibility},
        CLH1          => $wx{cloud1h},          CLC1         => do_get_DI(3,$wx{cloud1t},0),
        CLH2          => $wx{cloud2h},          CLC2         => do_get_DI(3,$wx{cloud2t},0),
        CLH3          => $wx{cloud3h},          CLC3         => do_get_DI(3,$wx{cloud3t},0),
        CLH4          => $wx{cloud4h},          CLC4         => do_get_DI(3,$wx{cloud4t},0),
        Humidity      => $wx{humidity},         Rainfall     => $wx{precip_today},
        SnowDepth     => $wx{snow},             Pressure     => $wx{pressure},
        Dew_Point     => $wx{dewpoint},         METAR        => $wx{metar},
        pollen        => $wx{pollen},           Wind_Speed   => $wx{wind_speed},
        Wind_From     => $wx{wind_dir},         Wind_Compass => $wx{wind_dir_degrees},
        Wind_Gust     => $wx{wind_gust_speed},  solarRad     => $wx{solarRad},
        );

    if ($TodayInfo{Wind_Compass} and $TodayInfo{Wind_Compass} == 0) {
        $TodayInfo{Wind_Compass} = '0 '
    }

    my ($check1); $rpt = 0;
    while ( my ($key1, $value1) = each %TodayInfo ) {
        if ($value1) {
            $check1 = check_num( $value1, $key1 );
            $TodayInfo{$key1} = $check1;
        }
        else {
            $value1 = 'NA';
            $TodayInfo{$key1} = $value1;
        }
        # msgout "$rpt: $key1 =>$value1";
    }

    if ( $TodayInfo{Updated} eq "NA" ) {
        msgout "\n\nError - Update time is NA. Use an alternate website for weather.\nPlease mail a message to the author with the Debug file.\nIn a terminal window type conkywx and choose option 4. Thanks\n\n"
    }
}

sub get_uvdata {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my $uvdata = "$conkywx_internal_dir/uvdata_$tempname";

    if ($TodayInfo{Lat1} and $TodayInfo{Long1}) {
        if ($TodayInfo{Lat1} =~ /s|S/)  {
            ($TodayInfo{Lat1}) = $TodayInfo{Lat1} =~ m{(\d+(\.\d+)?)};
            $TodayInfo{Lat1} *=  -1;
        }
        else {
            ($TodayInfo{Lat1})  = $TodayInfo{Lat1}  =~ m{(-?\d+(\.\d+)?)};
        }
        if ($TodayInfo{Long1} =~ /W|w/)  {
            ($TodayInfo{Long1}) = $TodayInfo{Long1} =~ m{(\d+(\.\d+)?)};
            $TodayInfo{Long1} *=  -1
        }
        else {
            ($TodayInfo{Long1}) = $TodayInfo{Long1} =~ m{(-?\d+(\.\d+)?)}
        }
    }

    my $uv_url = 'http://www.temis.nl/uvradiation/nrt/uvindex.php?lon='. "$TodayInfo{Long1}\\&lat=$TodayInfo{Lat1}" ;

    # if ( $copytmp eq '__' ) {
        ### stat command 7th item is size and 9th is modify time
        if ( not -e $uvdata or (stat ($uvdata))[7] == 0 or ($nowtime - (stat ($uvdata))[9]) > 388800  ) {
            `wget -q  -o /dev/null --tries=99 --timeout=30 $uv_url -O $uvdata`;
        }
    # }

    my ( $uvstr  ) ;
    {
    open (FILE, "<", $uvdata) or die "$LMsg{no_open_file} uvdata file $!";
        local $/ = undef;
        $uvstr = <FILE> ;
    close (FILE);
    }

    if (index($uvstr,'Invalid input; please try again') != -1) {
        unlink $uvdata;
        die "\nError :( get_uvdata: User position not correct!!\n\tPerhaps add one in the config file. $!"
    }

    my $uvtbl = ${&get_line_num( \$uvstr, '<table border=2','</table>','str')};
    $uvtbl =~ tr/\n//d;
    my ($cloud_factor,$time_factor);
    for (split('</tr>',$uvtbl)) {
        my ($uvt,$uvn) = (m{;\s*(.*?)\s*</td.*rap>\s*(.*?)\s*</td.*</td})
        ? ($1,$2): ('NA','NA') ;
        $TodayInfo{uv_max} = $uvn if $uvt eq "$day $monAbb $year";
    }

    my $teststr = ($wx{cloud4t} and $wx{cloud4t} ne 'NA')
    ? $wx{cloud4t}
    : ($wx{cloud3t} and $wx{cloud3t} ne 'NA')
    ? $wx{cloud3t}
    : ($wx{cloud2t} and $wx{cloud2t} ne 'NA')
    ? $wx{cloud2t}
    : ($wx{cloud1t} and $wx{cloud1t} ne 'NA')
    ? $wx{cloud1t}
    : ($wx{uvwx} ne 'na')
    ? $wx{uvwx}
    : "Clear" ;
    $teststr = lc($teststr);

    my $cond1 = 'overcast|cloudy|ovc|tstorm|sleet|rain|snow';
    my $cond2 = 'broken clouds|bkn|mcloudy|fog|mist';
    my $cond3 = 'scattered clouds|sct|pcloudy|dust|smoke|haze';

    if    (index($cond1,$wx{uvwx}) != -1 ) { $cloud_factor = 0.31 }
    elsif (index($cond1,$teststr ) != -1 ) { $cloud_factor = 0.31 }
    elsif (index($cond2,$teststr ) != -1 ) { $cloud_factor = 0.71 }
    elsif (index($cond3,$teststr ) != -1 ) { $cloud_factor = 0.89 }
    else { $cloud_factor = 1 }

    if ( $wx{curMin} < $wx{srise} or $wx{curMin} > $wx{sset} )
        { $time_factor = 0 }
    elsif ( $wx{curMin} < $wx{strans} ) {
        $time_factor = ($wx{curMin} - $wx{srise})/($wx{strans} - $wx{srise})
    }
    elsif ( $wx{curMin} > $wx{strans} ) {
        $time_factor = ($wx{sset} - $wx{curMin})/($wx{sset} - $wx{strans})
    }
    elsif ( $wx{curMin} == $wx{strans} ) {
        $time_factor = 1;
    }

    my $elevation = ($conf{elevation})
    ? $conf{elevation}
    : ($wx{elevation})
    ? $wx{elevation}
    : 0 ;

    ($elevation) = ($elevation =~ m{(-?\d+(\.\d+)?)})
        ? ((((ft_mtr($1))/1000) * 0.06 ) + 1 ) : 1 ;

    if ( defined $TodayInfo{uv_max} ) {
        $TodayInfo{uv_now} = sprintf("%.1f",$TodayInfo{uv_max} * $cloud_factor * $time_factor * $elevation);
        $TodayInfo{uv_now} = uv_pollen($TodayInfo{uv_now}, 'UV');
    }
    else {
        $TodayInfo{uv_max} = 0;
        $TodayInfo{uv_now} = 0;
        msgout("\n\nWarning: The Max UV is not defined - please send a DEBUG report to the author.\n\n")
    }
}


sub get_tides {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( @tset, $cnt, $tide_data, ) ;
    if ($conf{tides_url}) {
        my $ts = (split('/',$conf{tides_url}))[4];
        $tide_data = "$conkywx_internal_dir/tide_data_${tempname}_$ts";
    }

    my %tdhl  = ( "High Tide" => "HT", "Low Tide"  => "LT" );
    my $tForm = $timeFormat{$conf{Time_format}} // $conf{Time_format} ;

    my $tideFormat = sub {
        my ($time1, $ht1, $htlt1) = @_;
        $time1    = &time2str($tForm, $time1);
        $ht1      = &convert_unit($ht1,'Tide');
        $htlt1    = $tdhl{$htlt1};
        return "${time1} ${ht1} ${htlt1}";
    };
    if ($tide_data and -e $tide_data ) {
        my ( $tidestr  ) ;
        {
        open (FILE, "<", "$tide_data") or die "$LMsg{no_open_file} main data file $!";
            local $/ = undef;
            $tidestr = <FILE> ;
        close (FILE);
        }
        my $ttime = "${hour}:${min}:${sec}" ;

        my ($td_range) = (${&get_line_num(\$tidestr,'The largest known tidal range','</p>','str')} =~ m{range.*is.*m\s+(.*?)feet}) ? convert_unit($1,"Tide") : "NA";

        my ($td_loc) = (${&get_line_num(\$tidestr,'<section class="tide-events not_in_print">','</h2>','str')} =~ m{<h2>(.*?)\s*Tide})
        ? do_cleanup_lang( 2, \$1) : "Tide table:";

        $td_loc .= ": MaxRange:$td_range";

        my ($tdtext) = ${&get_line_num( \$tidestr, '<section class="tide-events not_in_print">','</section>','str')};

        $tdtext =~ tr/\x0a//d;
        $tdtext =~ s/<td class="date" rowspan=".*?">/\n<td class="date">/g;

        for my $j (split '\n', $tdtext) {
            my (%td);
            if ( $j =~ m{<td class="date">(.*?)</td>} ) { $td{day} = $1 }
            my $cntn=1;
            while ($j =~ m{time tide">\s*(.*?)\s*</td>.*?imperial">(.*?)\s*f.*?</span>.*?"tide">(.*?)</td>}g) {
                # msgout($cntn,$1,$2,$3);
                $td{$cntn.'time'} = $1;
                $td{$cntn.'ht'}   = $2;
                $td{$cntn.'htlt'} = $3;
                $cntn++;
            }
            push(@tset, {%td}) if $td{day};
        }
        my $tide_days_opt = ($conf{tide_days_opt}) ? $conf{tide_days_opt} : 1;
        my $tide_days_num = ($conf{tide_days_num}) ? $conf{tide_days_num} : 7;
        my $tide_date = ($conf{tide_date}) ? $conf{tide_date} : 'monday';

        open (TFILE, '>', "$temppath/tides" ) or die "$LMsg{no_open_file} tides file $!";
        print TFILE $td_loc;
        my $ydayt;
        $yday += 0;
        for my $i ( 0 .. $#tset ) {
            my $cnt;
            my $cnt2 = 1;
            for my $role ( keys %{ $tset[$i] } ) {
                if ( $role eq 'day') {
                    $ydayt = &time2str( '%j',"$tset[$i]{$role} $year $ttime") if $i == 0;
                    if ( $tide_days_opt == 1 ) {
                        $cnt = ( $ydayt == $yday ) ? 1 : undef
                    }
                    elsif ( $tide_days_opt == 2 ) {
                        $cnt = ( $ydayt <= $yday and $ydayt > ($yday - $tide_days_num) ) ?
                        1 : undef
                    }
                    elsif ( $tide_days_opt == 3 ) {
                        $cnt = ( $ydayt >= $yday and $ydayt < ($yday + $tide_days_num) ) ?
                        1 : undef
                    }
                    elsif ( $tide_days_opt == 4 ) { $cnt = 1 }
                    else { undef $cnt }
                    $ydayt += 1;
                }
                if ( $tide_days_opt == 5) {
                    if ( $tset[$i]{'day'} =~ /\b$tide_date/i ) {
                        print TFILE "\n$tset[$i]{day}" if $tset[$i]{$cnt2.'time'};
                        while ($tset[$i]{$cnt2.'time'}) {
                            my $prn = &$tideFormat($tset[$i]{$cnt2.'time'}, $tset[$i]{$cnt2.'ht'}, $tset[$i]{$cnt2.'htlt'});
                            print TFILE "\n$prn";
                            $cnt2++;
                        }
                    }
                }
                else {
                    if ($cnt) {
                        print TFILE "\n$tset[$i]{day}" if $tset[$i]{$cnt2.'time'};
                        while ($tset[$i]{$cnt2.'time'}) {
                            my $prn = &$tideFormat($tset[$i]{$cnt2.'time'}, $tset[$i]{$cnt2.'ht'}, $tset[$i]{$cnt2.'htlt'});
                            print TFILE "\n$prn";
                            $cnt2++;
                        }
                    }
                }
            }
        }
        close(TFILE);
    }
    else {
        unlink "$temppath/tides" ;
    }
}  # end sub get_tides

# -----------------------------------------------------------------------
# NOTE: Following functions are called from function check_num
# -----------------------------------------------------------------------

# -----------------------------------------------------------------------
# convert units
# -----------------------------------------------------------------------
sub convert_unit {
    my ($x, $y) = @_;

    # if ( $conf{Units_to_Use} ) {
    #     ( $y eq 'Depth'  ) ? return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x))) :
    #         return(sprintf($convUnits{$y}[1], $x))
    # } else {
    #     ( $y eq 'Depth'  ) ? return(sprintf($convUnits{$y}[1], $x)) :
    #         return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
    # }
    if ( $conf{Units_to_Use} ) {
        ( $y eq 'Depth'  )
            ? return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
            : ( $accuUnitWx and index($accuUnitWx, 'C<') != -1 )
            ? return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
            : return(sprintf($convUnits{$y}[1], $x))
    } else {
        ( $y eq 'Depth'  )
            ? return(sprintf($convUnits{$y}[1], $x))
            : ( $accuUnitWx and index($accuUnitWx, 'C<') != -1 )
            ? return(sprintf($convUnits{$y}[1], $x))
            : return(sprintf($convUnits{$y}[1], $convUnits{$y}[0]($x)))
    }
}

my %UVthing = (
    '1'  => 'Min', '6'  => 'High',
    '2'  => 'Low', '7'  => 'High',
    '3'  => 'Mod', '8'  => 'V.High',
    '4'  => 'Mod', '9'  => 'V.High',
    '5'  => 'Mod', '10' => 'V.High',
    '11' => 'V.High'
    );
my %Pthing = (
    '1'  => 'Low', '6'  => 'Mod',
    '2'  => 'Low', '7'  => 'High',
    '3'  => 'Low', '8'  => 'High',
    '4'  => 'Low', '9'  => 'High',
    '5'  => 'Mod', '10' => 'High',
    );

# -----------------------------------------------------------------------
# handle all UV + Pollen
# -----------------------------------------------------------------------
sub uv_pollen {
    my ($x, $y) = @_;

    if ( $y eq "UV" ) {
        if    ( $x <= 0 ) { return "NA"     }
        elsif ( $x >= 11) { return "$x:Ext" }
        else  { return "$x:$UVthing{&round($x + 0.5)}" }
    }
    if ( $y eq "pollen" )    {
        if    ( $x <= 0 ) { return "NA"     }
        elsif ( $x >= 10) { return "$x:Ext" }
        else { return "$x:$Pthing{&round($x + 0.5)}" }
    }
}

# -----------------------------------------------------------------------
# handle forecast temps
# -----------------------------------------------------------------------
sub ch_fcunits {
=comments
    Requirement
    [1] Hide all deg symbols (example: 95/63)
    [2] Show all deg symbols (example: 95°/63°)
    [3] Show all deg symbols, depending on Unit_to_Use (metric: 35°C/17°C) (imperial: 95°F/63°F)
    [4] Show deg symbol ONLY, after forecast pair (example: 95/63°)
    [5] Show deg symbol AND unit, after forecast pair (example: 95/63°F) [Default]
=cut

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($x) = @_ ;
    if ( $x eq 'NA' ) {
        return ('NA');
    }
    else {
        my ($a, $b, $c, $xLeft, $xRight, $u_tempfc, $outLeft, $outRight);
        # my $degsym_at_end = 5;
        $x =~ tr/\x20\x09//d;     # get rid of ALL spaces
        if (index($x, '/') != -1) {
            ($xLeft, $xRight) = split(/\//, "$x") ;
        }

        $xLeft  = ( not exists $naList{$xLeft}  and $xLeft  += 0 ) ? $xLeft  : -999;
        $xRight = ( not exists $naList{$xRight} and $xRight += 0 ) ? $xRight : -999;

        # if ( $conf{Units_to_Use} ) { $u_tempfc = "F" }
        # else {
        #     $xLeft  = sprintf("%1.0f", f_c($xLeft));
        #     $xRight = sprintf("%1.0f", f_c($xRight));
        #     $u_tempfc = "C";
        # }
        if ( $conf{Units_to_Use} ) {
            if ( $accuUnitFct and index($accuUnitFct, 'C<') != -1 ) {
                $xLeft  = sprintf("%1.0f", c_f($xLeft));
                $xRight = sprintf("%1.0f", c_f($xRight));
                $u_tempfc = "F"
            }
            else {
                $u_tempfc = "F"
            }
        }
        else {
            if ( $accuUnitFct and index($accuUnitFct, 'C<') != -1 ) {
                $u_tempfc = "C"
            }
            else {
                $xLeft  = sprintf("%1.0f", f_c($xLeft));
                $xRight = sprintf("%1.0f", f_c($xRight));
                $u_tempfc = "C"
            }
        }

        for ($conf{show_degsym_fc_temp}) {
            if    (/1/) {$b=""          ; $c=""           }
            elsif (/2/) {$b="°"         ; $c="°"          }
            elsif (/3/) {$b="°$u_tempfc"; $c="°$u_tempfc" }
            elsif (/4/) {$b=""          ; $c="°"          }
            elsif (/5/) {$b="°"         ; $c="°$u_tempfc" }
            else        {$b=""          ; $c=""           }
        }

        if    ($xLeft  < -150 ) { $a = "--/$xRight${c}" }
        elsif ($xRight < -150 ) { $a = "$xLeft${b}/--"  }
        else {$a = "$xLeft${b}/$xRight${c}"}

        return ($a);
    }
}

# -----------------------------------------------------------------------
# convert days long to short format - eg: Sunday to Sun
# -----------------------------------------------------------------------
sub get_short_day {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ($x,$y) = @_ ;

    if (! $y) { $y = 0 }

    if ( $x eq "NA" ) {return 'NA' }
    else {
        my ($a, $b, $c );

        # $x = "Sun Dec 9";
        $x =~ s/tod.*\s+/$wdayAbb /i;
        # msgout($x);

        # msgout($wdayAbb,$wdayFull);

        $x =~ s/\s+/ /g;
        my @e1 = ( $x =~ /\s+/g ) ;
        my $f = scalar(@e1);
        if ( $f == 0 ) {
            return $x;
        }
        elsif ( $f == 2 ) {
            ($a, $b, $c) = split( " ", $x);
            # msgout($d1{$a}, $b, $m{$c});
            if ($y == 1) { return "$fd_d{$a} $b $m_sm{$c}" }
        }
        else {
            ($a, $c) = split( " ", $x);
        }
        # if ( ! /Tonight|Today/ ) {
        # if ( length($a) > 3 or $a !~ /Tonight|Today/ ) { $a = substr($a, 0, 3) }
        if ( length($a) > 3 ) { $a = substr($a, 0, 3) }
        # msgout($f, $a, $b, $c, $x);
        # msgout( '0' x (2 - length $c) . $c);

        for ($conf{use_short_date}) {
            if    ( /0/ ) { return "$d_fd{$a} $c" if $a and $c and exists $d_fd{$a} }
            elsif ( /1/ ) {
                if ( length($c) == 1 ) { $c ='0' x (2 - length($c)) . $c }
                return "$a $c"
            }
            elsif ( /2/ ) { $c = scalar($c+0); return "$d_fd{$a} $c"       }
            elsif ( /3/ ) { $c = scalar($c+0); return "$a $c"              }
        }
    }
    #else { return ("NA"); }
}
# end if check_num function calls

# -----------------------------------------------------------------------
# NOTE: Following functions call the full wx data
# -----------------------------------------------------------------------

# -----------------------------------------------------------------------
# NOTE: Apparent temp calculation based on US Navy / Australian derived Formula
# -----------------------------------------------------------------------
sub get_apptemp {
    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $app_temp, $a_temp, $a_hum, $a_ws );
    ($a_temp) = ($TodayInfo{Temp_now} =~ m{(-?\d+(\.\d+)?)}) ? $1 : 'NA';
    ($a_hum)  = $TodayInfo{Humidity} =~ m{(\d+)} ? $1 : 'NA';
    ($a_ws)   = $TodayInfo{Wind_Speed} =~ m{(\d+(\.\d+)?)} ? $1 : 0;

    if ( $conf{Units_to_Use} ) {
        $a_temp = ($a_temp ne 'NA') ? f_c($a_temp) : 'NA';
        $a_ws   = ($a_ws != 0)      ? mi_km($a_ws) : 0   ;
    }

    if ( $a_temp eq "NA" ) {
        $TodayInfo{app_temp} = "NA";
    }
    elsif ( $a_hum eq "NA" or $a_hum == 0 ) {
        $TodayInfo{app_temp} = $TodayInfo{Temp_now};
    }
    else {
        # conversion to wind speed m/s is built into the formula
        my $h2o_vp = ( $a_hum / 100 * 6.105 * 2.718281828 ** (( 17.27 * $a_temp ) / ( 237.7 + $a_temp )));
        $app_temp = ( $a_temp + (0.33 * $h2o_vp) - (0.7 *( $a_ws * 0.27777778)) - 4.00 );

        if ( $conf{Units_to_Use} ) {
            $TodayInfo{app_temp} = sprintf( "%1.0f$u_temp", c_f($app_temp) );
        }
        else {
            $TodayInfo{app_temp} = sprintf( "%1.0f$u_temp", $app_temp );
        }
    }
}

# -----------------------------------------------------------------------
# Wind direction and wind - direction and Gust Handling
# -----------------------------------------------------------------------
sub get_bearing_font {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my @brn1 = qw(  9 : ; < = > ? @ 1 2 3 4 5 6 7 8 % ! -);
    my @brn2 = qw/  I J K L M N O P A B C D E F G H % " -/;
    my @brn3 = ( qw/Y Z/, '\[', '\\', '\]', qw/^ _ ` Q R S T U V W X/, '%', '\#', '-');
    my @brn4 = ( qw/i j k l m n o p a b c d e f g h %/,'\#','-'); # uses the " character from set 3
    my @brnI = qw/  s ssw sw wsw w wnw nw nnw n nne ne ene e ese se sse un var na/;
    my @crd1 = (qw/NORTH NNE NE ENE EAST ESE SE SSE SOUTH SSW SW WSW WEST WNW NW NNW/,
        "UNKNOWN", "VAR", "NA" );
    my (%brf, %bri, @UserSet);

    my $wfn = length($conf{wx_wind_fontset});
    if ( $wfn eq 1 ) {
        $conf{wx_wind_fontset} = ($conf{wx_wind_fontset} =~ /^[1-4]$/) ? $conf{wx_wind_fontset} - 1 : 0;
    }
    else {
        @UserSet = split( " ", $conf{wx_wind_fontset} );
    }

    my @bfset = ( \@brn1, \@brn2, \@brn3, \@brn4 );
    my $xb = 0;
    foreach(@crd1) {
        if ( $wfn eq 1 ) {
            $brf{$_} = $bfset[$conf{wx_wind_fontset}][$xb];
        }
        else {
            $brf{$_} = $UserSet[$xb];
        }
        $bri{$_} = $brnI[$xb];
        $xb++;
    }
    # my ($key1, $value1);
    # while ( ($key1, $value1) = each %brf ) {
    #   msgout "$key1 =>$value1";
    # }

    my ( @bearing_font_array, @push_out, $fn1, $ic1, $windspeed_trend, %Wind_Now );

    $Wind_Now{Wind_Speed} =  $TodayInfo{Wind_Speed};
    $Wind_Now{Wind_Speed} =~ s/[ ].*//;
    # $Wind_Now{Wind_Speed} =~ s/variable/var/i;

    $Wind_Now{Wind_Gust} =  $TodayInfo{Wind_Gust};
    $Wind_Now{Wind_Gust} =~ s/[ ].*//;
    # $Wind_Now{Wind_Gust} =~ s/variable/var/i;

    $Wind_Now{Wind_From} =  $TodayInfo{Wind_From};
    $Wind_Now{Wind_From} =~ s/variable/var/i;
    # $Wind_Now{Wind_Speed} = "Calm";

    # msgout "from=$Wind_Now{Wind_From}, speed=$TodayInfo{Wind_Speed}";

    if ( $Wind_Now{Wind_Speed} eq 'NA' ) {
        # $Wind_Now{Wind_Speed} = "NA";
        $windspeed_trend = "NA";
        if (( $Wind_Now{Wind_Gust} eq "NA" or $Wind_Now{Wind_Gust} == 0 )) {
            $Wind_Now{Wind_From} = "NA";
            $Wind_Now{Wind_Gust} = "NA";
        }
        else {
            $Wind_Now{Wind_From} = "Var";
            $Wind_Now{Wind_Gust} = "$Wind_Now{Wind_Gust} $u_speed";
        }
    }
    elsif (( $Wind_Now{Wind_Speed} eq "Calm" or $Wind_Now{Wind_Speed} == 0 )) {
        $Wind_Now{Wind_Speed} = "Calm";
        $windspeed_trend = 0;
        if (( $Wind_Now{Wind_Gust} eq "NA" ) || ( $Wind_Now{Wind_Gust} == 0 )) {
            $Wind_Now{Wind_From} = "NA";
            $Wind_Now{Wind_Gust} = "NA";
        }
        else {
            $Wind_Now{Wind_From} = "Var";
            $Wind_Now{Wind_Gust} = "$Wind_Now{Wind_Gust} $u_speed";
        }
    }
    else {
        $windspeed_trend = $Wind_Now{Wind_Speed};
        $Wind_Now{Wind_Speed} = "$Wind_Now{Wind_Speed} $u_speed";
        $Wind_Now{Wind_From} = ($Wind_Now{Wind_From} eq "NA" )
        ? $Wind_Now{Wind_From} = "Var"
        : $Wind_Now{Wind_From} = $Wind_Now{Wind_From} ;
        if (( $Wind_Now{Wind_Gust} eq "NA" ) || ( $Wind_Now{Wind_Gust} == 0 )) {
            $Wind_Now{Wind_Gust} = "NA";
        }
        else {
            $Wind_Now{Wind_Gust} = "$Wind_Now{Wind_Gust} $u_speed";
        }
    }

    $fn1 = ($brf{uc($Wind_Now{Wind_From})}) ? $brf{uc($Wind_Now{Wind_From})} : "%" ;
    $ic1 = ($bri{uc($Wind_Now{Wind_From})}) ? $bri{uc($Wind_Now{Wind_From})} : "un" ;
    # msgout "from=$Wind_Now{Wind_From}, speed=$Wind_Now{Wind_Speed}";
    my $windicon_org = "${iwindpath}/${ic1}.png";
    # my $windicon_cp = "${temppath}/dailyimg_${cimage}.png";


    $TodayInfo{Wind_Speed} = $Wind_Now{Wind_Speed};

    # msgout $TodayInfo{Wind_Compass};

    if ( $Wind_Now{Wind_From} eq "Var" ) {
        $TodayInfo{Wind_Compass} = "Var";
    }
    elsif ( $Wind_Now{Wind_From} eq "NA" ) {
        $TodayInfo{Wind_Compass} = "NA";
    }

    $TodayInfo{Wind_From}         = $Wind_Now{Wind_From};
    $TodayInfo{Wind_Gust}         = $Wind_Now{Wind_Gust};
    $TodayInfo{wind_speed_trend}  = $windspeed_trend;
    $TodayInfo{wind_bearing_font} = $fn1;

    # msgout($fn1,$brf{uc($Wind_Now{Wind_From})});
    # copy($windicon_org, $nowWindDir);

    $TodayInfo{nowWindDir} = $windicon_org;


    if ($debug) {
        foreach my $wxkey ( sort keys %TodayInfo ) {
            push @push_out, "$wxkey = $TodayInfo{$wxkey}\n";
        }
        do_debug( "today (+xml)", @push_out );
        # msgout( "$LMsg{pass_today}" );
    }
}

sub get_magellan {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $flag,$t_zone,$moonImage, $rfact, $present_hour );
    my $wx_tzone_A = $wx_tzone / 3600 ;
    if ( ! ($conf{t_zone_pick}) ) {
        $t_zone = $wx_tzone_A;
        unlink $die_error if ( -e $die_error )
    }
    else {
        $t_zone = fix_tzone($conf{t_zone_pick}) ;
        my $tz_err_msg = "Error: :(  t_zone_pick user value $conf{t_zone_pick} in config file not valid: $!" ;
        if ( $t_zone == -999 ) { die_errors($tz_err_msg) ; die $tz_err_msg }
        else { unlink $die_error if ( -e $die_error ) }
    }

    if ($conf{Latitude})  { $TodayInfo{Lat1}  = $conf{Latitude}  }
    if ($conf{Longitude}) { $TodayInfo{Long1} = $conf{Longitude} }

    if ($TodayInfo{Lat1})  { $TodayInfo{Lat1}  =~ s/"|°|\s*//g; }
    if ($TodayInfo{Long1}) { $TodayInfo{Long1} =~ s/"|°|\s*//g; }

    my $LatVal = ($TodayInfo{Lat1}  =~ m{(-?\d+(\.\d+)?)}) ? $1 : 'NA';
    my $LonVal = ($TodayInfo{Long1} =~ m{(-?\d+(\.\d+)?)}) ? $1 : 'NA';
    my $LatSign = substr( $TodayInfo{Lat1},  -1 );
    my $LonSign = substr( $TodayInfo{Long1}, -1 );

    if ( $TodayInfo{Lat1} ne "NA" || $TodayInfo{Long1} ne "NA" ) {
        if ($LatSign !~ /N|S|n|s/) {
            $TodayInfo{Lat1} = ($LatVal < 0) ? ($LatVal*-1)."S" : $LatVal."N";
        }
        else {
            if ( $LatVal < 0 and $LatSign =~ /N|n/ ) {
                die "\nError: :( \nLatitude sign mismatch: $TodayInfo{Lat1}";
            }
        }
        if ($LonSign !~ /E|W|e|w/) {
            $TodayInfo{Long1} = ($LonVal < 0) ? ($LonVal*-1)."W" : $LonVal."E";
        }
        else {
            if ( $LonVal < 0 and $LonSign =~ /E|e/ ) {
                die "\nError: :( \nLongitude sign mismatch: $TodayInfo{Long1}";
            }
        }
    }
    else {
        $TodayInfo{Lat1} = 0.1 ."N" ; $TodayInfo{Long1} = 0.1 . "E" ;
    }
    # msgout ">>>>>>>>>>$LatSign::$LonSign>>>$LatVal::$LonVal,$TodayInfo{Lat1},$TodayInfo{Long1}";
    # msgout ">>>>>>>>>>Lat:$TodayInfo{Lat1},Long:$TodayInfo{Long1}";

    my $t_format = $timeFormat{$conf{Time_format}} // $conf{Time_format};
    my %moonFormat = (
        '1' => "%b %d $t_format",
        '2' => "%b %d",
        );
    my $moon_format = $moonFormat{$conf{moon_time}} // $conf{moon_time} ;

    # $conf{moon_time} = "%b %d";
    # $conf{moon_time} =~ s/'//g;
    # $conf{moon_time} =~ tr/\"//;
    # msgout($conf{moon_time});

    my $nt_format = "%R" ; # get next day data in 24 hour format - fixes night images

    # msgout "$cwx_magellan -y $year -m $mon -d $day -t $t_zone -o $conf{moon_outline} -a $nt_format -b $moon_format -n 1 $TodayInfo{Lat1} $TodayInfo{Long1}"  ;

    my $magellan_tday = `"$cwx_magellan" -y $year -m $mon -d $day -t $t_zone -o $conf{moon_outline} -a "$t_format" -b "$moon_format" $TodayInfo{Lat1} $TodayInfo{Long1}` ;

    if ($debug) {
        msgout( "$LMsg{pass_mag}" );
        do_debug( "magellan today", "$magellan_tday");
    }

    $magellan_tday =~ tr/\x22//d;
    $magellan_tday =~ s/(?<=[\s])#.*\n/\n/;

    my $magPrintOut;
    for (split /\n/, $magellan_tday) {
        if (index($_,'[next_day_start]') != -1) { $flag = 1; next }
        if ( $flag ) {
            my ($k, $v) = split('=',$_,2);
            $Mag_Nday{$k} = $v if $k;
            if (index($_,'[next_day_end]') != -1) { undef($flag); next }
        }
        if ( ! $flag ) {
            my ($k, $v) = split('=',$_,2);
            if ($k ) {
                $Mag_2day{$k} = $v;
                $magPrintOut .= "$k=$v\n" ;
            }
        }
    }

    open (out_data, '>', "$temppath/magellan") or
                                die "$LMsg{no_open_file} magellan: save $!";
        print out_data $magPrintOut;
    close (out_data);

    $Mag_2day{civ_twilt_S} = $Mag_2day{civ_twilt_S} // "-";
    $Mag_2day{nau_twilt_S} = $Mag_2day{nau_twilt_S} // "-";
    $Mag_2day{ast_twilt_S} = $Mag_2day{ast_twilt_S} // "-";

    my $lu1 = ($TodayInfo{Updated} or $TodayInfo{Updated} ne 'NA' ) ?
        &time2str($t_format,$TodayInfo{Updated}) :
        &time2str($t_format,$nowtime);

    my $lu2 = ($wx{last_update_xml}) ? &time2str($t_format,$wx{last_update_xml}) : "NA" ;

    $moonImage = "$cwxpath/images/moonicons/$Mag_2day{moonImage}.png";

    # moon_dec_state - 1 is if observer North of moon, 0 if observer South of moon
    if ( $conf{Dynamic_Moon} ) {
        if ( $conf{Dynamic_Moon} == 1 ) { $present_hour = $hour; }
        else { $present_hour = &time2str("%H",$TodayInfo{Updated}); }

        my (%t0,%t1);
        $t0{$_} = 0 for (15,30); $t0{$_} = -45 for (1..14); $t0{$_} =  45 for (16..29);
        $t1{$_} = 0 for (15,30); $t1{$_} = -45 for (1..14); $t1{$_} =  45 for (16..29);

        ( $Mag_2day{moon_dec_state} == 0 )      ?
        $rfact = $t0{$Mag_2day{moonImage}} // 0 :
        $rfact = $t1{$Mag_2day{moonImage}} // 0 ;

        if ( $rfact != 0 ) {
            ( $Mag_2day{moon_dec_state} == 0 ) ?
            $TodayInfo{moon_rotate_val} = ((( $present_hour * 15 ) + $rfact) * -1) :
            $TodayInfo{moon_rotate_val} = ((( $present_hour * 15 ) + $rfact)) ;

            $TodayInfo{moonImage} = $moonImage;
        }
        else { $TodayInfo{moonImage} = $moonImage; $TodayInfo{moon_rotate_val} = 0 }
    }
    else { $TodayInfo{moonImage} = $moonImage; $TodayInfo{moon_rotate_val} = 0 }

    $TodayInfo{today_date} = &time2str($conf{today_date_format},$nowtime);

    my $dd1 = sprintf("%1.0f",(($Mag_Nday{length_day_raw} - $Mag_2day{length_day_raw}) * 3600 ));

    # msgout($Mag_Nday{length_day_raw}, $Mag_2day{length_day_raw});
    # msgout($Mag_Nday{percentilluminated},$Mag_2day{percentilluminated});

    if ( $dd1 < 0 ) {
        $TodayInfo{next_day_length_msg}   = "Shorter";
        $TodayInfo{next_day_length_trend} = $conf{fallingtrend};
        $dd1 *= -1;
    }
    elsif ( $dd1 > 0 ) {
        $TodayInfo{next_day_length_msg}   = "Longer";
        $TodayInfo{next_day_length_trend} = $conf{risingtrend};
    }
    else {
        $TodayInfo{next_day_length_msg}   = "Steady";
        $TodayInfo{next_day_length_trend} = $conf{steadytrend};
    }

    my $ds  =   $dd1 % 60;
    my $dm  =   (($dd1 / 60) % 60);
    $TodayInfo{next_day_length} =   sprintf("%02dm %02ds", $dm, $ds);

    $TodayInfo{next_day_length_trend} = ($TodayInfo{next_day_length_trend}) ? $TodayInfo{next_day_length_trend} : "~" ;

    # calculate moon rate of changes based on illumination
    $TodayInfo{moon_rate_of_change} = $Mag_Nday{percentilluminated} - $Mag_2day{percentilluminated};

    if ( -f "$temppath/old_data" ) {
        $wx{alertdata} .= "$LMsg{no_net} | ";
        $TodayInfo{fetchtime} = "$LMsg{no_net}";
    }
    else {
        my $ftime = (stat( $currdatam))[9];
        $TodayInfo{fetchtime} = &time2str($t_format,$ftime)
    }

    $TodayInfo{last_update} = ( -e "$currdatax" ) ? "$lu1 | $lu2 - $TodayInfo{Station_ID}" : "$lu1" ;
    $TodayInfo{last_update} = ( $wxFctTitle ) ? "$TodayInfo{last_update} :: ( $wxMainTitle + $wxFctTitle ) " : "$TodayInfo{last_update} :: ( $wxMainTitle )" ;
    # msgout($TodayInfo{last_update},'<<<<<<<<<<<<');

    my $currTz = 0;
    $wx{curMin} = ( $hour * 60 ) + $min + $currTz ;
    $wx{srise}  = ($Mag_2day{sun_rise_T}      =~ m{(\d+).(\d+)}) ? (($1 * 60) + $2) : 0;
    $wx{sset}   = ($Mag_2day{sun_set_T}       =~ m{(\d+).(\d+)}) ? (($1 * 60) + $2) : 0;
    $wx{strans} = ($Mag_2day{sun_mer_trans_T} =~ m{(\d+).(\d+)}) ? (($1 * 60) + $2) : 0;

    my ($dntest);
    # we calc here the night or day icons based on civil twilight
    $dntest = ( $wx{curMin} >= ($wx{sset} + 30 ) or $wx{curMin} <= ($wx{srise} - 30) ) ? 1 : 2 ;
    ($TodayInfo{NowFont}, $TodayInfo{condition}) = do_get_DI( 1, $TodayInfo{curr_icon}, 0, $dntest );

    # msgout(">>>$TodayInfo{condition}<<<") ;
    # $ftest ||= "NA";
    # msgout "$dntest, $TodayInfo{NowFont}, $hour:$min, $wx{srise}, $wx{sset}, $TodayInfo{condition}, $t_zone, $wx_tzone, $currTz, $wx{curMin}";

}

sub trend_graphs {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my $orghistin="$temppath/orghistin";
    my $orghistwu="$temppath/orghistwu";
    my $graphdata="$temppath/graphdata";
    my $histstr;

    # my $tst01 = "abc-123";
    # my $tst02 = ($tst01 =~ m{^([\+\-\d\.]+)$}) ne '' ? $tst01 : 0 ;
    # msgout($tst02);

    if ( -s $graphdata and ($nowtime - (stat ($graphdata))[9]) > 10800  ) {
        unlink $graphdata
    }

    if ( -s $graphdata ) {
        {
        open (FILE, "<:crlf", "$graphdata") or die "$LMsg{no_open_file} local history file $!";
            local $/ = undef;
            $histstr = <FILE> ;
        close (FILE);
        }

        $histstr =~ tr/\x0a\x20//d;

        ($wxtrend{time}, $wxtrend{temp}, $wxtrend{press}, $wxtrend{dp}, $wxtrend{hum}, $wxtrend{ws}, $wxtrend{wsg}, $wxtrend{sol}) = split(/:::/, $histstr);

        ( $wxtrend{temp}  ||= 0 ) .= ',';
        ( $wxtrend{press} ||= 0 ) .= ',';
        ( $wxtrend{dp}    ||= 0 ) .= ',';
        ( $wxtrend{hum}   ||= 0 ) .= ',';
        ( $wxtrend{ws}    ||= 0 ) .= ',';
        ( $wxtrend{wsg}   ||= 0 ) .= ',';
        ( $wxtrend{sol}   ||= 0 ) .= ',';
    }
    elsif ( -s $orghistwu ) {
    # if ( -s $orghistwu ) {
        {
        open (FILE, "<:crlf", "$orghistwu") or die "$LMsg{no_open_file} main data file $!";
            local $/ = undef;
            $histstr = <FILE> ;
        close (FILE);
        }
        my $cntr = -1;
        my ( $histType, @histD, $delim, $at1, $at2, $tdiff );
        my $histSize = 0;
        if (index($histstr,'<br>') != -1) { $delim = '<br>\s*\n'}
        else { $delim = '<br />\n'}
        for (split($delim,$histstr)) {$cntr++; push @{$histD[$cntr]},split( /,/,$_)}
        for my $i ( 1..$#histD ) {
            # msgout($histD[0][13]);
            if ( index($histD[0][4], 'Sea Level') != -1 ) { $histType = 'air'; $histSize = 1 }
            elsif ( index($histD[0][13], 'Solar') != -1 ) { $histType = 'sol' }
            else { $histType = 'NA' }

            if ( $histType ne 'NA' ) {
                ($tdiff) = $histD[$i][0] =~ m{.*:(\d+)\s+.*};
                last
            }
            else {
                $at1 = ($at2) ? $at2 : 0 ;
                if ($i >= 1 ) {
                    ($at2) = $histD[$i][0] =~ m{.*:(\d+):.*}
                }
                if ($i == 2 ) {
                    $tdiff = $at2 - $at1;
                    if ($tdiff > 27) { $histSize = ($#histD > 12 ) ? $#histD - 12 : 1 }
                    else { $histSize = 1 }
                    last
                }
            }
        }
        my @tstr;
        for my $i ( 1..$#histD ) {
            if ( $i >= $histSize ) {
                if ( $histType eq 'air' ) {
                    $histD[$i][0] =~ m{.*:(\d+)\s+.*};
                    if ( $1 == $tdiff ) {
                        $wxtrend{temp}  .= $histD[$i][1].',';
                        $wxtrend{dp}    .= $histD[$i][2].',';
                        $wxtrend{hum}   .= $histD[$i][3].',';
                        $wxtrend{press} .= $histD[$i][4].',';
                        $wxtrend{ws}    .= $histD[$i][7].',';
                        $wxtrend{wsg}   = '0,';
                        $wxtrend{sol}   = '0,';
                    }
                }
                elsif ( $histType eq 'sol' ) {
                    $histD[$i][0] =~ m{.*:(\d+)\s+.*};
                    if ( $i % 5 == 1 ) {
                        $wxtrend{temp}  .= $histD[$i][1].',';
                        $wxtrend{dp}    .= $histD[$i][2].',';
                        $wxtrend{hum}   .= $histD[$i][8].',';
                        $wxtrend{press} .= $histD[$i][3].',';
                        $wxtrend{ws}    .= $histD[$i][6].',';
                        $wxtrend{wsg}   .= $histD[$i][7].',';
                        $wxtrend{sol}   .= $histD[$i][13].',';
                    }
                }
                else {
                    if ( $i % 5 == 1 ) {
                        $wxtrend{temp}  .= $histD[$i][1].',';
                        $wxtrend{dp}    .= $histD[$i][2].',';
                        $wxtrend{hum}   .= $histD[$i][8].',';
                        $wxtrend{press} .= $histD[$i][3].',';
                        $wxtrend{ws}    .= $histD[$i][6].',';
                        $wxtrend{wsg}   = '0,';
                        $wxtrend{sol}   = '0,';
                    }
                }

            }
        }
    }
    elsif ( -s $orghistin ) {
        {
        open (FILE, "<:crlf", "$orghistin") or die "$LMsg{no_open_file} main data file $!";
            local $/ = undef;
            $histstr = <FILE> ;
        close (FILE);
        }
        my $intelHist = ${&get_line_num(\$histstr,'chart_pastObservations.setDataXML','chart>");',,'str')};
        my @tstr;
        @tstr           = $intelHist =~ m{Temp:.*?([\.\d]+)F}g;
        $wxtrend{temp}  = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Dew Point:.*?([\.\d]+)F}g;
        $wxtrend{dp}    = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Humidity:.*?([\.\d]+)%}g;
        $wxtrend{hum}   = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Pressure:.*?([\.\d]+)in}g;
        $wxtrend{press} = join(',', @tstr) . ',';
        @tstr           = $intelHist =~ m{Wind:.*?([\.\d]+)mph}g;
        $wxtrend{ws}    = join(',', @tstr) . ',';
        $wxtrend{wsg}   = '0,';
        $wxtrend{sol}   = '0,';
    }

    ### ensure values are numeric
    my @trendArr = qw(temperature pressure dewpoint humidity wind_speed wind_gust_speed solarRad);
    for my $k (@trendArr) {
        $wx{$k} =~ tr/\x20//d if $wx{$k};
        if ( $wx{$k} and $naList{$wx{$k}} ) { $wx{$k} = 0 }
    }

    if ( not -e $graphdata or $wxtrend{time} and $nowtime - $wxtrend{time} > 2700 ) {
    # if ( not -e $graphdata or $wxtrend{time} and $nowtime - $wxtrend{time} > 3510 ) {
    # if ( not -e $graphdata or $wxtrend{time} and $nowtime - $wxtrend{time} > 10 ) {
        ($wxtrend{temp}   ||= 0) .= ($wx{temperature}     ||= 0) ;
        ($wxtrend{press}  ||= 0) .= ($wx{pressure}        ||= 0) ;
        ($wxtrend{dp}     ||= 0) .= ($wx{dewpoint}        ||= 0) ;
        ($wxtrend{hum}    ||= 0) .= ($wx{humidity}        ||= 0) ;
        ($wxtrend{ws}     ||= 0) .= ($wx{wind_speed}      ||= 0) ;
        ($wxtrend{wsg}    ||= 0) .= ($wx{wind_gust_speed} ||= 0) ;
        ($wxtrend{sol}    ||= 0) .= ($wx{solarRad}        ||= 0) ;

        my @preTrendArr = qw(temp press dp hum ws wsg sol);
        for my $k (@preTrendArr) {
            my ($totalElements) = $wxtrend{$k} =~ tr/,// ;
            if ($totalElements > 12 ) {
                my $startElement = $totalElements - 12 ;
                $wxtrend{$k} = join ',',(split ',',$wxtrend{$k})[$startElement..$totalElements];
            }
        }

        my $sep = ":::\n";
        open (out_data, '>', $graphdata) or die "$LMsg{no_open_file} local history file : $!";
            print out_data "$nowtime${sep}$wxtrend{temp}${sep}$wxtrend{press}${sep}$wxtrend{dp}${sep}$wxtrend{hum}${sep}$wxtrend{ws}${sep}$wxtrend{wsg}${sep}$wxtrend{sol}\n";
        close (out_data);
    }

    $wxtrend{temp}  = &normalize( split(',', $wxtrend{temp} ) ) ;
    $wxtrend{press} = &normalize( split(',', $wxtrend{press}) ) ;
    $wxtrend{dp}    = &normalize( split(',', $wxtrend{dp}   ) ) ;
    $wxtrend{hum}   = &normalize( split(',', $wxtrend{hum}  ) ) ;
    $wxtrend{ws}    = &normalize( split(',', $wxtrend{ws}   ) ) ;
    $wxtrend{wsg}   = &normalize( split(',', $wxtrend{wsg}  ) ) ;
    $wxtrend{sol}   = &normalize( split(',', $wxtrend{sol}  ) ) ;

}

# -----------------------------------------------------------------------
# get forecast trends and arrow fonts
# -----------------------------------------------------------------------
sub get_forecast_trends {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( @fcarray, @ft_now, @ft_now_24, @full_fc_data, @full_fc_data1, @full_fc_data2, @trends_now, @trends_store, @trends_now_24, @trends_store_24, @nowdata, @nowstored, $trend1, $trend_arrow1, @trend, @trend_arrow );

    my %today_condF = (
        Rising  => 'Today will be WARMER than yesterday.',
        Falling => 'Today will be COOLER than yesterday.',
        Steady  => 'Today temperature will be the SAME as yesterday.',
        NA      => 'Forecast information is not available.',
    );
    my %today_condS = (
        Rising  => 'WARMER',
        Falling => 'COOLER',
        Steady  => 'SAME',
        NA      => 'NA',
    );

    # Done: - get yesterday temp stored in forecast table for comparison [Florian]
    # stored temp for comparison with today max forecast
    my $fcttemp = ($fctinfo{d_1_temp} =~ m{(\d+)(°)?/.*}) ? $1 : 'NA';

    @ft_now = ( $wx{pressure}      ||= 0,
                $wx{temperature}   ||= 0,
                $wx{wind_speed}    ||= 0,
                $wx{humidity}      ||= 0,
                $TodayInfo{uv_now} ||= 0,
                $wx{dewpoint}      ||= 0,
                $wx{visibility}    ||= 0,
                $wx{solarRad}      ||= 0,
                );

    for (@ft_now) { $_ = ( $_ =~ m{(-?\d+(\.\d+)?)} ) ? $1 : 0 }

    @ft_now_24 = ( $Mag_2day{sun_rise_T},
                   $Mag_2day{sun_set_T},
                   $Mag_2day{moon_rise_T},
                   $Mag_2day{moon_set_T},
                   $Mag_2day{length_visible_light_T},
                   $Mag_2day{length_day_T},
                   $Mag_2day{percentilluminated},
                   $fcttemp,
                );

    for (@ft_now_24) { $_ = ( $_ =~ m{(-?\d+(\.\d+)?)} ) ? $1 : 0 }

    @full_fc_data = ( "$nowtime @ft_now", "\n$nowtime @ft_now", "\n$nowtime @ft_now_24", "\n$nowtime @ft_now_24" );

    unless ( -e $forecast_trends ) {
        open (out_data, '>', "$forecast_trends") or die "$LMsg{no_open_file} forecast save : $!";
            print out_data "@full_fc_data";
        close (out_data);
    }

    open (fc_read, '<', "$forecast_trends") or die "$LMsg{no_open_file} forecast read : $!";
        @fcarray = <fc_read>;
    close (fc_read);

    @trends_now      = split(" ", $fcarray[0]);
    @trends_store    = split(" ", $fcarray[1]);        # we check time to this
    @trends_now_24   = split(" ", $fcarray[2]);
    @trends_store_24 = split(" ", $fcarray[3]);        # we check time for this

    ### reset forecast data if old data
    if ( scalar( @trends_store ) < 9 or scalar( @trends_store_24 ) < 9 ) {
        unlink( $forecast_trends );
        exec($^X, $0, @ARGV);
    }

    # handle moon reaching 0 or 100 % illumination
    if (( $ft_now_24[6] == 100 ) and ( $trends_now_24[7] != 100 )) {
        $trends_now_24[7]     =   100;
        $trends_store_24[7]   =   100;
    }
    elsif (( $ft_now_24[6] == 0 ) and ( $trends_now_24[7] != 0 )) {
        $trends_now_24[7]     =   0;
        $trends_store_24[7]   =   0;
    }
    elsif (( $ft_now_24[6] eq "NA" ) or ( $trends_now_24[7] eq "NA" )) {
        unlink "$forecast_trends"
    }

    # Consolidate present and stored data
    @nowdata    =   ( @ft_now, @ft_now_24 );
    @nowstored  =   ( @trends_now[1..$#trends_now], @trends_now_24[1..$#trends_now_24] );

    ### limits for the 2 data sets
    my $hrLimit  = $#ft_now ;
    my $dayLimit = $hrLimit + 4;
    # msgout($hrLimit,$dayLimit,@nowstored);

    foreach my $l (0..$#nowdata) {
        # msgout($l,$nowdata[$l]);
        if (( "$nowdata[$l]" eq "NA" ) or ("$nowstored[$l]" eq "NA" )) {
            $trend1         =   "-";
            $trend_arrow1   =   "-";
        }
        else {
            if ( $nowdata[$l] == $nowstored[$l] ) {
                $trend1         =   "Steady" ;
                $trend_arrow1   =   $conf{steadytrend} ;
            }
            elsif ( $nowdata[$l] < $nowstored[$l] ) {
                if (( $l > $hrLimit ) && ( $l < $dayLimit )) {
                    $trend1     =   "Earlier" ;
                }
                else {
                    $trend1     =   "Falling" ;
                }
                $trend_arrow1   =   $conf{fallingtrend} ;
            }
            elsif ( $nowdata[$l] > $nowstored[$l] ) {
                if (( $l > $hrLimit ) && ( $l < $dayLimit )) {
                    $trend1     =   "Later" ;
                }
                else {
                    $trend1     =   "Rising" ;
                }
                $trend_arrow1   =   $conf{risingtrend} ;
            }
            else {
                $trend1         =   "-";
                $trend_arrow1   =   "-";
            }
        }
        $trend_arrow1 = ($trend_arrow1) ? $trend_arrow1 : "-";
        push @trend, $trend1 ;
        push @trend_arrow, $trend_arrow1 ;
    }

    my $time_diff2      =   $nowtime - $trends_store[0] ;
    my $time_diff_time  =   $nowtime - $trends_store_24[0] ;

    @trends_out = @trend ;
    push (@trends_out, @trend_arrow);
    push (@trends_out, ($today_condF{$trends_out[14]}, $today_condS{$trends_out[14]}) );

    # write after every half hour
    if ( $time_diff2 > 1800 ) {
        @full_fc_data1 = ( "@trends_store", "\n$nowtime @ft_now",  "\n@trends_now_24", "\n@trends_store_24" );
        open (out_data, '>', "$forecast_trends") or die "$LMsg{no_open_file} forecast 1800 : $!";
            print out_data "@full_fc_data1";
        close (out_data);
    }

    # write after every 24 hour
    # TODO: - check if this is valid or we need to increase time to 36 hours
    if ( $time_diff_time > 86400 ) {
        @full_fc_data2=( "@trends_now", "\n@trends_store", "\n@trends_store_24", "\n$nowtime @ft_now_24" );
        open (out_data, '>', "$forecast_trends") or die "$LMsg{no_open_file} forecast 86400 : $!";
            print out_data "@full_fc_data2";
        close (out_data);
    }
    if ($debug) {
        @full_fc_data1 = ( "now_data: ", "$nowtime @ft_now", "\nnow_data_stored: ", "@trends_store", "\nnow_24: ", "@trends_now_24", "\nnow_24_stored: ", "@trends_store_24\n" );
        do_debug( "get forecast trends", @full_fc_data1 );
        msgout( "$LMsg{pass_trends}" );
    }
}

sub present_cond_bl {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    my ( $test_cond, $pcond, $bl_text, $lt1, $rt1, $wx_warn_sym );

    if ( %langx ) {
        $test_cond = $langx{lc($TodayInfo{condition})} if exists $langx{lc($TodayInfo{condition})}
    }
    $test_cond ||= $TodayInfo{condition};

    if ( $conf{pcondition_blacklist} ) {
        foreach(split(/_,/, $conf{pcondition_blacklist})) {
            if (index($_,':') != -1) {
                ($lt1, $rt1) = split(/:/, $_);
                $test_cond =~ s/$lt1/$rt1/i;
            }
            else {
                $test_cond =~ s/$_//i;
                $test_cond =~ s/\s+/ /;
            }
        }
        $pcond = $test_cond
    }
    else { $pcond = $test_cond }


    if ( $wx{alertdata} ) {
        $wx{alertnum} = () = ($wx{alertdata} =~ /\|/g);
        $wx{alertnum} = $wx{alertnum} + 1;
        if ( $wx{alertnum} > 1 ) { $wx_warn_sym = "!" . "$wx{alertnum} " }
        else { $wx_warn_sym = "! " }
    }
    else { $wx_warn_sym = "" }

    if ($conf{pc_what_case}) { $pcond = do_pc_what_case( $pcond ) }

    return ($wx_warn_sym . $pcond );
}

sub do_pc_what_case {
    my ( $val ) = @_;
    for ($conf{pc_what_case}) {
        if    ( /1/ ) { $val =~ s/(\w+)/\u\L$1/g }
        # [1] All UPPERCASE
        elsif ( /2/ ) { $val = uc($val) }
        # [2] all lowercase
        elsif ( /3/ ) { $val = lc($val) }
        # [3] Capitilize the first letter/first word only, in the present condition
        elsif ( /4/ ) { $val = lc($val); $val = ucfirst($val) }
    }
    return $val
}

# -----------------------------------------------------------------------
# process pre_showtime_template template_cleanup and or language_to_use
# -----------------------------------------------------------------------
sub do_cleanup_lang {

    my ($do_lang, $intext) = @_ ;

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    $$intext ||= undef ;
    # if ( not $$intext ) { return '' }
    $do_lang ||= 0;
    my ( $lt1, $rt1, @cleanup );

    ### do_lang 1 is from alerts do_cl_opts
    if ( $do_lang == 1 ) {
        if ( %langx ) {
            my ($key1, $value1);
            while ( ($key1, $value1) = each %langx ) {
                $wx{alertdata} =~ s/\b$key1\b/$value1/ig;
            }
        }
        return
    }

    if ( $conf{template_cleanup} ) {
        my $lt1Regex = qr/\.\*(?![?])/;
        for (split('_,', $conf{template_cleanup})) {
            next if not $_ ;
            ($lt1, $rt1) = split(/:/, $_);
            if (index($lt1,'.*') != -1) {
                $lt1 =~ s/$lt1Regex/.*?/g
            }
            if ( $lt1 and $$intext and $$intext =~ /$lt1/i ) {
                push @cleanup, [ $lt1, $rt1 ]
            }
        }

        my $tmpTest = sub {
            my ($value) = @_;
            for (@cleanup) {
                my @k = $_;
                if ( $value =~ /$k[0][0]/i ) {
                    $value =~ s/$k[0][0]/$k[0][1]/i
                }
            }
            return $value
        };

        my $tcRegex = qr/(?<=\})([\w\.]{3}[\w\-\s\.]*)(?=\$\{|\#|:_:_:_:_:|\-)/;
        $$intext =~ s/\x0a/:_:_:_:_:/g;
        $$intext =~ s/$tcRegex/&$tmpTest($1)/eg;
        $$intext =~ s/:_:_:_:_:/\x0a/g;

        return $$intext if $do_lang == 2 ;
    }
    elsif ($do_lang == 2) { return $$intext }

    my $normalStr;
    if (index($$intext,'NORMALSTART') != -1 ) {
        ($normalStr) = ($$intext =~ m{NORMALSTART(.*?)NORMALEND}s);
    }

    # if ( ! $conf{do_not_output_to_conky} ) {
    if ( not $conf{do_not_output_to_conky} ) {
        open (out_data, '>', "$temp_template") or die "$LMsg{no_open_file} temp_template: $!";
            print out_data $$intext;
        close (out_data);
    }
    elsif ( $normalStr and ! $conf{link_tmp_folder} ) {
        open (out_data, '>', "$temp_templatesc") or die "$LMsg{no_open_file} temp_templateSC: $!";
            print out_data $normalStr;
        close (out_data);
    }

    my @wxdebug;
    if ($debug) {
        do_debug( "do_cleanup_lang dump", $$intext );
        msgout "***************************************";
    }
}

sub language_hash {

    if ($debug) { msgout( $LMsg{dbg_msg_str1}, sub_name() ); }

    if ( $conf{language_to_use} ) {
        my $lang_to_use = lc($conf{language_to_use});
        $lang_to_use =~ s/"|'//g;
        my $lang_file = "$cwxpath/lang/$lang_to_use.lang";

        if ( -e $lang_file ) {
            my ( $langRead  ) ;
            {
            open (in_data, "<", $lang_file) or die "$LMsg{no_open_file} language file: $!";
                local $/ = undef;
                $langRead = <in_data> ;
            close (in_data);
            }
            $langRead =~ s/#.*\n//g ;
            for (split(/\n/,$langRead)) {
                next if not $_ ;
                my ($key, $value) = split(/:/);
                $langx{lc($key)} = $value;
            }
        }
    }
}

sub do_cl_opts {

    my ($whatalerts);
    my $wxalert = "$temppath/wxalert" ;

    if ( $conf{show_wx_alerts} ) {
        if ( $wx{alertdata} ) {
            do_cleanup_lang( 1 );
            open (WXALERTFILE, '>', "$wxalert") or die "$LMsg{no_open_file} wx alert file $wxalert: $!";
                print WXALERTFILE $wx{alertdata};
            close (WXALERTFILE);
            if ($debug) {
                msgout( $LMsg{dbg_msg_str1}, sub_name() );
                msgout( $wx{alertdata} );
            }
        }
        else { $whatalerts = 1 }
    }
    else { $whatalerts = 1 }

    if ( $whatalerts and -e $wxalert ) {
        unlink $wxalert or warn "Could not delete wx alert file $wxalert: $!";
    }

    %output = (
        PTBI   => "$cwxpath/images/bgrounds",          PIF   => "$TodayInfo{NowFont}",
        PIC    => "$TodayInfo{NowImage}",              PT    => "$TodayInfo{Temp_now}",
        FL     => "$TodayInfo{app_temp}",              TD    => "$TodayInfo{today_date}",
        CT     => present_cond_bl,                     CLH1  => "$TodayInfo{CLH1}",
        CLC1   => "$TodayInfo{CLC1}",                  CLH2  => "$TodayInfo{CLH2}",
        CLC2   => "$TodayInfo{CLC2}",                  CLH3  => "$TodayInfo{CLH3}",
        CLC3   => "$TodayInfo{CLC3}",                  CLH4  => "$TodayInfo{CLH4}",
        CLC4   => "$TodayInfo{CLC4}",                  RF    => "$TodayInfo{Rainfall}",
        SD     => "$TodayInfo{SnowDepth}",             HM    => "$TodayInfo{Humidity}" . '%',
        VI     => "$TodayInfo{Visibility}",            WDD   => "$TodayInfo{Wind_Compass}",
        WD     => "$TodayInfo{Wind_From}",             WS    => "$TodayInfo{Wind_Speed}",
        WG     => "$TodayInfo{Wind_Gust}",             BF    => "$TodayInfo{wind_bearing_font}",
        BI     => "$TodayInfo{nowWindDir}",            LN    => "$TodayInfo{Location_Name}",
        SN     => "$TodayInfo{Station_Name}",          ID    => "$TodayInfo{Station_ID}",
        BR     => "$TodayInfo{Pressure}",              UV    => "$TodayInfo{uv_now}",
        UVM    => "$TodayInfo{uv_max}",                PLN   => "$TodayInfo{pollen}",
        DP     => "$TodayInfo{Dew_Point}",             METAR => "$TodayInfo{METAR}",
        SR     => "$Mag_2day{sun_rise}",               SS    => "$Mag_2day{sun_set}",
        LOV    => "$Mag_2day{length_visible_light}",   LOD   => "$Mag_2day{length_day}",
        LODN   => "$TodayInfo{next_day_length}",       LODNM => "$TodayInfo{next_day_length_msg}",
        LODNT  => "$TodayInfo{next_day_length_trend}", CTR   => "$Mag_2day{civ_twilt_R}",
        CTS    => "$Mag_2day{civ_twilt_S}",            NTR   => "$Mag_2day{nau_twilt_R}",
        NTS    => "$Mag_2day{nau_twilt_S}",            ATR   => "$Mag_2day{ast_twilt_R}",
        ATS    => "$Mag_2day{ast_twilt_S}",            SMT   => "$Mag_2day{sun_mer_trans}",
        MMT    => "$Mag_2day{moon_mer_trans}",         SDC   => "$Mag_2day{sun_dec}",
        MDC    => "$Mag_2day{moon_dec}",               MP1   => "$Mag_2day{moonp1}",
        MP2    => "$Mag_2day{moonp2}",                 MP3   => "$Mag_2day{moonp3}",
        MP     => "$Mag_2day{moonphase}",              MIF   => "$Mag_2day{moonfontchar}",
        MR     => "$Mag_2day{moon_rise}",              MS    => "$Mag_2day{moon_set}",
        NM     => "$Mag_2day{Next_New_Moon}",          FM    => "$Mag_2day{Full_Moon}",
        MFQ    => "$Mag_2day{First_Quarter}",          MLQ   => "$Mag_2day{Last_Quarter}",
        MA     => "$Mag_2day{moon_age}",               MV => "$Mag_2day{percentilluminated}"."%",
        MROC   => "$TodayInfo{moon_rate_of_change}",   MROT   =>  "$TodayInfo{moon_rotate_val}",
        MIC    => "$TodayInfo{moonImage}",             LU     => "$TodayInfo{last_update}",
        LF     => "$TodayInfo{fetchtime}",             D1D    => "$fctinfo{d_1_day}",
        D1T    => "$fctinfo{d_1_temp}",                D1C    => "$fctinfo{d_1_cond}",
        D1P    => "$fctinfo{d_1_cor}",                 D1F    => "$fctinfo{d_1_font}",
        D1I    => "$fctinfo{d_1_image}",               D2D    => "$fctinfo{d_2_day}",
        D2T    => "$fctinfo{d_2_temp}",                D2C    => "$fctinfo{d_2_cond}",
        D2P    => "$fctinfo{d_2_cor}",                 D2F    => "$fctinfo{d_2_font}",
        D2I    => "$fctinfo{d_2_image}",               D3D    => "$fctinfo{d_3_day}",
        D3T    => "$fctinfo{d_3_temp}",                D3C    => "$fctinfo{d_3_cond}",
        D3P    => "$fctinfo{d_3_cor}",                 D3F    => "$fctinfo{d_3_font}",
        D3I    => "$fctinfo{d_3_image}",               D4D    => "$fctinfo{d_4_day}",
        D4T    => "$fctinfo{d_4_temp}",                D4C    => "$fctinfo{d_4_cond}",
        D4P    => "$fctinfo{d_4_cor}",                 D4F    => "$fctinfo{d_4_font}",
        D4I    => "$fctinfo{d_4_image}",               D5D    => "$fctinfo{d_5_day}",
        D5T    => "$fctinfo{d_5_temp}",                D5C    => "$fctinfo{d_5_cond}",
        D5P    => "$fctinfo{d_5_cor}",                 D5F    => "$fctinfo{d_5_font}",
        D5I    => "$fctinfo{d_5_image}",               D6D    => "$fctinfo{d_6_day}",
        D6T    => "$fctinfo{d_6_temp}",                D6C    => "$fctinfo{d_6_cond}",
        D6P    => "$fctinfo{d_6_cor}",                 D6F    => "$fctinfo{d_6_font}",
        D6I    => "$fctinfo{d_6_image}",               D7D    => "$fctinfo{d_7_day}",
        D7T    => "$fctinfo{d_7_temp}",                D7C    => "$fctinfo{d_7_cond}",
        D7P    => "$fctinfo{d_7_cor}",                 D7F    => "$fctinfo{d_7_font}",
        D7I    => "$fctinfo{d_7_image}",               D8D    => "$fctinfo{d_8_day}",
        D8T    => "$fctinfo{d_8_temp}",                D8C    => "$fctinfo{d_8_cond}",
        D8P    => "$fctinfo{d_8_cor}",                 D8F    => "$fctinfo{d_8_font}",
        D8I    => "$fctinfo{d_8_image}",               D9D    => "$fctinfo{d_9_day}",
        D9T    => "$fctinfo{d_9_temp}",                D9C    => "$fctinfo{d_9_cond}",
        D9P    => "$fctinfo{d_9_cor}",                 D9F    => "$fctinfo{d_9_font}",
        D9I    => "$fctinfo{d_9_image}",               D10D   => "$fctinfo{d_10_day}",
        D10T   => "$fctinfo{d_10_temp}",               D10C   => "$fctinfo{d_10_cond}",
        D10P   => "$fctinfo{d_10_cor}",                D10F   => "$fctinfo{d_10_font}",
        D10I   => "$fctinfo{d_10_image}",
        FCTP   => "$trends_out[0]",
        FCTT   => "$trends_out[1]",
        FCTW   => "$trends_out[2]",
        FCTH   => "$trends_out[3]",
        FCTU   => "$trends_out[4]",
        FCTD   => "$trends_out[5]",
        FCTV   => "$trends_out[6]",
        FCTS   => "$trends_out[7]",
        FCTSR  => "$trends_out[8]",
        FCTSS  => "$trends_out[9]",
        FCTMR  => "$trends_out[10]",
        FCTMS  => "$trends_out[11]",
        FCTLL  => "$trends_out[12]",
        FCTLD  => "$trends_out[13]",
        FCTMI  => "$trends_out[14]",
        FCTTY  => "$trends_out[15]",
        FCTPF  => "$trends_out[16]",
        FCTTF  => "$trends_out[17]",
        FCTWF  => "$trends_out[18]",
        FCTHF  => "$trends_out[19]",
        FCTUF  => "$trends_out[20]",
        FCTDF  => "$trends_out[21]",
        FCTVF  => "$trends_out[22]",
        FCTSF  => "$trends_out[23]",
        FCTSRF => "$trends_out[24]",
        FCTSSF => "$trends_out[25]",
        FCTMRF => "$trends_out[26]",
        FCTMSF => "$trends_out[27]",
        FCTLLF => "$trends_out[28]",
        FCTLDF => "$trends_out[29]",
        FCTMIF => "$trends_out[30]",
        FCTTYF => "$trends_out[31]",
        FCTTYM => "$trends_out[32]",
        FCTTYS => "$trends_out[33]",
        GT     => "$wxtrend{temp}",
        GP     => "$wxtrend{press}",
        GD     => "$wxtrend{dp}",
        GH     => "$wxtrend{hum}",
        GW     => "$wxtrend{ws}",
        GG     => "$wxtrend{wsg}",
        UU     => "$lua_wx_units",
        TMP    => "$temppath",
        SOL    => "$TodayInfo{solarRad}",
        GS     => "$wxtrend{sol}",
    );

    chomp($clopt1);
    if ( $clopt1 eq "-t" ) {
        if ( -e "$input_template" ) {
            my ( $tmp_copy  ) ;
            {
            open (in_data, "<", $input_template) or die "$LMsg{no_open_file} template: $!";
                local $/ = undef;
                $tmp_copy = <in_data> ;
            close (in_data);
            }
            $tmp_copy =~ s/(?<=\n)#.*\n//g;

            ## convert element to wx value
            my ($nd1, $element, $offset, $st1);
            $offset = 0;
            $st1 = index( $tmp_copy, '[' ) ;
            while ($st1 != -1) {
                $offset = $st1 + 1;
                $nd1 = index($tmp_copy, ']', $offset);
                $element = substr($tmp_copy, $st1+1, $nd1 - $st1-1);
                substr($tmp_copy, $st1, $nd1 - $st1+1) = ( %langx ) ? &translate($element) : ( exists $output{$element}) ? $output{$element} : 'Error' ;
                ## look for next occurrence
                $st1 = index($tmp_copy, '[', $offset);
            }

            &do_cleanup_lang(3,\$tmp_copy);

            my ( @scroller );
            if ( ! $conf{do_not_output_to_conky} ) {
                print $tmp_copy;
            }
            else {
                my $normalStr;
                ($normalStr) = ($tmp_copy =~ m{NORMALSTART(.*?)NORMALEND}s);
                $tmp_copy =~ s/NORMALSTART.*NORMALEND//s;
                print $normalStr if $normalStr;

                my $conkywx_scroller = "$temppath/wxconkywx_scroller";
                open (out_data, '>', "$conkywx_scroller") or die "$LMsg{no_open_file} conkywx_scroller: $!";
                    $tmp_copy =~ s/#.*//;
                    $tmp_copy =~ s/^\s*\n//g;
                    $tmp_copy =~ s/^/@@|@@|@@|\n/g;
                    $tmp_copy =~ s/\n\s*\n//g;
                    print out_data $tmp_copy;
                close (out_data);
            }
        }
    }
    elsif ( $input_element ) {
        $input_element = uc($input_element);
        print &translate( $input_element )."\n";
    }
    my @wxdebug;
    # $debug = 1;
    if ( $debug or $ENV{debug} ) {
        foreach my $wxkey ( sort keys %output ) {
            my $out1 = &translate($wxkey);
            push @wxdebug, "$wxkey = $out1\n";
        }
        msgout "$LMsg{pass_wxel}" ;
        do_debug( "do_cli_opts dump", @wxdebug );
        msgout "******************* End of conkywx debug log ********************";
    }
}

sub translate {
    if ( $output{$_[0]} ) {
        if ( $output{$_[0]} eq 'NA' )   {
            return 'NA'
        }
        elsif ( exists $naKeys{$_[0]} ) {
            return $output{$_[0]}
        }
        else {
            $_[1] = $output{$_[0]};
            if ($_[0] eq 'UV') {
                my @val1 = split( / : /, $_[1]);
                if ( ${val1[1]} and $langx{lc(${val1[1]})} ) {
                    return( "$val1[0] : $langx{lc(${val1[1]})}" )
                }
                else {
                    return( $_[1])
                }
            }
            elsif ( $gkey{$_[0]} ) {
                my @val1 = split( / /, $_[1]);
                if (exists $langx{lc(${val1[0]})}) {
                    return( "$langx{lc(${val1[0]})} $val1[1]" )
                }
                else {
                    return($_[1])
                }
            }
            else {
                if ( $_[1] and $langx{lc $_[1]}) {
                    return( $langx{lc $_[1]} )
                }
                else {
                    $_[1] =~   s/
                                    ([a-zA-Z]{2}[a-zA-Z]*)
                                /
                                    ( exists $langx{lc $1 } ) ? $langx{lc $1} : $1
                                /gexi;
                    return( $_[1])
                }
            }
        }
    }
    ## if it gets here it is an error
    return "Error"
}

sub run_conkywx {

    my $dev = $ENV{"wxdevelopment"} // 0;
    # msgout($dev);
    &parse_html;
    if ( -e "$currdatax" ) {
        if ( ! -z "$currdatax" ) {
            &parse_xml;
            &validate_xml if ( not $conf{link_tmp_folder} and not $dev )
        }
        else { &msgout( "\n\nError: :( \nxml data file is zero size.\n") }
    }

    if ( not $dev) {
        if ( $wx{xml_state} and $wx{xml_state} == 1 ) { &validate_xml }
    }
    if ( $wx{dataisthere} ) {
        &parse_fct;
        &get_apptemp;
        &get_bearing_font;
        &get_magellan;
        &get_uvdata;
        &trend_graphs;
        &get_forecast_trends;
        &language_hash if $conf{language_to_use};
        &do_cl_opts;
        &seismic if $conf{get_seismic} ;
        &get_tides if $conf{tides_url};
        &rssd if $conf{newsUrl} or $conf{gdacsUrl} ;
    }
    else {
        &msgout( "$LMsg{nodata}" );
        unlink $temp_template;
        if ($debug) {
            &msgout( "$LMsg{nodatadbg}" );
            &do_debug( "No available" );
        }
    }

}

&run_conkywx ;

#-------------------------------------------------------------------------------
# End of file
#-------------------------------------------------------------------------------

=comments

=cut
