/*
//      Magellan
//
//      Copyright 2012-2016 Paramvir Likhari <paramvir at gmx dot com>
//
//		Some Sections (c) Paul Schlyter, 1989, 1992
//         http://www.stjarnhimlen.se/comp/tutorial.html
//		Some Sections (c) Douglas Bacon, 2008
//
//		This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//
*/


#include <stdio.h>
#include <locale.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include "mphase.h"
#include "conkywx.h"
#include "metar_structs.h"
//#include "local_defs.h"


#define LAT_SET 0x1
#define LON_SET 0x2

int DcdMETAR( char *, Decoded_METAR * );
void prtDMETR( Decoded_METAR *);


//  from test_cases.c
//lint -esym(843, string)  variable could be declared as const
extern char *string[] ;


/*
 | Month names.  These will be overwritten (ie. the pointers, not the static data)
 | with names taken from the Messages file if one exists.
 */
// static char *moshort[12] = { "Jan", "Feb", "Mar", "Apr",
// 							"May", "Jun", "Jul", "Aug", "Sep",
// 							"Oct", "Nov", "Dec"};

// Moon font arrays for getting the moon font for conkywx
// Northern Hemisphere
char *moon_font1[31] = { "a", "b", "c", "c", "d", "e", "f",
						 "g", "h", "i", "j", "k", "l", "m",
						"n", "o", "p", "q", "r", "s", "t",
						"u", "v", "w", "x", "x", "y", "z", "0", "1", "-" };

char *moon_font2[31] = { "N", "O", "P", "P", "Q", "R", "S",
						"T", "U", "V", "W", "X", "Y", "Z",
						"A", "B", "C", "D", "E", "F", "G",
						"H", "I", "J", "K", "K", "L", "M", "0", "1", "-" };
// Southern Hemisphere
// char *moon_font3[31] = { "z", "y", "x", "x", "w", "v", "u",
						// "t", "s", "r", "q", "p", "o", "n",
						// "m", "l", "k", "j", "i", "h", "g",
						// "f", "e", "d", "c", "c", "b", "a", "0", "1", "-" };
//
// char *moon_font4[31] = { "M", "L", "K", "K", "J", "I", "H",
						// "G", "F", "E", "D", "C", "B", "A",
						// "Z", "Y", "X", "W", "V", "U", "T",
						// "S", "R", "Q", "P", "P", "O", "N", "0", "1", "-" };

uint use_fahrenheit = FALSE ;

//********************************************************************************************
static void process_metar_string(char *metar_str)
{
   Decoded_METAR Metar;

   /*-- PRINT INPUT METAR REPORT ----------------------------*/
   //puts("") ;
   if ( use_fahrenheit == FALSE ) {
	   puts("\n#*****************************************************") ;
	   puts("#  INPUT METAR REPORT:") ;
	   printf("#  %s", metar_str );
   }

   /*-- DECODE INPUT REPORT ---------------------------------*/
   int ErReturn = DcdMETAR(metar_str, &Metar) ;
   if (ErReturn != 0 ) {
      printf("DcdMETAR:  Error Return Number: %d\n",ErReturn);
      return ;
   }
   /*-- PRINT DECODED METAR REPORT ELEMENTS -----------------*/
   puts("") ;
   prtDMETR(&Metar);
   //@@@  included test code that was here, has been moved to synoptic.cxx
}

static void usage(void)
{
  puts(
  "magellan calculates Sun / Moon data and METAR translation.\n\n"
  "Usage:: magellan -y [year] -m [month] -d [day] [latitude]NS [longitude]EW -t [time zone] -o [moon-outline] -a time_format -b moon_time_format\n\n"
  "The latitude/longitude are expressed in floating-point degrees, NS/EW \n"
  "example: magellan -y 2016 -m 12 -d 18 28.6N 77.2E -t 5.5\n"
  "-t set to 0 - will give UTC values.\n"
  "-a set time format for all except moon phases.\n"
  "-b set time format for moon phases.\n\n"
  "Usage for METAR:: magellan [--metar][--metarwx] \"metar text\" \n"
  "For test cases :: magellan --metart \n\n"
  "[--metar]   = as designed by various authors\n"
  "[--metarwx] = units customized for conkywx :-))\n"
  "[--metart]  = test cases output\n\n"
  "magellan (c) 2012-2016 Paramvir Likhari\n"
  "Version 1.5.0 \n");
}


int main (int argc, char *argv[])
{
	//int year, month, day, hour, day2;
	int year, month, day;
	//int i, j;				// time zone
	int i;				// time zone
	double underlong=77.2, ourlong, ourlat=28.6, tz=5.5, zone, JD, JD2;
	//modified julian date, days since 1858 Nov 17 00:00, 1858 Nov 17 00:00 is JD 2400000.5
	double date, date2, tins_now;
	// double mydec, moon_state;
	double moon_state;
	double UT;
	double moonrise, moontransit, moonset, sunrise, suntransit, sunset;
	int object = 0;         //0 IS MOON, 1 IS SUN
	double	juld, juld1, p, aom, illum, cdist, cangdia, csund, csuang, lptime;
	int	yy, mm, dd, hh, mmm, ss;
	int nt1 = 0;
	int coords_set = 0;
	time_t	t;
	double temp;
	int local = 1;
	int moon_outline, moonImage;
	char hemisphere, met_str[200];
	met_str[0] = 0;
	struct tm *gm, tmv1, tmv2;
	// struct tm tmv2;
	static double phases[5], nm_phases[5];
	char mphase[30];
	int illumcase, mfval;
	char moonfontchar[2];

   uint use_test_vectors = FALSE ;
   //  parse command line
   uint found_metar_arg = FALSE ;


	// setlocale( LC_ALL, "" );
	setlocale( LC_TIME, "" );
	// size_t
    // i18n_time_n_data (char *s, size_t len, const struct tm *tp)
    // {
      // return strftime (s, len, nl_langinfo (D_T_FMT), tp);
    // }
	// char buffer[SIZE];
	// char tformat[SIZE], moon_format[SIZE];
//******************************************************************************

//******************************************************************************
//get date and position from user.

	for (i=1; i< argc; i++) {
		if (!strcmp("-h", argv[i])) {
			usage();
			exit(0);
		}
		if (!strcmp("-y", argv[i])) {
			i++;
			year = atoi(argv[i]);
		}
		if (!strcmp("-m", argv[i])) {
			i++;
			month = atoi(argv[i]);
		}
		if (!strcmp("-d", argv[i])) {
			i++;
			day = atoi(argv[i]);
		}
		if (!strcmp("--metar", argv[i])) {
			i++;
			if ( argv[i] != 0 ) { strcpy(met_str, argv[i]); }
			else { break; }
		}
		if (!strcmp("--metarwx", argv[i])) {
			i++;
			if ( argv[i] != 0 ) {
				strcpy(met_str, argv[i]);
				use_fahrenheit = TRUE ;
			} else { break; }
		}
		if (!strcmp("--metart", argv[i])) {
			i++;
			strcpy(met_str, " t");
			use_fahrenheit = TRUE ;
			use_test_vectors = TRUE ;
			break;
		}
		if (!strcmp("-t", argv[i])) {
			i++;
			tz = atof(argv[i]);
		}
		if (!strcmp("-o", argv[i])) {
			i++;
			moon_outline = atoi(argv[i]);
		}
		if (!strcmp("-n", argv[i])) { //next day
			i++;
			nt1 = atoi(argv[i]);
		}
		if (!strcmp("-a", argv[i])) {
			i++;
			remove_chars(argv[i], '+');
			strcpy(tformat, argv[i]);
		}
		if (!strcmp("-b", argv[i])) {
			i++;
			remove_chars(argv[i], '+');
			strcpy(moon_format, argv[i]);
		}
		if (2 == sscanf(argv[i], "%lf%1[Nn]", &temp, &hemisphere)) {
			ourlat = temp;
			coords_set |= LAT_SET;
		}
		if (2 == sscanf(argv[i], "%lf%1[Ss]", &temp, &hemisphere)) {
			ourlat = -temp;
			coords_set |= LAT_SET;
		}
		if (2 == sscanf(argv[i], "%lf%1[Ww]", &temp, &hemisphere)) {
			underlong = -temp;
			coords_set |= LON_SET;
		}
		/* handle c confused scientific notation */
		if (1 == sscanf(argv[i], "%lf", &temp, &hemisphere)
			&& (argv[i][strlen(argv[i])-1] == 'E' ||
			argv[i][strlen(argv[i])-1] == 'e')) {
			underlong = temp;
			coords_set |= LON_SET;
		}
	}

	// if ( tformat == '\0' ) { strcpy(tformat, "%I:%M %P") ; }
	// if ( moon_format == '\0' ) { strcpy(moon_format, "%b %d") ; }

	if (met_str[0] != 0) {
		//printf( "\n\nwe are here\n\n");
		//process_metar_string(met_str) ;
		//found_metar_arg = TRUE ;

		if (use_test_vectors) {
			int idx;
			for (idx=0; string[idx] != NULL; idx++) {
				process_metar_string(string[idx]) ;
			}
		} else {
			process_metar_string(met_str) ;
			found_metar_arg = TRUE ;
		}
	} else if ( coords_set != (LAT_SET | LON_SET)) {
		usage();
	} else {

		printf ("\n# conkywx magellan data for %2.2d/%2.2d/%4d %3.6f %3.6f %2.1f\n", day, month, year, ourlat, underlong, tz);

	//init some values
	    UT = 0.0;
		ourlong = 0.0 - underlong;  //equations use east longitude negative convention
		zone = tz / 24.0;

	//get rise set times for moon and sun
		JD2 = get_julian_date (year, month, (day+1), UT);
		date2 = (JD2 - 2400000.5 - zone);
		printf ("\n[next_day_start]");
		object = 1;	//sun
		get_rst ( object, date2, ourlong, ourlat, year, month, day, &sunrise, &sunset, &suntransit);
		//object = 1;	//sun
		(void) time(&t);
		juld = (jtime(gm = gmtime(&t)));
		p = phase(juld+1, &illum, &aom, &cdist, &cangdia, &csund, &csuang);
		printf( "\npercentilluminated=\"%1.0f\"", trim_val(illum) );
		printf ("\n[next_day_end]");

		JD = get_julian_date (year, month, day, UT);
		date = (JD - 2400000.5 - zone);
		object = 1;	//sun
		get_rst ( object, date, ourlong, ourlat, year, month, day, &sunrise, &sunset, &suntransit);

		object = 0;	//moon
		get_rst ( object, date, ourlong, ourlat, year, month, day, &moonrise, &moonset, &moontransit);
		// calculate moon declination
		//ins_now = date;
		//ins_now = juld;
		//ins_now = date + hour / 24 ;
		tins_now = (juld - 51544.5) / 36525;
		// logic for getting lat dec value
		// Declination n Latitute North is +ve South is -ve
		// if Obs North of Moon Dec
			// Obs is North Moon is North - Obs Lat N gthan Moon Dec N -  33 >  10
			// Obs is North Moon is South - Obs Lat N Gthan Moon Dec S -  33 > -10
			// Obs is South Moon is South - Obs Lat S gthan Moon Dec S - -10 > -15
			// for obs N of moon - obs lat always gt moon dec => mydec = 1
		// if Obs is South of Moon Dec
			// Obs is North Moon is North - Obs Lat N Lthan Moon Dec N -  10 <  15
			// Obs is South Moon is North - Obs Lat S Lthan Moon Dec N - -10 <  15
			// Obs is South Moon is South - Obs Lat S Lthan Moon Dec N - -33 < -10
			// else mydec = 0
		// mylat1 = -20;
			//jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			//tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);

		// this is set up so that correct values can be taken ro ra and
		// declination of the sun - moon still need to be figured out.
		// Sun data tallies with NOAA spreadsheet.
		// US Navy mil site is not working to verify
		///////////////////////////////////////////////////////////////
		///////////////// sun moon azimuth and declination
		double ourlong2,date9;
		// double hr1, mn1, ourlong2,date9;
		ourlong2 = ourlong * -1 ;

		printf( "\nobs_lat=\"%3.9f\"", ourlat );
		printf( "\nobs_lon=\"%3.9f\"", ourlong2 );

		// printf( "\njuld=\"%3.9f\"", juld );
		/*  NOTE for Sun n Moon Day Night positions
			We take lat on equator and lon on Greenwich so lat and lon
			are both 0. Time is taken from gmtime and includes hours etc.
			date9 takes the julian calender from jtime
		*/
		time_t curtime;
		struct tm *loc_time;
		curtime = time( NULL );
		loc_time = gmtime(&curtime);
		date9 = jtime(loc_time);
		// print time at UTC at long 0E
		// printf( "\ngm Time=\"%2i:%2i:%2i\"", loc_time->tm_hour,loc_time->tm_min,loc_time->tm_sec );
		date9 = (date9 - 2400000.5);

		azalt(1, date9, 0, ourlat, ourlong2 );
		azalt(0, date9, 0, ourlat, ourlong2 );


		print_everything(year, month, day, ourlat, ourlong, tz, local);


		///////////////////////////////////////////////////////////////////

	//get moon phase
		// use system to get julian date - for production //////////////

		//(void) time(&t);
		//juld = (jtime(gm = gmtime(&t)));

		// use user date to get julian date - for testing //////////////
		// tmv2 = mk_tm(year, month, day, 00, 00, 00);
		// juld = (jtime(&tmv2));

		//if ( nt1 == 1 ) { juld += 0.9868055556; }
		//if ( nt1 == 1 ) { juld += 1; }
		p = phase(juld, &illum, &aom, &cdist, &cangdia, &csund, &csuang);
		//printf( "\nMMM percentilluminated=\"%1.0f\"\n", trim_val(illum) );

		agetodate(aom, &dd, &hh, &mm);
		printf( "\nmoon_age=\"%2.2dd %2.2dh %2.2dm\"\n", dd, hh, mm );

		 // printf( "\nmoon_age_raw=\"%1.9f\"\n", aom );
		 // printf( "moon_dist=\"%3.9f\"\n", cdist  );
		 // printf( "moon_angle=\"%f\"\n", cangdia  );
		 // printf( "sun_dist=\"%3.9f\"\n", csund  );
		 // printf( "sun_angle=\"%f\"\n", csuang  );
		 // printf( "\njulian date=\"%f\"\n", juld  );

		// get moon phases in local time - zone is added for that in jyear and jhms functions
		phasehunt( juld+0.5, zone, phases );

		// get next lunar cycle phases
		// add 1 lunar [synodic] month 29d 12h 44m 03s [2.86s actually]
		day += 29;
		tmv1 = mk_tm(year, month, day, 12, 44, 03);
		juld1 = (jtime(&tmv1));
		// get moon phases in local time - zone is added for that in jhms functions
		phasehunt( juld1+0.5, zone, nm_phases );

		// print the moon phases in local time
		lptime = phases[0];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Last_New_Moon=\"%s\"\n", buffer );
			// printf( "Last_New_Moon=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		if (juld <= phases[1])
		{
			lptime = phases[1];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("First_Quarter=\"%s\"\n", buffer );
			// printf( "First_Quarter=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		} else {
			lptime = nm_phases[1];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("First_Quarter=\"%s\"\n", buffer );
			// printf( "First_Quarter=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		}
		if (juld <= phases[2])
		{
			lptime = phases[2];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Full_Moon=\"%s\"\n", buffer );
			// printf( "Full_Moon=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		} else {
			lptime = nm_phases[2];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Full_Moon=\"%s\"\n", buffer );
			// printf( "Full_Moon=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		}
		if (juld <= phases[3])
		{
			lptime = phases[3];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Last_Quarter=\"%s\"\n", buffer );
			// printf( "Last_Quarter=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		} else {
			lptime = nm_phases[3];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Last_Quarter=\"%s\"\n", buffer );
			// printf( "Last_Quarter=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		}
		if (juld <= phases[4])
		{
			lptime = phases[4];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Next_New_Moon=\"%s\"\n", buffer );
			// printf( "Next_New_Moon=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		} else {
			lptime = nm_phases[4];
			jhms(lptime, zone, &yy, &mm, &dd, &hh, &mmm, &ss);
			tmv2 = mk_tm(yy, mm, dd, hh, mmm, ss);
			strftime (buffer, SIZE, moon_format, &tmv2);
			printf("Next_New_Moon=\"%s\"\n", buffer );
			// printf( "Next_New_Moon=\"%2.2d %s %2d %2.2d:%2.2d\"\n", dd, moshort[mm - 1], yy, hh, mmm );
		}

		// printf( "julian_date=\"%f\"\n", juld  );
		// printf( "phases[0]=\"%f\"\n", phases[0]  ); // 0
		// printf( "phases[1]=\"%f\"\n", phases[1]  ); // 50
		// printf( "phases[2]=\"%f\"\n", phases[2]  ); // 100
		// printf( "phases[3]=\"%f\"\n", phases[3]  ); // 50
		// printf( "phases[4]=\"%f\"\n", phases[4]  ); // 0

	//******************************************************************************
		// moon never goes 100% but .99999999 etc
		illumcase = trim_val(illum);
		//printf( "\nMMM percentilluminated=\"%1d\"\n", illumcase );
		// illumcase = 1;

		printf( "percentilluminated=\"%1d\"\n", illumcase );
	//******************************************************************************
		if (juld <= phases[0])
		{
			if      ( illumcase <= 1 )
				{ strcpy(mphase, "New Moon");        }
		}
		else if ((juld >= phases[0]) && (juld <= phases[1]))
		{
			if      ( illumcase <= 1 )
				{ strcpy(mphase, "New Moon");        }
			else if (( illumcase > 1  ) && ( illumcase < 48 ))
				{ strcpy(mphase, "Waxing Crescent"); }
			else if ( illumcase >= 48 )
				{ strcpy(mphase, "First Quarter");   }
		}
		else if ((juld >= phases[1]) && (juld <= phases[2]))
		{
			if      ( illumcase <= 52 )
				{ strcpy(mphase, "First Quarter");   }
			else if (( illumcase > 52 ) && ( illumcase < 98 ))
				{ strcpy(mphase, "Waxing Gibbous");  }
			else if ( illumcase >= 98 )
				{ strcpy(mphase, "Full Moon");       }
		}
		else if ((juld >= phases[2]) && (juld <= phases[3]))
		{
			if      ( illumcase <= 52 )
				{ strcpy(mphase, "Last Quarter");    }
			else if (( illumcase > 52 ) && ( illumcase < 98 ))
				{ strcpy(mphase, "Waning Gibbous");  }
			else if ( illumcase >= 98 )
				{ strcpy(mphase, "Full Moon");       }
		}
		else if ((juld >= phases[3]) && (juld <= phases[4]))
		{
			if      ( illumcase <= 1 )
				{ strcpy(mphase, "New Moon");        }
			else if (( illumcase > 1 ) && ( illumcase < 48 ))
				{ strcpy(mphase, "Waning Crescent"); }
			else if ( illumcase >= 48 )
				{ strcpy(mphase, "Last Quarter");    }
		}

		printf( "moonphase=\"%s\"\n", mphase );
		moonWords( mphase );
		// moonWords( "Full Moon" );

		// strcpy(mphase, "Waning Crescent");
		// illumcase = 60;
		// mydec = 1 - Northern Hemisphere
		// if ( mydec == 1 ) {
		// 	if ((!strcmp(mphase, "Waxing Crescent" )) || (!strcmp(mphase,"Waxing Gibbous")) || (!strcmp(mphase,"First Quarter")))
		// 		{ moon_state = 1; }
		// 	else if ((!strcmp(mphase, "Waning Crescent" )) || (!strcmp(mphase,"Waning Gibbous")) || (!strcmp(mphase,"Last Quarter")))
		// 		{ moon_state = 0; }
		// } else {
			if ((!strcmp(mphase, "Waxing Crescent" )) || (!strcmp(mphase,"Waxing Gibbous")) || (!strcmp(mphase,"First Quarter")))
				{ moon_state = 0; }
			else if ((!strcmp(mphase, "Waning Crescent" )) || (!strcmp(mphase,"Waning Gibbous")) || (!strcmp(mphase,"Last Quarter")))
				{ moon_state = 1; }
		// }
		// get the moonImage and font character
		switch (illumcase) {
			case 0:
				moonImage = 30;
				mfval = 29;
				break;
			case 100:
				moonImage = 15;
				mfval = 28;
				break;
			default:
				if ((!strcmp(mphase, "New Moon" ))) {
					moonImage = 30;
					mfval = 29;
				}
				else if ((!strcmp(mphase, "Full Moon" ))) {
					moonImage = 15;
					mfval = 28;
				}
				// else if ((!strcmp(mphase, "Waxing Crescent" )) || (!strcmp(mphase,"Waxing Gibbous")) || (!strcmp(mphase,"First Quarter"))) {
				else if ( moon_state == 1 ) {
					switch (illumcase) {
						case 1 ... 7:
							moonImage = 1;
							mfval = 0;
							break;
						case 8 ... 14:
							moonImage = 2;
							mfval = 1;
							break;
						case 15 ... 21:
							moonImage = 3;
							mfval = 2;
							break;
						case 22 ... 28:
							moonImage = 4;
							mfval = 3;
							break;
						case 29 ... 35:
							moonImage = 5;
							mfval = 4;
							break;
						case 36 ... 42:
							moonImage = 6;
							mfval = 5;
							break;
						case 43 ... 49:
							moonImage = 7;
							mfval = 6;
							break;
						case 50 ... 56:
							moonImage = 8;
							mfval = 7;
							break;
						case 57 ... 63:
							moonImage = 9;
							mfval = 8;
							break;
						case 64 ... 70:
							moonImage = 10;
							mfval = 9;
							break;
						case 71 ... 77:
							moonImage = 11;
							mfval = 10;
							break;
						case 78 ... 84:
							moonImage = 12;
							mfval = 11;
							break;
						case 85 ... 91:
							moonImage = 13;
							mfval = 12;
							break;
						case 92 ... 99:
							moonImage = 14;
							mfval = 13;
							break;
					}
				}
				// else if ((!strcmp(mphase, "Waning Crescent" )) || (!strcmp(mphase,"Waning Gibbous")) || (!strcmp(mphase,"Last Quarter"))) {
				else if ( moon_state == 0 ) {
					switch (illumcase) 	{
						case 92 ... 99:
							moonImage = 16;
							mfval = 14;
							break;
						case 85 ... 91:
							moonImage = 17;
							mfval = 15;
							break;
						case 78 ... 84:
							moonImage = 18;
							mfval = 16;
							break;
						case 71 ... 77:
							moonImage = 19;
							mfval = 17;
							break;
						case 64 ... 70:
							moonImage = 20;
							mfval = 18;
							break;
						case 57 ... 63:
							moonImage = 21;
							mfval = 19;
							break;
						case 50 ... 56:
							moonImage = 22;
							mfval = 20;
							break;
						case 43 ... 49:
							moonImage = 23;
							mfval = 21;
							break;
						case 36 ... 42:
							moonImage = 24;
							mfval = 22;
							break;
						case 29 ... 35:
							moonImage = 25;
							mfval = 23;
							break;
						case 22 ... 28:
							moonImage = 26;
							mfval = 24;
							break;
						case 15 ... 21:
							moonImage = 27;
							mfval = 25;
							break;
						case 8 ... 14:
							moonImage = 28;
							mfval = 26;
							break;
						case 1 ... 7:
							moonImage = 29;
							mfval = 27;
							break;
					}
				}
				else {
					moonImage = 99;
					mfval = 30;
				}
			}

			printf( "moonImage=\"%2.2d\"\n", moonImage );

			// mydec = 1 - Northern Hemisphere
			// if ( mydec == 1 ) {
				if ( moon_outline == 0 ) {
					strcpy(moonfontchar, moon_font1[mfval]);
				} else {
					strcpy(moonfontchar, moon_font2[mfval]);
				}
			// } else {
				// if ( moon_outline == 0 ) {
					// strcpy(moonfontchar, moon_font3[mfval]);
				// } else {
					// strcpy(moonfontchar, moon_font4[mfval]);
				// }
			// }
			printf( "moonfontchar=\"%s\"\n", moonfontchar ) ;


		//tmv1 = mk_tm(2013, 01, 19, 23, 41, 00);
		//juld1 = (jtime(&tmv1));
		//printf( "new_julian_date=\"%1.9f\"\n", juld1 ) ;

	}
	//OK - thats it, we are done
	printf("\n");
return 0;
}

